//
//  ZegoAudioPlayer.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/12.
//

#import "ZegoAudioPlayer.h"

@interface ZegoAudioPlayer () <ZegoAudioEffectPlayerEventHandler>

@property (nonatomic, strong) ZegoAudioEffectPlayer *audioEffectPlayer;
@property (nonatomic, strong) NSMutableDictionary *nameSoundIDMap;
@property (nonatomic, strong) NSMutableDictionary <NSString *, NSString *> *namePathMap;
@property (nonatomic, copy) NSString *filePath;
@property (nonatomic, assign) int audioEffectID;
@end


@implementation ZegoAudioPlayer

- (instancetype)init {
    self = [super init];
    if(self){
        [self initAudioPlayer];
        _audioEffectID = 1;
    }
    return self;
}

- (void)initAudioPlayer {
    self.audioEffectPlayer = [[ZegoExpressEngine sharedEngine] createAudioEffectPlayer];
    if (!self.audioEffectPlayer) {
        kLog(@"创建音效播放器失败");
    }
    [self.audioEffectPlayer setEventHandler:self];
//    self.audioEffectPlayer = audioEffectPlayer;
}

- (void)playSound:(NSString *)name isPlay:(BOOL)isPlay url:(NSString *)url isLoop:(BOOL)isLoop volume:(int)volume {
   NSString *libraryPath = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES).lastObject;
    NSString *savedPath = [[self createAudioDirectories:libraryPath] stringByAppendingPathComponent:[NSString stringWithFormat:@"/%@.mp3", name]];
    NSString *path = self.namePathMap[url];
    int soundID = [self.nameSoundIDMap[name] intValue];
    if (ZGStringIsEmpty(path)) {
        [self downloadFileWithURL:url parameters:nil savedPath:savedPath downloadSuccess:^(NSURLResponse *response, NSURL *filePath) {
            NSString *path = filePath.path;
            [self.namePathMap setObject:path forKey:url];
            if (isPlay) {
                if (soundID == 0) {
                    int newSoundID = self.audioEffectID ++;
                    [self.nameSoundIDMap setObject:@(newSoundID) forKey:name];
                    [self startSound:newSoundID path:path isLoop:isLoop volume:volume];
                } else {
                    [self startSound:soundID path:path isLoop:isLoop volume:volume];
                }
            } else {
                   if (soundID != 0) {
                       [self pause:(int)soundID];
                   }
            }
            kLog(@"存储的路径：%@", path);
        } downloadFailure:^(NSError *error) {

        } downloadProgress:^(NSProgress *downloadProgress) {
            kLog(@"总大小：%lld, 当前大小:%lld",downloadProgress.totalUnitCount,downloadProgress.completedUnitCount);
        }];
    } else {
        int soundID = [self.nameSoundIDMap[name] intValue];
        if (isPlay) {
            if (soundID == 0) {
                int newSoundID = self.audioEffectID ++;
                [self.audioEffectPlayer loadResource:path audioEffectID:newSoundID callback:^(int errorCode) {
                    [self.nameSoundIDMap setObject:@(newSoundID) forKey:name];
                    [self startSound:soundID path:path isLoop:isLoop volume:volume];
                }];
               
            } else {
                [self startSound:soundID path:path isLoop:isLoop volume:volume];
            }
        } else {
            if (soundID != 0) {
               [self pause:soundID];
           }
        }
    }
//    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"effect_1_stereo" ofType:@"wav"];
//    self.filePath = filePath;
//    ZegoAudioEffectPlayConfig *config = [[ZegoAudioEffectPlayConfig alloc] init];
//    config.playCount = 0; // 播放一次
//    config.isPublishOut = YES; // 播放混入推流中
//    [self.audioEffectPlayer loadResource:filePath audioEffectID:1 callback:^(int errorCode) {
//
//    }];
//    [self.audioEffectPlayer setVolume:volume audioEffectID:1];
//    [self.audioEffectPlayer start:1 path:filePath config:config];
}

- (void)startSound:(int)soundEffectID path:(NSString *)path isLoop:(BOOL)isLoop volume:(int)volume {
    ZegoAudioEffectPlayConfig *config = [[ZegoAudioEffectPlayConfig alloc] init];
    config.playCount = isLoop ? 0 : 1; // 循环播放 或者 播放一次
//                    config.isPublishOut = YES; // 播放混入推流中
    [self.audioEffectPlayer setVolume:volume audioEffectID:(int)soundEffectID];
    [self.audioEffectPlayer start:(int)soundEffectID path:path config:config];
}

- (void)changeSoundVolume:(NSString *)name volume:(int)volume {
    if (self.audioEffectPlayer == nil) {
        return;
    }
    int streamID = [self.nameSoundIDMap[name] intValue];
    if (streamID != 0) {
        [self.audioEffectPlayer setVolume:volume audioEffectID:streamID];
    } else {
        
    }
}

- (void)destroy {
    if (self.audioEffectPlayer){
        [ZegoExpressEngine.sharedEngine destroyAudioEffectPlayer:self.audioEffectPlayer];
        self.audioEffectPlayer = nil;
    }
}

- (void)pause:(int)audioEffectID {
    [self.audioEffectPlayer pause:audioEffectID];
}

- (void)pause {
    if (self.audioEffectPlayer){
        [self.audioEffectPlayer stopAll];
    }
}

- (void)resume {
    if (self.audioEffectPlayer){
        [self.audioEffectPlayer resumeAll];
    }
}
#pragma mark ----AudioEffectPlayerEvent

- (void)audioEffectPlayer:(ZegoAudioEffectPlayer *)audioEffectPlayer audioEffectID:(unsigned int)audioEffectID playStateUpdate:(ZegoAudioEffectPlayState)state errorCode:(int)errorCode {
}

- (void)downloadFileWithURL:(NSString*)requestURLString
                 parameters:(NSDictionary *)parameters
                  savedPath:(NSString*)savedPath
            downloadSuccess:(void (^)(NSURLResponse *response, NSURL *filePath))success
            downloadFailure:(void (^)(NSError *error))failure
           downloadProgress:(void (^)(NSProgress *downloadProgress))progress {

    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    manager.requestSerializer = [AFHTTPRequestSerializer serializer];
    manager.responseSerializer.acceptableContentTypes = [NSSet setWithObjects:@"text/html", @"text/plain", @"application/json", nil];
    AFSecurityPolicy *securityPolicy = [AFSecurityPolicy policyWithPinningMode:AFSSLPinningModeNone];
    [securityPolicy setValidatesDomainName:NO];
    securityPolicy.allowInvalidCertificates = YES; //还是必须设成YES
    manager.securityPolicy = securityPolicy;
    manager.requestSerializer.cachePolicy = NSURLRequestReturnCacheDataElseLoad;
    
    NSMutableURLRequest *request = [manager.requestSerializer requestWithMethod:@"GET" URLString:requestURLString parameters:parameters error:nil];
    NSURLSessionDownloadTask *task = [[AFHTTPSessionManager manager] downloadTaskWithRequest:request progress:^(NSProgress * _Nonnull downloadProgress) {
        progress(downloadProgress);
    } destination:^NSURL * _Nonnull(NSURL * _Nonnull targetPath, NSURLResponse * _Nonnull response) {
        return [NSURL fileURLWithPath:savedPath];
    } completionHandler:^(NSURLResponse * _Nonnull response, NSURL * _Nullable filePath, NSError * _Nullable error) {
        if(error){
            failure(error);
        }else{
            success(response, filePath);
        }
    }];
    [task resume];

}

- (NSString *)createAudioDirectories:(NSString *)path {
    NSString *audioFilePath = [path stringByAppendingPathComponent:@"ZegoAudios"]; // 在Library目录下创建 "ZegoAudios" 文件夹
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    BOOL isDirectories = NO;
    
    // fileExistsAtPath 判断一个文件或目录是否有效，isDirectory判断是否一个目录
    BOOL existed = [fileManager fileExistsAtPath:audioFilePath isDirectory:&isDirectories];
    
    if (!(isDirectories && existed)) {
        // 在Library目录下创建一个ZegoAudios目录
        [fileManager createDirectoryAtPath:audioFilePath withIntermediateDirectories:YES attributes:nil error:nil];
    }
    
    return audioFilePath;
    
}

- (NSMutableDictionary<NSString *,NSString *> *)namePathMap {
    if (!_namePathMap) {
        _namePathMap = [NSMutableDictionary dictionary];
    }
    return _namePathMap;
}

- (NSMutableDictionary *)nameSoundIDMap {
    if (!_nameSoundIDMap) {
        _nameSoundIDMap = [NSMutableDictionary dictionary];
    }
    return _nameSoundIDMap;
}
@end
