//
//  ChargeUtils.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/6.
//

#import "ChargeUtils.h"
#import "ChargeModel.h"
#import <AFNetworking/AFNetworking.h>
#import <NSObject+YYModel.h>

static NSString *ZEGO_HOST_URL = @"https://mini-game-test-server.zego.im";   // 开发环境
//static NSString *ZEGO_HOST_URL = @"http://192.168.84.141:3020";

@implementation ChargeUtils

+ (void)queryGameCoinWithUserID:(NSString *)userID appID:(long)appID gameID:(nonnull NSString *)gameID completion:(nonnull void (^)(NSInteger errorCode, long gameCoins))completion {
    QueryGameSpinsRequest *queryGameCoinRequest = [[QueryGameSpinsRequest alloc] init];
    [queryGameCoinRequest queryGameSpinsRequest:userID appID:appID gameID:gameID];
    [self postToURL:[NSString stringWithFormat:@"%@/api/getUserCurrency", ZEGO_HOST_URL]
            isJson:YES
              param:queryGameCoinRequest.toRequestDictionary
         completion:^(NSInteger errorCode,  NSDictionary *result) {
        QueryGameSpinsResponse *queryGameSpinsResponse = [QueryGameSpinsResponse modelWithDictionary:result];
        long gameCoins = 0;
        if (queryGameSpinsResponse != nil && queryGameSpinsResponse.code == 0) {
            gameCoins = queryGameSpinsResponse.data.gameCoin;
        }
        completion(errorCode, gameCoins);
    }];
}

+ (void)exchangeGameCoinWithUserID:(NSString *)userID appID:(long)appID gameID:(NSString *)gameID gameCoins:(long)gameCoins completion:(void(^)(NSInteger errorCode, long gameCoins))completion {
    NSString *orderID = [self getCurrentTimeStamp];
    ExchangeGameSpinsRequest *request = [[ExchangeGameSpinsRequest alloc] init];
    [request exchangeGameSpinsRequest:orderID userID:userID appID:appID gameID:gameID gameSpins:gameCoins];
    [self postToURL:[NSString stringWithFormat:@"%@/api/exchangeUserCurrency", ZEGO_HOST_URL]
             isJson:YES
              param:request.toRequestDictionary
         completion:^(NSInteger errorCode, NSDictionary *result) {
        ExchangeGameSpinsResponse *exchangeGameSpinsResponse = [ExchangeGameSpinsResponse modelWithDictionary:result];
        long gameCoin = 0;
        if (exchangeGameSpinsResponse != nil && exchangeGameSpinsResponse.code == 0) {
            gameCoin = exchangeGameSpinsResponse.data.gameCoin;
            kLog(@"exchangeGameCoin orderID：%@, managerOrderID：%@", exchangeGameSpinsResponse.data.orderID, exchangeGameSpinsResponse.data.outOrderID);
        }
        completion(errorCode, gameCoin);
    }];
}

+ (void)getTokenWithUserID:(NSString *)userID appID:(long)appID completion:(nonnull void (^)(NSInteger errorCode, NSString *token))completion {
    GetTokenRequest *request = [[GetTokenRequest alloc] init];
    [request getTokenRequest:userID appID:appID];
    [self postToURL:[NSString stringWithFormat:@"%@/api/token", ZEGO_HOST_URL]
             isJson:NO
              param:request.toRequestDictionary
         completion:^(NSInteger errorCode, id response) {
        NSString *token =[[NSString alloc] initWithData:response encoding:NSUTF8StringEncoding];
        completion(errorCode, token);
    }];
}

+ (void)describeUserCurrency:(NSString *)userID miniGameId:(NSString *)miniGameId completion:(nonnull void (^)(NSInteger errorCode, NSString *token))completion {
    DescribeUserCurrencyRequest *request = [[DescribeUserCurrencyRequest alloc] init];
    [request describeUserCurrencyRequest:userID miniGameId:miniGameId];
    [self postToURL:[NSString stringWithFormat:@"%@/api/describeUserCurrency", ZEGO_HOST_URL]
             isJson:NO
              param:request.toRequestDictionary
         completion:^(NSInteger errorCode, id response) {
//        NSString *token =[[NSString alloc] initWithData:response encoding:NSUTF8StringEncoding];
//        completion(errorCode, token);
    }];
}

+ (void)getGameLaunchCode:(long)appID gameID:(NSString *)gameID roomID:(NSString *)roomID userID:(NSString *)userID userName:(NSString *)userName avatar:(NSString *)avatar sex:(int)sex  completion:(void(^)(NSInteger errorCode, NSString * gameLaunchCode))completion {
    DescribeGameLaunchCodeRequest *request = [[DescribeGameLaunchCodeRequest alloc] init];
    [request describeGameLaunchCode:appID roomID:roomID miniGameId:gameID userID:userID nickName:userName avatar:avatar sex:sex];
    [self postToURL:[NSString stringWithFormat:@"%@/api/getGameLaunchCode", ZEGO_HOST_URL]
             isJson:YES
              param:request.toRequestDictionary
         completion:^(NSInteger errorCode, NSDictionary *result) {
        DescribeGameLaunchCodeResponse *resp = [DescribeGameLaunchCodeResponse modelWithDictionary:result];
        NSString *gameLaunchCode = nil;
        if (resp != nil && resp.code == 0) {
            gameLaunchCode = resp.Data.gameLaunchCode;
            kLog(@"getGameLaunchCode gameCode：%@", resp.Data.gameLaunchCode);
        }
        completion(errorCode, gameLaunchCode);
    }];
}


#pragma mark - Private
+ (void)postToURL:(NSString *)url
           isJson:(BOOL)isJson
            param:(NSDictionary *)param
       completion:(void(^)(NSInteger errorCode, id result))completion {
    NSLog(@"\n----------------------------------\noutRoomService, postToURL: %@\nparam: %@\n----------------------------------", url, param);
    
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    manager.requestSerializer = [AFJSONRequestSerializer serializer];
    if (isJson == YES) {
        manager.responseSerializer = [AFJSONResponseSerializer serializer];
    } else {
        manager.responseSerializer = [AFHTTPResponseSerializer serializer];
    }
    manager.responseSerializer.acceptableContentTypes = [NSSet setWithObjects:@"text/html", @"text/plain", @"application/json", nil];
    AFSecurityPolicy *securityPolicy = [AFSecurityPolicy policyWithPinningMode:AFSSLPinningModeNone];
    [securityPolicy setValidatesDomainName:NO];
    securityPolicy.allowInvalidCertificates = YES; //还是必须设成YES
    manager.securityPolicy = securityPolicy;

    [manager POST:url
       parameters:param
          headers:nil
         progress:nil
          success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        NSLog(@"\n----------------------------------\npost succeed, url: %@, response: %@\n----------------------------------", url, responseObject);
        NSDictionary *dataJson = nil;
        NSDictionary *retJson = nil;
        //        if ([responseObject isKindOfClass:[NSDictionary class]]) {
        //            dataJson = responseObject[@"data"];
        //            retJson = responseObject[@"ret"];
        //        }
        //        completion([retJson[@"code"] integerValue], dataJson);
        if (isJson) {
            completion([responseObject[@"Code"] integerValue], responseObject);
        } else {
            completion(0, responseObject);
        }
    }
          failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        NSLog(@"----------------------------------\npost fail, url: %@, error: %@\n----------------------------------", url, error);
        completion(error.code, nil);
    }];
}


+ (NSString *)getCurrentTimeStamp {
    NSDate *dat = [NSDate dateWithTimeIntervalSinceNow:0];
    NSTimeInterval a = [dat timeIntervalSince1970] * 1000;
    NSString *timeString = [NSString stringWithFormat:@"%.0lf", a];
    return timeString;
}

@end

