//
//  ZegoRTCEngineManager.h
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/8.
//

#import <Foundation/Foundation.h>

typedef NS_ENUM(NSInteger, RoomStateChangeReason) {
    KickOut = 0,
    Disconnect = 1,
    LoginFailed = 2,
    Reconnected = 3,
};

NS_ASSUME_NONNULL_BEGIN
typedef void(^JoinRoomCompletion)(int errorCode, NSDictionary * _Nonnull extendedData);
typedef void (^ZegoDestroyCallback)(void);
@class StreamInfo;

@protocol RTCEventListener <NSObject>

@optional
- (void)onStreamAdd:(NSArray <StreamInfo *> *)streamInfoList;
- (void)onStreamRemove:(NSArray <StreamInfo *> *)streamInfoList;
- (void)onRoomStateChanged:(RoomStateChangeReason)reason;
- (void)onRoomOnlineUserCountUpdate:(NSString *)roomID count:(int)count;
- (void)onRoomExtraInfoUpdate:(NSString *)roomID arrayList:(NSArray<ZegoRoomExtraInfo *> *)arrayList;
- (void)onRoomUserUpdate:(NSArray<ZegoUser *>*)userList;
- (void)onRoomPublisherStreamEvent:(ZegoStreamEvent)eventID streamID:(NSString *)streamID extraInfo:(NSString *)extraInfo;
@end

@interface ZegoRTCEngineManager : NSObject

@property (nonatomic, copy) JoinRoomCompletion joinRoomCompletion;
@property (nonatomic, copy) NSString *publishStreamID;

+ (instancetype)sharedInstance;

/// 初始化Zego RTC
- (void)initRtc;

/// 反初始化Zego RTC
/// - Parameter callBack: callBack
- (void)unInitRtc:(ZegoDestroyCallback)callBack;

/// 加入房间
/// - Parameters:
///   - roomID: 房间ID
///   - userID: 用户ID
///   - userName: 用户昵称
///   - completion: completion
- (void)loginRoom:(NSString *)roomID userID:(NSString *)userID userName:(NSString *)userName
       completion:(JoinRoomCompletion)completion;

/// 退出房间
- (void)logoutRoom;

/// 开始推流
/// - Parameter isHost: 是否是主播
- (void)startPublishStream:(BOOL)isHost;

/// 停止推流
- (void)stopPublishStream;

/// 是否在房间
- (BOOL)isInRoom;

/// 开始拉流
/// - Parameters:
///   - streamID: 流ID
///   - view: view
- (void)startPlayingStream:(NSString *)streamID renderView:(UIView *)view;

/// 停止拉流
/// - Parameter streamID: 流ID
- (void)stopPlayingStream:(NSString *)streamID;

/// 开始预览
/// - Parameter renderView: renderView
- (void)startPreview:(UIView *)renderView;

/// 获取流列表
- (NSArray <StreamInfo *> *)getStreamInfoList;

/// 获取房间成员数量
- (int)getMemberCount;

/// 设置房间附加消息
/// - Parameters:
///   - roomID: 房间ID
///   - key: 附加消息的key
///   - value: 附加消息的 value
- (void)setRoomExtraInfo:(NSString *)roomID forKey:(NSString *)key value:(NSString *)value;

/// 获取房间附加消息
- (NSArray <ZegoRoomExtraInfo *> *)getRoomExtraInfoList;

/// 添加manager 监听
/// - Parameter listener: listener
- (void)addRTCEventListener:(id<RTCEventListener>)listener;

/// 移除manager 监听
/// - Parameter listener: listener
- (void)removeRTCEventListener:(id<RTCEventListener>)listener;

/// 获取房间用户列表
- (NSArray <ZegoUser *>*)getRtcUserList;

/// 麦克风开关
/// - Parameter mute: mute
- (void)muteMicrophone:(BOOL)mute;

/// 开关指定userID 的麦克风
/// - Parameters:
///   - userID: 用户ID
///   - mute: mute
- (void)muteSpeaker:(NSString *)userID mute:(BOOL)mute;


///  初始化语音类相关的RTC配置
- (void)initAudioGameConfig;
@end

@interface StreamInfo : NSObject

@property (nonatomic, assign) BOOL isHost;
@property (nonatomic, copy) NSString *userID;
@property (nonatomic, copy) NSString *userName;
@property (nonatomic, copy) NSString *streamID;
@end

@interface RTCUser : NSObject

@property (nonatomic, strong) NSString *userID;
@property (nonatomic, strong) NSString *userName;
@property (nonatomic, assign) int type;

@end



NS_ASSUME_NONNULL_END
