//
//  ViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2022/12/2.
//

#import "ViewController.h"
#import <ZegoMiniGameEngine/ZegoMiniGameEngine.h>
#import "ChargeUtils.h"
#import "CollectionViewCell.h"
#import "ZegoRTCEngineManager.h"
#import "ZGStreamLiveViewController.h"
#import "ZGPlayGameViewController.h"
#import "ZGLanguageViewController.h"
#import "ZGExchangeRecordController.h"
#import "ZegoGameViewModel.h"
#import "ZGStreamLiveAlterView.h"
#import "MineGameMainView.h"
#import "ZegoGameTopView.h"
#import "ChargeGameInfo.h"
#import "PlayGameView.h"
#import "CloudGameView.h"
#import "OpenStreamLiveView.h"
#import "KeyCenter.h"
#import "AlignButton.h"
#import "ZegoCollectionReusableView.h"
#import "ZegoAudioroomViewController.h"
#import "ZegoSelectAppIDView.h"
#import "ZegoCloudGameViewController.h"
#import "ZegoGameRoomViewController.h"

#import <CoreLocation/CoreLocation.h>

typedef NS_ENUM(NSUInteger, ZegoTabButtonType) {
    ZGGameTabButtonTypeHall = 0,
    ZGGameTabButtonTypeLive,
    ZGGameTabButtonTypePlay,
    ZGGameTabButtonTypecloud,
    ZGGameTabButtonTypeMine,
};

@interface ViewController ()<UICollectionViewDelegate, UICollectionViewDataSource, UIScrollViewDelegate, UIGestureRecognizerDelegate, ZegoGameTopViewDelegate, PlayGameViewDelegate,CloudGameViewDelegate,CLLocationManagerDelegate>
@property (nonatomic, strong) UIScrollView *scrollView;
// 直播大厅view
@property (nonatomic, strong) UICollectionView *collectionView;
// 我的view
@property (nonatomic, strong) MineGameMainView *mineGameView;
// 开直播view
@property (nonatomic, strong) OpenStreamLiveView *streamLiveView;
// 游戏大厅
@property (nonatomic, strong) PlayGameView *playGameView;
// 游戏大厅
@property (nonatomic, strong) CloudGameView *cloudGameView;
// 底部tabbar
@property (nonatomic, strong) UIView *bottomView;
// 随机名字
@property (nonatomic, strong) NSArray *randomNames;
// 随机头像
@property (nonatomic, strong) NSArray *randomAvatar;
// 数据model
@property (nonatomic, strong) ZegoGameViewModel *viewModel;
// 底部tab 游戏大厅按钮
@property (nonatomic, strong) AlignButton *gameHallButton;
// 底部tab 直播大厅按钮
@property (nonatomic, strong) AlignButton *liveHallButton;
// 底部tab 开直播按钮
@property (nonatomic, strong) AlignButton *liveActionBtn;
// 底部tab 云游戏
@property (nonatomic, strong) AlignButton *cloudActionBtn;
// 底部tab 我的按钮
@property (nonatomic, strong) AlignButton *mineButton;
// 我的页面时--- 设置按钮
@property (nonatomic, strong) UIButton *settingBtn;
@property (nonatomic, strong) UIButton *recordBtn;//暂时废弃
// 头像视图
@property (nonatomic, strong) UIImageView *avatarImageView;
// 用户昵称
@property (nonatomic, strong) UILabel *userNameLabel;
// appID label
@property (nonatomic, strong) UILabel *environmentLabel;
@property (nonatomic, assign) NSUInteger currentTag;
@property (nonatomic, assign) BOOL isInit;

// 测试区域限制时使用
@property (strong, nonatomic) CLLocationManager *locationManager;
@property (strong, nonatomic) CLGeocoder *geoCoder;
@property (nonatomic, retain) CLLocation *myLocation;


@end

@implementation ViewController

- (void)tapAvatar{
    ZegoCloudGameViewController *vc = [[ZegoCloudGameViewController alloc] init];
    [self.navigationController pushViewController:vc animated:true];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self initData];
    [self setupBaseUI];
    //[self testLocation];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.navigationController.navigationBarHidden = YES;
    self.tabBarController.hidesBottomBarWhenPushed = YES;
    self.mineGameView.goldCoinsAmountLabel.text = [NSString stringWithFormat:@"%d", (int)self.viewModel.businessCoin];
    [self.mineGameView.gameTableView reloadData];
    [self.mineGameView configureData];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self.navigationController setNavigationBarHidden:YES animated:NO];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.navigationController.navigationBarHidden = NO;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] postNotificationName:@"updateGameData" object:nil];
}

- (NSString *)userID {
    UIDevice *currentDevice = [UIDevice currentDevice];
    NSString *deviceId = [[currentDevice identifierForVendor] UUIDString];
    NSString *md5Str = deviceId.md5String;
    NSString *userID = [md5Str substringFromIndex:md5Str.length - 10];
    NSLog(@"logined userID generated: %@", userID);
    return userID;
}

- (void)initData {
    if (!self.isInit) {
        @weakify(self)
        [self initRandomData];
        NSString *userID = [self userID] ;
        NSInteger randomNamesIndex = arc4random() % self.randomNames.count;
        NSInteger randomAvatarIndex = arc4random() % self.randomAvatar.count;
        ZegoGameViewModel *viewModel = [[ZegoGameViewModel alloc] init];
        viewModel.userID = userID;
        viewModel.userName = self.randomNames[randomNamesIndex];
        viewModel.userAvatar = self.randomAvatar[randomAvatarIndex];
        self.viewModel = viewModel;
        ZegoGameUserInfo *userInfo = [[ZegoGameUserInfo alloc] init];
        userInfo.userID = viewModel.userID;
        userInfo.userName = viewModel.userName;
        userInfo.avatar = viewModel.userAvatar;
        [ChargeUtils getTokenWithUserID:userID appID:[KeyCenter appID] completion:^(NSInteger errorCode, NSString *token) {
            kLog(@"[getRTCToken] errorCode：%ld, token：%@", errorCode, token);
            if (errorCode != 0) {
                return;
            }
            [KeyCenter setRTCToken:token];
            [ZegoMiniGameEngine.sharedEngine init:[KeyCenter appID] token:token userInfo:userInfo callBack:^(int errorCode2, id  _Nullable object) {
                kLog(@"miniGameEngine.init errorCode：%d", errorCode2);
                if (errorCode2 != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(init_failed), [ZegoConstants getErrorMsg:(int)errorCode2]] withFinishBlock:nil];
                } else {
//                    if ([[NSBundle currentLanguage] containsString:@"zh-Hans"]) {
//                        [ZegoMiniGameEngine.sharedEngine setGameLanguage:ZegoGameLanguageZHHans];
//                    } else if ([[NSBundle currentLanguage] containsString:@"en"]) {
//                        [ZegoMiniGameEngine.sharedEngine setGameLanguage:ZegoGameLanguageEN];
//                    } else if ([[NSBundle currentLanguage] containsString:@"ar"]) {
//                        [ZegoMiniGameEngine.sharedEngine setGameLanguage:ZegoGameLanguageAR];
//                    }
                    // 获取所有游戏列表
                    [ZegoMiniGameEngine.sharedEngine getAllGameList:^(NSInteger errorCode3, NSArray<ZegoGameInfo *> * _Nullable gameList) {
                        @strongify(self)
                        if (errorCode3 == 0) {
                            NSMutableArray <ChargeGameInfo *> *chargeGameInfList = [NSMutableArray array];
                            NSMutableArray <ZegoGameInfo *> *hostGameList = [NSMutableArray array];
                            NSMutableArray <ZegoGameInfo *> *showGameList = [NSMutableArray array];
                            NSMutableArray <ZegoGameInfo *> *mallGameList = [NSMutableArray array];
                            NSMutableArray <ZegoGameInfo *> *cloudGameList = [NSMutableArray array];
                            NSMutableArray <ZegoGameInfo *> *matchInGameList = [NSMutableArray array];
                            
                            for (ZegoGameInfo *gameInfo in gameList) {
                                ChargeGameInfo *chargeGameInfo = [[ChargeGameInfo alloc] init];
                                chargeGameInfo.gameCoin = EXCHANGE_COIN_NUMBER;
                                chargeGameInfo.gameInfo = gameInfo;
                                [chargeGameInfList addObject:chargeGameInfo];
                                
                                if([gameInfo.gameMode containsObject:@(ZegoGameModeHostsGame)]) {
                                    [hostGameList addObject:gameInfo];
                                }
                                
                                if([gameInfo.gameMode containsObject:@(ZegoGameModeShowGame)]) {
                                    [showGameList addObject:gameInfo];
                                }
                                
                                if([gameInfo.gameMode containsObject:@(ZegoGameModeMallGame)]) {
                                    [mallGameList addObject:gameInfo];
                                }
                                
                                if([gameInfo.gameMode containsObject:@(ZegoGameModeCloudGame)]) {
                                    [cloudGameList addObject:gameInfo];
                                }
                                
                                if([gameInfo.gameMode containsObject:@(ZegoGameModeMatchInGame)]) {
                                    [matchInGameList addObject:gameInfo];
                                }
                                
                            }
                            
                            self.viewModel.allGameList = chargeGameInfList;
                            self.mineGameView.viewModel = self.viewModel;
                            [self.mineGameView.gameTableView reloadData];
                            [self.mineGameView configureData];
                            
                            self.viewModel.hostGameList = hostGameList;
                            self.viewModel.showGameList = showGameList;
                            
                            self.viewModel.fullScreenGameList = mallGameList;
                            self.playGameView.gameList = mallGameList;
                            [self.playGameView.collectionView reloadData];
                            
                            self.viewModel.CloudGameList = cloudGameList;
                            self.cloudGameView.gameList = cloudGameList;
                            [self.cloudGameView.collectionView reloadData];
                            
                            self.viewModel.matchInGameList = matchInGameList;
                            
                        } else {
                            [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(get_game_list_failed), [ZegoConstants getErrorMsg:(int)errorCode3]] withFinishBlock:nil];
                        }
                    }];
                }
            }];
        }];
        
        [self.avatarImageView setImageWithURL:[NSURL URLWithString:self.randomAvatar[randomAvatarIndex]] placeholder:[UIImage imageNamed:@""]];
        self.userNameLabel.text = self.randomNames[randomNamesIndex];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_updateGameData) name:@"updateGameData" object:nil];
    }
    self.isInit = YES;
    /// demo token 的有效期为一个小时，demo层自己控制token 过期，更新token ,也可以通过小游戏SDK
    double time = 60 * 60 * 1000;
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(time * NSEC_PER_MSEC)), dispatch_get_main_queue(), ^{
        [self autoUpdateTokenRunnable];
    });
    @weakify(self)
    self.streamLiveView.viewModel = self.viewModel;
    /// 开直播-点击具体的房间
    [self.streamLiveView joinLiveRoomPKClickBlock:^(ZegoGameViewModel *model) {
        @strongify(self)
        
        if (model.isLive == 1) {
            ZGStreamLiveViewController *streamRoomVc = [[ZGStreamLiveViewController alloc] init];
            streamRoomVc.viewModel = self.viewModel;
            [self.navigationController pushViewController:streamRoomVc animated:YES];
        } else if(model.isLive == 2) {
            ZegoGameRoomViewController *RoomVc = [[ZegoGameRoomViewController alloc] init];
            RoomVc.viewModel = self.viewModel;
            [self.navigationController pushViewController:RoomVc animated:YES];
        } else  {
            ZegoAudioroomViewController *audioRoomVc = [[ZegoAudioroomViewController alloc] init];
            audioRoomVc.viewModel = self.viewModel;
            [self.navigationController pushViewController:audioRoomVc animated:YES];
        }
        
    }];
    [self.streamLiveView setDismissLiveViewCompletion:^{
        @strongify(self)
        [self setButtonSelectedWithType:self.currentTag];
    }];
}

- (void)_updateGameData {
    self.mineGameView.goldCoinsAmountLabel.text = [NSString stringWithFormat:@"%d", (int)self.viewModel.businessCoin];
    [self.mineGameView.gameTableView reloadData];
    [self.mineGameView configureData];
}

/// 更新token
- (void)autoUpdateTokenRunnable {
    [ChargeUtils getTokenWithUserID:self.viewModel.userID appID:[KeyCenter appID] completion:^(NSInteger errorCode, NSString *token) {
        if (errorCode == 0) {
            // 更新token
            [KeyCenter setRTCToken:token];
            [ZegoMiniGameEngine.sharedEngine updateToken:token];
            double time = 60 * 60 * 1000;
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(time * NSEC_PER_MSEC)), dispatch_get_main_queue(), ^{
                [self autoUpdateTokenRunnable];
            });
        } else {
            double time = 10 * 1000;
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(time * NSEC_PER_MSEC)), dispatch_get_main_queue(), ^{
                [self autoUpdateTokenRunnable];
            });
        }
    }];
}
- (void)setupBaseUI {
    
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"img_bg"]];

    [self.view addSubview:self.avatarImageView];
    [self.view addSubview:self.userNameLabel];
    [self.view addSubview:self.environmentLabel];
    [self.view addSubview:self.settingBtn];
//    [self.view addSubview:self.recordBtn];
    [self.view addSubview:self.scrollView];
    [self.view addSubview:self.bottomView];
    [self.bottomView bringSubviewToFront:self.view];
    
    [self.scrollView addSubview:self.playGameView];
    [self.scrollView addSubview:self.collectionView];
    [self.scrollView addSubview:self.mineGameView];
    [self.scrollView addSubview:self.streamLiveView];
    [self.scrollView addSubview:self.cloudGameView];
    
    [self.bottomView addSubview:self.gameHallButton];
    [self.bottomView addSubview:self.liveHallButton];
    [self.bottomView addSubview:self.liveActionBtn];
    [self.bottomView addSubview:self.cloudActionBtn];
    [self.bottomView addSubview:self.mineButton];

    [self.avatarImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left).offset(16);
        make.top.mas_equalTo(self.view.mas_top).offset(kTLTopStatusBarHeight + 20);
        make.height.mas_equalTo(44);
        make.width.mas_equalTo(44);
    }];
    
    [self.userNameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatarImageView.mas_right).offset(16);
        make.centerY.mas_equalTo(self.avatarImageView.mas_centerY);
    }];

    [self.environmentLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.view).offset(-20);
        make.top.mas_equalTo(self.view.mas_top).offset(kTLTopStatusBarHeight);
        make.height.mas_equalTo(25);
        make.width.mas_equalTo(250);
    }];
    
//    [self.recordBtn mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.right.mas_equalTo(self.settingBtn.mas_left).offset(-16);
//        make.centerY.equalTo(self.avatarImageView.mas_centerY);
//        make.height.mas_equalTo(30);
//        make.width.mas_equalTo(76);
//    }];
    
    [self.settingBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.view).offset(-21);
        make.centerY.equalTo(self.avatarImageView.mas_centerY);
        make.height.mas_equalTo(22);
        make.width.mas_equalTo(22);
    }];
    
    
    [self.scrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.avatarImageView.mas_bottom).offset(20) ;
        make.left.right.width.equalTo(self.view);
        make.height.mas_equalTo(SCROLL_HEIGHT);
    }];
    
    [self.playGameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.equalTo(self.scrollView);
        make.width.mas_equalTo(kScreenWidth);
        make.height.mas_equalTo(self.scrollView);
    }];
    
    [self.collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.equalTo(self.scrollView);
        make.left.equalTo(self.playGameView.mas_right);
        make.height.equalTo(self.scrollView);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.streamLiveView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.equalTo(self.scrollView);
        make.left.equalTo(self.collectionView.mas_right);
        make.height.equalTo(self.scrollView);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.cloudGameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.equalTo(self.scrollView);
        make.left.equalTo(self.streamLiveView.mas_right);
        make.height.equalTo(self.scrollView);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.mineGameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.equalTo(self.scrollView);
        make.left.equalTo(self.cloudGameView.mas_right);
        make.height.equalTo(self.scrollView);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.bottomView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.scrollView.mas_bottom);
        make.bottom.equalTo(self.view);
        make.height.mas_equalTo(84 + kTLBottomStatusBarHeight);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    CGFloat width = (kScreenWidth - 10*2)/5;
    
    [self.gameHallButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.bottomView).offset(10);
        make.top.height.width.bottom.equalTo(self.liveActionBtn);
    }];
    
    [self.liveHallButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.gameHallButton.mas_right).offset(10);
        make.top.height.width.bottom.equalTo(self.liveActionBtn);
    }];
    
    [self.liveActionBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.liveHallButton.mas_right).offset(10);
        make.top.mas_equalTo(self.bottomView.mas_top).offset(12);
        make.width.mas_equalTo((kScreenWidth - 54 - 10) / 5);
        make.height.mas_equalTo(42);
    }];
    
    [self.cloudActionBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.liveActionBtn.mas_right).offset(10);
        make.top.mas_equalTo(self.bottomView.mas_top).offset(12);
        make.width.mas_equalTo((kScreenWidth - 54) / 4);
        make.height.mas_equalTo(42);
    }];
    
    [self.mineButton mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.right.equalTo(self.view).offset(-10);
        make.left.equalTo(self.cloudActionBtn.mas_right).offset(10);
        make.top.height.width.bottom.equalTo(self.liveActionBtn);
    }];
}

- (void)initRandomData {
    self.randomNames = @[@"张三", @"李四", @"王五", @"武则天", @"秦始皇", @"刘邦", @"项羽", @"女娲", @"王昭君", @"西施",
                         @"貂蝉", @"杨玉环", @"赵飞燕", @"上官婉儿", @"孙悟空", @"唐僧", @"猪八戒", @"牛魔王", @"盘古", @"太上老君",
                         @"周瑜", @"吴刚"];
    self.randomAvatar = @[@"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Fb7d2b72179deb6f134ca3006709e865c%2F16%2Fb7d2b72179deb6f134ca3006709e865c%2Fthumbnails%2F1.png?8m79mtgw0h2rqqhx", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Fb7d2b72179deb6f134ca3006709e865c%2F16%2Fb7d2b72179deb6f134ca3006709e865c%2Fthumbnails%2F1.png?8m79mtgw0h2rqqhx", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F61795d64787b32be071a61815cf305f0%2F16%2F61795d64787b32be071a61815cf305f0%2Fthumbnails%2F1.png?x71e2k5hkdjy6d9d", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Fc123c70be5a933d24119ab9a7023d941%2F16%2Fc123c70be5a933d24119ab9a7023d941%2Fthumbnails%2F1.png?fnvzwzqdfa5umu0q", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Fbdf1f499db619c8d007993f2f80b3a08%2F16%2Fbdf1f499db619c8d007993f2f80b3a08%2Fthumbnails%2F1.png?ugdhfdidkepug1uw", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F99fe2e224d26ad4f747f3f8c973b8a37%2F16%2F99fe2e224d26ad4f747f3f8c973b8a37%2Fthumbnails%2F1.png?6tub8wo058btyxlt", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F4ed11a20532ba7b2115e9288ed72b5a5%2F16%2F4ed11a20532ba7b2115e9288ed72b5a5%2Fthumbnails%2F1.png?zfbbm6uny2w7mezf", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F8d6abe9c6f744bfe26e12f459e4d454a%2F16%2F8d6abe9c6f744bfe26e12f459e4d454a%2Fthumbnails%2F1.png?55krvuez9ogs5lgf", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F299f3c1b5d502e3fc1acb70222385c9b%2F16%2F299f3c1b5d502e3fc1acb70222385c9b%2Fthumbnails%2F1.png?opvop22azg75d6e6", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F29ff384c19afd85418a665ad30ed599f%2F16%2F29ff384c19afd85418a665ad30ed599f%2Fthumbnails%2F1.png?g8gny92brmg421h4",                          @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F0fefb0b0dddbaf5b0d8801e17bd27085%2F16%2F0fefb0b0dddbaf5b0d8801e17bd27085%2Fthumbnails%2F1.png?n0uj3to4diztaz2c", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F45ed2bf73239d6bc9e90bfc27d3d059e%2F16%2F45ed2bf73239d6bc9e90bfc27d3d059e%2Fthumbnails%2F1.png?ybqr3lrmuw45c7v7", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F732f10bfe988cca9c32779b2d6cd50ec%2F16%2F732f10bfe988cca9c32779b2d6cd50ec%2Fthumbnails%2F1.png?wbyfrzrsuxjnjdpp", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Ffbb3fd12a3c8e5d03b6608c231c6899e%2F16%2Ffbb3fd12a3c8e5d03b6608c231c6899e%2Fthumbnails%2F1.png?eu7bqwr8lsydfk1v", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2F7c59ce2bc7fe93083a1d68a34ff37c22%2F16%2F7c59ce2bc7fe93083a1d68a34ff37c22%2Fthumbnails%2F1.png?i72ntxnqcd7npmem", @"https://docservice-storage-alpha.zego.im/00c483552faefc8df47184643f9defee%2Fb63236dcfca4ea5efeb48fbaf2769147%2F16%2Fb63236dcfca4ea5efeb48fbaf2769147%2Fthumbnails%2F1.png?u980x6c9mhcj87gj"];
    self.environmentLabel.text = [NSString stringWithFormat:@"APPID:%ld/%@", [KeyCenter appID],[self userID]];
//    self.environmentLabel.text = [NSString stringWithFormat:@"ID:%@", [self userID]];
}

- (void)setButtonSelectedWithType:(ZegoTabButtonType)type {
    self.gameHallButton.selected = type == self.gameHallButton.tag;
    self.liveHallButton.selected = type == self.liveHallButton.tag;
    self.liveActionBtn.selected = type == self.liveActionBtn.tag;
    self.cloudActionBtn.selected = type == self.cloudActionBtn.tag;
    self.mineButton.selected = type == self.mineButton.tag;
    if (type != ZGGameTabButtonTypeMine) {
        self.settingBtn.hidden = YES;
        self.recordBtn.hidden = YES;
    } else {
        self.settingBtn.hidden = NO;
        self.recordBtn.hidden = NO;
    }
}

/// 切换到---游戏大厅tabbar
- (void)gameHallActionButtonClick:(UIButton *)sender {
    CGFloat x = sender.tag * kScreenWidth;
    [self.scrollView setContentOffset:CGPointMake(x, 0) animated:YES];
    [self setButtonSelectedWithType:sender.tag];
}

/// 切换到---直播大厅tabbar
- (void)liveHallActionButtonClick:(UIButton *)sender {
    [self setButtonSelectedWithType:ZGGameTabButtonTypeLive];
    CGFloat x = sender.tag * kScreenWidth;
    [self.scrollView setContentOffset:CGPointMake(x, 0) animated:YES];
    [self setButtonSelectedWithType:sender.tag];
}

/// 切换到---开直播tabbar
- (void)liveActionButtonClick:(UIButton *)sender {
    CGFloat x = sender.tag * kScreenWidth;
    [self.scrollView setContentOffset:CGPointMake(x, 0) animated:YES];
    [self setButtonSelectedWithType:sender.tag];
    [self.mineGameView.gameTableView reloadData];
}

- (void)cloudActionButtonClick:(UIButton *)sender {
    CGFloat x = sender.tag * kScreenWidth;
    [self.scrollView setContentOffset:CGPointMake(x, 0) animated:YES];
    [self setButtonSelectedWithType:sender.tag];
    [self.mineGameView.gameTableView reloadData];
    [self.mineGameView configureData];
}

/// 切换到---我的tabbar
- (void)mineActionButtonClick:(UIButton *)sender {
    CGFloat x = sender.tag * kScreenWidth;
    [self.scrollView setContentOffset:CGPointMake(x, 0) animated:YES];
    [self setButtonSelectedWithType:sender.tag];
    [self.mineGameView.gameTableView reloadData];
    [self.mineGameView configureData];
}

- (void)recordActionButtonClick:(UIButton *)sender {
    ZGExchangeRecordController *vc = [[ZGExchangeRecordController alloc] init];
    [self.navigationController pushViewController:vc animated:YES];
}

- (void)settingActionButtonClick:(UIButton *)sender {
    ZGLanguageViewController *vc = [[ZGLanguageViewController alloc] init];
    [self.navigationController pushViewController:vc animated:YES];
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 3;
}

// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return 6;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    CollectionViewCell *cell  = [collectionView dequeueReusableCellWithReuseIdentifier:@"CollectionViewCell" forIndexPath:indexPath];
    cell.backgroundColor = [UIColor colorWithRGB:0xFF574979 alpha:0.32];
    if(indexPath.section == 0) {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(audio_room), (long)indexPath.row + 1];
    } else if(indexPath.section == 1) {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(live_room), (long)indexPath.row + 1];
    } else {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(game_room), (long)indexPath.row + 1];
    }
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section {
    if(section == 1 || section == 2) {
        return CGSizeMake(kScreenWidth, 40);
    } else {
        return CGSizeMake(kScreenWidth, 1);
    }
    
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath {
    if ([kind isEqualToString:UICollectionElementKindSectionHeader]) {
        ZegoCollectionReusableView *headerView = [collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:NSStringFromClass(ZegoCollectionReusableView.class) forIndexPath:indexPath];
        NSString *title = indexPath.section == 0 ? TLLocalizedString(chose_audio_room) : TLLocalizedString(chose_live_room);
        
        return headerView;
    }
    return  nil;
}

- (void)viewDidLayoutSubviews {
    self.scrollView.contentSize = CGSizeMake(kScreenWidth * 5, 0);
}

/// 直播大厅-点击具体的房间
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    @weakify(self)
    self.viewModel.isHost = false;
    NSString *newRoomID = nil;
    if(indexPath.section == 0) {
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_CHAT, indexPath.row + 1];
    } else if(indexPath.section == 1) {
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_LIVE, indexPath.row + 1];
    } else {
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_GAME, indexPath.row + 1];
    }
    if (![ZegoRTCEngineManager.sharedInstance isInRoom]) {
        [ZegoRTCEngineManager.sharedInstance logoutRoom];
    }
    [ZegoRTCEngineManager.sharedInstance loginRoom:newRoomID userID:self.viewModel.userID userName:self.viewModel.userName completion:^(int errorCode, NSDictionary * _Nonnull extendedData) {
        @strongify(self)
        self.viewModel.roomID = newRoomID;
        if(indexPath.section == 0) {
            ZegoAudioroomViewController *audioRoomVc = [[ZegoAudioroomViewController alloc] init];
            audioRoomVc.viewModel = self.viewModel;
            [self.navigationController pushViewController:audioRoomVc animated:YES];
        } else if(indexPath.section == 1) {
            ZGStreamLiveViewController *streamRoomVc = [[ZGStreamLiveViewController alloc] init];
            streamRoomVc.viewModel = self.viewModel;
            [self.navigationController pushViewController:streamRoomVc animated:YES];
        } else {
            ZegoGameRoomViewController *vc = [[ZegoGameRoomViewController alloc] init];
            vc.viewModel = self.viewModel;
            [self.navigationController pushViewController:vc animated:YES];
        }
    }];
}


#pragma mark - UIScrollViewDelegate
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    //计算位置
    CGFloat offsetX = scrollView.contentOffset.x;
    NSUInteger tag = offsetX / kScreenWidth;
//    [self.topView selectTopViewActionButtonWithType:tag];
    self.mineGameView.viewModel = self.viewModel;
    [self setButtonSelectedWithType:tag];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    //计算位置
    CGFloat offsetX = scrollView.contentOffset.x;
    NSUInteger tag = offsetX / kScreenWidth;
    NSLog(@"");
}

#pragma mark - ZegoGameTopViewDelegate
/// 游戏大厅-点击具体游戏
- (void)selectGameActionCellWithRow:(NSInteger)row {
    ZegoGameInfo *gameInfo = self.viewModel.fullScreenGameList[row];
    ZGPlayGameViewController *vc = [[ZGPlayGameViewController alloc] init];
    vc.viewModel = self.viewModel;
    vc.currentGameID = gameInfo.gameID;
    [self.navigationController pushViewController:vc animated:YES];
}

- (void)switchEnvironment:(UITapGestureRecognizer *)tap {
    ZegoSelectAppIDView *selectAppIDView = [[ZegoSelectAppIDView alloc] initWithFrame:ZegoKeyWindow.bounds dataSource:KeyCenter.environmentAppids];
    [ZegoKeyWindow addSubview:selectAppIDView];
}

/// 云游戏-点击具体游戏
- (void)selectCloudGameActionCellWithRow:(NSInteger)row {
    ZegoGameInfo *gameInfo = self.viewModel.CloudGameList[row];
    ZegoCloudGameViewController *vc = [[ZegoCloudGameViewController alloc] init];
    vc.gameID = gameInfo.gameID;
    vc.viewModel = self.viewModel;
    [self.navigationController pushViewController:vc animated:YES];
}

#pragma mark - getter

- (UILabel *)environmentLabel {
    if(!_environmentLabel) {
        _environmentLabel = [[UILabel alloc] init];
        _environmentLabel.textColor = [UIColor grayColor];
        _environmentLabel.font = BOLD_FONT(16);
        _environmentLabel.textAlignment = NSTextAlignmentRight;
        _environmentLabel.userInteractionEnabled = true;
        [_environmentLabel sizeToFit];
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(switchEnvironment:)];
        tap.numberOfTapsRequired = 6;
        [_environmentLabel addGestureRecognizer:tap];
    }
    return _environmentLabel;
}

- (UIScrollView *)scrollView {
    if (!_scrollView) {
        _scrollView = [UIScrollView new];
        _scrollView.showsHorizontalScrollIndicator = NO;
        _scrollView.pagingEnabled = YES;
        _scrollView.delegate = self;
        _scrollView.backgroundColor = [UIColor clearColor];
    }
    return _scrollView;
}

- (UICollectionView *)collectionView {
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
        layout.minimumLineSpacing = 18;
        layout.minimumInteritemSpacing = 12;
        layout.sectionInset = UIEdgeInsetsMake(0, 16, 10, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 56) / 3,  (kScreenWidth - 56) / 3);
        _collectionView = [[UICollectionView alloc] initWithFrame:self.view.bounds collectionViewLayout:layout];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        [_collectionView registerClass:[CollectionViewCell class] forCellWithReuseIdentifier:@"CollectionViewCell"];
        _collectionView.backgroundColor = [UIColor clearColor];
        [_collectionView registerClass:[ZegoCollectionReusableView class] forSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"ZegoCollectionReusableView"];
    }
    return _collectionView;
}

- (PlayGameView *)playGameView {
    if (!_playGameView) {
        _playGameView = [PlayGameView new];
        _playGameView.backgroundColor = [UIColor clearColor];
        _playGameView.delegate  = self;
    }
    return _playGameView;
}

- (MineGameMainView *)mineGameView {
    if (!_mineGameView) {
        _mineGameView = [MineGameMainView new];
        _mineGameView.userInteractionEnabled = YES;
        _mineGameView.backgroundColor = [UIColor clearColor];
    }
    return _mineGameView;
}

- (OpenStreamLiveView *)streamLiveView {
    if (!_streamLiveView) {
        _streamLiveView = [OpenStreamLiveView new];
        _streamLiveView.userInteractionEnabled = YES;
        _streamLiveView.backgroundColor = [UIColor clearColor];
    }
    return _streamLiveView;
}

- (CloudGameView *)cloudGameView {
    if (!_cloudGameView) {
        _cloudGameView = [CloudGameView new];
        _cloudGameView.backgroundColor = [UIColor clearColor];
        _cloudGameView.delegate  = self;
    }
    return _cloudGameView;
}

- (UIView *)bottomView {
    if (!_bottomView) {
        _bottomView = [UIView new];
        _bottomView.userInteractionEnabled = YES;
        _bottomView.backgroundColor = [UIColor colorWithRGB:0xFF211732 alpha:0.9];
    }
    return _bottomView;
}

- (UIImageView *)avatarImageView {
    if (!_avatarImageView) {
        _avatarImageView = [[UIImageView alloc] init];
        _avatarImageView.contentMode = UIViewContentModeScaleAspectFill;
        _avatarImageView.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        _avatarImageView.userInteractionEnabled = YES;
        _avatarImageView.layer.cornerRadius = 22;
        _avatarImageView.layer.masksToBounds = YES;
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapAvatar)];
        [_avatarImageView addGestureRecognizer:tap];
    }
    return _avatarImageView;
}

- (UILabel *)userNameLabel {
    if(!_userNameLabel) {
        _userNameLabel = [[UILabel alloc] init];
        _userNameLabel.textColor = [UIColor whiteColor];
        _userNameLabel.font = [UIFont fontWithName:@"PingFangSC-Medium" size:18];
        _userNameLabel.textAlignment = NSTextAlignmentCenter;
        [_userNameLabel sizeToFit];
    }
    return _userNameLabel;
}

- (AlignButton *)gameHallButton {
    if (!_gameHallButton) {
        _gameHallButton = [[AlignButton alloc] init];
        _gameHallButton.alignType = AlignType_TextBottom;
        _gameHallButton.padding = 8;
        _gameHallButton.titleLabel.font = [UIFont systemFontOfSize:10];
        [_gameHallButton setTitle:TLLocalizedString(game_mall) forState:UIControlStateNormal];
        [_gameHallButton setTitleColor:[UIColor colorWithRGB:0xffffffff alpha:0.5] forState:UIControlStateNormal];
        [_gameHallButton setTitleColor:[UIColor whiteColor] forState:UIControlStateSelected];
        [_gameHallButton setImage:[UIImage imageNamed:@"icon_game_pre.png"] forState:UIControlStateNormal];
        [_gameHallButton setImage:[UIImage imageNamed:@"icon_game_sel.png"] forState:UIControlStateSelected];
        _gameHallButton.tag = ZGGameTabButtonTypeHall;
        [_gameHallButton addTarget:self action:@selector(gameHallActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
        _gameHallButton.selected = YES;
    }
    return _gameHallButton;
}

- (AlignButton *)liveHallButton {
    if (!_liveHallButton) {
        _liveHallButton = [[AlignButton alloc] init];
        _liveHallButton.alignType = AlignType_TextBottom;
        _liveHallButton.padding = 8;
        _liveHallButton.titleLabel.font = [UIFont systemFontOfSize:10];
        [_liveHallButton setTitle:TLLocalizedString(live_mall) forState:UIControlStateNormal];
        [_liveHallButton setTitleColor:[UIColor colorWithRGB:0xffffffff alpha:0.5] forState:UIControlStateNormal];
        [_liveHallButton setTitleColor:[UIColor whiteColor] forState:UIControlStateSelected];
        [_liveHallButton setImage:[UIImage imageNamed:@"icon_viseo_pre.png"] forState:UIControlStateNormal];
        [_liveHallButton setImage:[UIImage imageNamed:@"icon_viseo_sel.png"] forState:UIControlStateSelected];
        _liveHallButton.tag = ZGGameTabButtonTypeLive;
        [_liveHallButton addTarget:self action:@selector(liveHallActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _liveHallButton;
}

- (AlignButton *)liveActionBtn {
    if (!_liveActionBtn) {
        _liveActionBtn = [[AlignButton alloc] init];
        _liveActionBtn.tag = ZGGameTabButtonTypePlay;
        _liveActionBtn.alignType = AlignType_TextBottom;
        _liveActionBtn.padding = 8;
        _liveActionBtn.titleLabel.font = [UIFont systemFontOfSize:10];
        [_liveActionBtn setTitle:TLLocalizedString(start_live) forState:UIControlStateNormal];
        [_liveActionBtn setTitleColor:[UIColor colorWithRGB:0xffffffff alpha:0.5] forState:UIControlStateNormal];
        [_liveActionBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateSelected];
        [_liveActionBtn setImage:[UIImage imageNamed:@"icon_zhibo_pre.png"] forState:UIControlStateNormal];
        [_liveActionBtn setImage:[UIImage imageNamed:@"icon_zhibo_sel.png"] forState:UIControlStateSelected];
        [_liveActionBtn addTarget:self action:@selector(liveActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _liveActionBtn;
}

- (AlignButton *)cloudActionBtn {
    if (!_cloudActionBtn) {
        _cloudActionBtn = [[AlignButton alloc] init];
        _cloudActionBtn.tag = ZGGameTabButtonTypecloud;
        _cloudActionBtn.alignType = AlignType_TextBottom;
        _cloudActionBtn.padding = 8;
        _cloudActionBtn.titleLabel.font = [UIFont systemFontOfSize:10];
        [_cloudActionBtn setTitle:TLLocalizedString(cloud_game) forState:UIControlStateNormal];
        [_cloudActionBtn setTitleColor:[UIColor colorWithRGB:0xffffffff alpha:0.5] forState:UIControlStateNormal];
        [_cloudActionBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateSelected];
        [_cloudActionBtn setImage:[UIImage imageNamed:@"icon_cloud_pre.png"] forState:UIControlStateNormal];
        [_cloudActionBtn setImage:[UIImage imageNamed:@"icon_cloud_sel.png"] forState:UIControlStateSelected];
        [_cloudActionBtn addTarget:self action:@selector(cloudActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _cloudActionBtn;
}

- (AlignButton *)mineButton {
    if (!_mineButton) {
        _mineButton = [[AlignButton alloc] init];
        _mineButton.tag = ZGGameTabButtonTypeMine;
        _mineButton.alignType = AlignType_TextBottom;
        _mineButton.padding = 8;
        _mineButton.titleLabel.font = [UIFont systemFontOfSize:10];
        [_mineButton setTitleColor:[UIColor colorWithRGB:0xffffffff alpha:0.5] forState:UIControlStateNormal];
        [_mineButton setTitle:TLLocalizedString(my_info) forState:UIControlStateNormal];
        [_mineButton setTitleColor:[UIColor whiteColor] forState:UIControlStateSelected];
        [_mineButton setImage:[UIImage imageNamed:@"icon_mine_pre.png"] forState:UIControlStateNormal];
        [_mineButton setImage:[UIImage imageNamed:@"icon_mine_sel.png"] forState:UIControlStateSelected];
        [_mineButton addTarget:self action:@selector(mineActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _mineButton;
}

- (UIButton *)recordBtn {
    if (!_recordBtn) {
        _recordBtn = [[UIButton alloc] init];
        _recordBtn.hidden = YES;
        _recordBtn.layer.cornerRadius = 15;
        _recordBtn.layer.masksToBounds = YES;
        _recordBtn.backgroundColor = [UIColor colorWithRGB:0xffffffff alpha:0.1];
        [_recordBtn setTitle:TLLocalizedString(record) forState:UIControlStateNormal];
        _recordBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_recordBtn addTarget:self action:@selector(recordActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _recordBtn;
}

- (UIButton *)settingBtn {
    if (!_settingBtn) {
        _settingBtn = [[UIButton alloc] init];
        _settingBtn.hidden = YES;
        [_settingBtn setImage:[UIImage imageNamed:@"icon_setup.png"] forState:UIControlStateNormal];
        [_settingBtn addTarget:self action:@selector(settingActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _settingBtn;
}

#pragma 测试区域限制
//开始定位
- (void)startLocation {
    self.locationManager = [[CLLocationManager alloc] init];
    //判断设备是否能够进行定位
    if ([CLLocationManager locationServicesEnabled]) {
        self.locationManager.delegate = self;
        //精确度获取到米
        self.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        //设置过滤器为无
        self.locationManager.distanceFilter = kCLDistanceFilterNone;
        // 取得定位权限，有两个方法，取决于你的定位使用情况
        //一个是requestAlwaysAuthorization，一个是requestWhenInUseAuthorization
        // 这句话ios8以上版本使用。
        [self.locationManager requestWhenInUseAuthorization];
        //开始获取定位
        [self.locationManager startUpdatingLocation];
        //地理信息
        self.geoCoder = [[CLGeocoder alloc] init];
    } else {
        NSLog(@"error");
    }
}

- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations {
        NSLog(@"~~~~~~~~~~~~~~~~~~~~~~~~%lu", (unsigned long)locations.count);
        CLLocation *newLocation = locations[0];
        [[[CLGeocoder alloc]init] reverseGeocodeLocation:newLocation completionHandler:^(NSArray *placemarks, NSError *error) {
        if (error || placemarks.count == 0) {
            NSLog(@"不能确定是在中国");
        } else {

        CLPlacemark *placemark=[placemarks firstObject];

        if ([placemark.ISOcountryCode isEqualToString:@"CN"]) {

        NSLog(@"确定是在中国");

        } else {

        NSLog(@"确定不在中国");

        }

        }

        }];
    //不用的时候关闭更新位置服务，不关闭的话这个 delegate 隔一定的时间间隔就会有回调
//    [self.locationManager stopUpdatingLocation];
}

- (void)testLocation {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
//        [self startLocation];
    });
}
@end
