//
//  ZGExchangeRecordController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/4/12.
//

#import "ZGExchangeRecordController.h"

@interface ZGExchangeRecordController ()

@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *languageLabel;
@property (nonatomic, strong) UIButton *iconRightBtn;
@end

@implementation ZGExchangeRecordController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupBaseUI];
    [self setupData];
}

- (void)setupData {
    
}

- (void)setupBaseUI {
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"img_bg"]];
    self.navigationItem.title = @"Exchange record";
    [self.view addSubview:self.contentView];
    [self.contentView addSubview:self.titleLabel];
    [self.contentView addSubview:self.languageLabel];
    [self.contentView addSubview:self.iconRightBtn];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.view).offset(kTLTopStatusBarHeight + 76);
        make.left.equalTo(self.view).offset(16);
        make.right.equalTo(self.view).offset(-16);
        make.height.mas_equalTo(48);
    }];
    
    [self.titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.contentView);
        make.left.equalTo(self.contentView).offset(17);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(-14);
    }];
    
    [self.languageLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.titleLabel);
        make.right.equalTo(self.contentView).offset(-32);
    }];
    
    [self.iconRightBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.titleLabel);
        make.left.equalTo(self.languageLabel.mas_right).offset(2);
        make.width.height.mas_equalTo(20);
    }];
    
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onRightActionButtonClick:)];
    [self.contentView addGestureRecognizer:tapGR];
}

- (void)onRightActionButtonClick:(UIGestureRecognizer *)tap {
    @weakify(self)
    
}

- (UIView *)contentView {
    if (!_contentView) {
        _contentView = [UIView new];
        _contentView.layer.cornerRadius = 10;
        _contentView.layer.masksToBounds = YES;
        _contentView.backgroundColor = [UIColor colorWithHexString:@"#574979"];
    }
    return _contentView;
}

- (UILabel *)titleLabel {
    if(!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.text = TLLocalizedString(language_setting);
        _titleLabel.textColor = [UIColor whiteColor];
        _titleLabel.font = [UIFont systemFontOfSize:14];
        _titleLabel.textAlignment = NSTextAlignmentLeft;
        [_titleLabel sizeToFit];
    }
    return _titleLabel;
}

- (UILabel *)languageLabel {
    if(!_languageLabel) {
        _languageLabel = [[UILabel alloc] init];
        _languageLabel.text = TLLocalizedString(language_system);
        _languageLabel.textColor = [UIColor colorWithHexString:@"A8A4AE"];
        _languageLabel.font = [UIFont systemFontOfSize:14];
        _languageLabel.textAlignment = NSTextAlignmentLeft;
        [_languageLabel sizeToFit];
    }
    return _languageLabel;
}

- (UIButton *)iconRightBtn {
    if (!_iconRightBtn) {
        _iconRightBtn = [[UIButton alloc] init];
        _iconRightBtn.enabled = NO;
        [_iconRightBtn setImage:[UIImage imageNamed:@"icon_right.png"] forState:UIControlStateNormal];
    }
    return _iconRightBtn;
}

@end
