//
//  ZGLiveBaseViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/7/20.
//

#import "ZGLiveBaseViewController.h"
#import "MenuTableViewCell.h"
#import "ZegoRTCEngineManager.h"
#import "ChargeUtils.h"
#import "ZegoRechargeAlterView.h"
#import "ZGUserInfoRandom.h"
#import <ZegoMiniGameEngine/ZegoMiniGameEngine.h>

@interface ZGLiveBaseViewController ()<RTCEventListener,ZegoGameEngineHandler,
                                       UIGestureRecognizerDelegate,UITableViewDelegate,
                                       UITableViewDataSource>
// 是否点击
@property (nonatomic, assign) BOOL isClick;
// 是否打开音乐
@property (nonatomic, assign) BOOL isMOpen;
// 是否打开音效
@property (nonatomic, assign) BOOL isSOpen;
// 是否打开震动
@property (nonatomic, assign) BOOL isVibrate;
// 是否设置过
@property (nonatomic, assign) BOOL isSet;
// 功能菜单测试数据源
@property (nonatomic, strong) NSArray *menuZoneData;
// 测试菜单输入值
@property (nonatomic, copy)   NSString *valueStr;
// 功能菜单设置语言
@property (nonatomic, strong) UIButton *languageBtn;

@end

@implementation ZGLiveBaseViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self initData];
    // 功能测试数据
    [self initMenu];
    [self setupBaseUI];
}

- (void)initData {
    self.isPlayingGame = NO;
    self.isBack = NO;
    self.rtcManager = ZegoRTCEngineManager.sharedInstance;
    
    [ZegoMiniGameEngine.sharedEngine setGameEngineHandler:self];
    [ZegoRTCEngineManager.sharedInstance addRTCEventListener:self];
    
    self.arrangeView.arrangeData = self.viewModel.arrangeData;
    @weakify(self)
    self.arrangeView.arrangeblock = ^(NSDictionary * _Nonnull selectedData){
        @strongify(self)
        [ZegoMiniGameEngine.sharedEngine setGameLanguage:[selectedData[@"language"] longValue]];
        [self.languageBtn setTitle:selectedData[@"name"] forState:UIControlStateNormal];
    };
    
    [self.memberCountBtn setTitle:[NSString stringWithFormat: TLLocalizedString(online_member), [self.rtcManager getMemberCount]] forState:UIControlStateNormal];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_appWillBeTerminated) name:UIApplicationWillTerminateNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_refreshCoins) name:@"updateGameData" object:nil];
}

- (void)setupBaseUI {
    self.view.backgroundColor = [UIColor blackColor];
}

#pragma mark----ZegoGameEngineHandler 有些回调在子类中实现
- (void)onPlayerStateUpdate:(ZegoGamePlayerState)playerState {
    kLog(@"onPlayerStateUpdate playerEvent：%ld",playerState);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(update_player_status), playerState] withFinishBlock:nil];
}
// 金币不足回调
- (void)onChargeRequire:(NSString *)gameID {
    ZegoRechargeAlterView *alertView = [ZegoRechargeAlterView alertView];
    __weak typeof(ZegoRechargeAlterView) *weakAlertView = alertView;
    [alertView onRechargeBlock:^(NSString *gameSpin) {
        if ((int)self.viewModel.businessCoin >= (int)EXCHANGE_COIN_NUMBER && (int)self.viewModel.businessCoin > 0) {
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:EXCHANGE_COIN_NUMBER completion:^(NSInteger errorCode, long gameCoins) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    kLog(@"onReCharge errorCode：%ld, gameCoins：%ld", errorCode, gameCoins);
                    if (errorCode != 0) {
                        [MBProgressHUD showSuccess:@"onChargeRequire：游戏金币兑换失败" withFinishBlock:nil];
                    }
                    self.viewModel.businessCoin = self.viewModel.businessCoin - EXCHANGE_COIN_NUMBER;
                    kLog(@"viewModel.businessCoin：%ld", self.viewModel.businessCoin);
                    [ZegoMiniGameEngine.sharedEngine refreshGameCoin];
                    [weakAlertView updateRechargeData:[NSString stringWithFormat:@"%d", (int)self.viewModel.businessCoin] gameCoin:[NSString stringWithFormat:@"%d", (int)gameCoins]];
                });
            }];
        } else {
            [MBProgressHUD showSuccess:TLLocalizedString(business_coin_is_not_enough) withFinishBlock:nil];
        }

    }];
    [alertView show];
}

- (void)onGameError:(int)errorCode errorMessage:(nonnull NSString *)errorMessage {
    kLog(@"[onGameError] errorCode：%d errorMessage:%@", errorCode, errorMessage);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(game_running_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
}

- (void)onActionEventUpdate:(int)actionID data:(NSString *)data {
    kLog(@"[onActionEventUpdate] buttonID：%d, data：%@",  actionID, data);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onActionEventUpdate - buttonID:%d, data:%@", actionID, data] withFinishBlock:nil];
}

- (void)onGameResult:(NSString *)result {
    kLog(@"[onGameResult] result：%@", result);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onGameRoundResult - result：%@", result] withFinishBlock:nil];
}

- (ZegoGameRobotConfig *)onRobotConfigRequire:(NSString *)gameID {
    kLog(@"[onRobotConfigRequire] gameID：%@", gameID);
    ZegoGameRobotConfig *config = [[ZegoGameRobotConfig alloc] initWithRobotLevel:7 robotNickName:[ZGUserInfoRandom getRandomName] robotAvatar:[ZGUserInfoRandom getRandomAvatar] useRobotAfterTime:10 minRobotCoins:1000 maxRobotCoins:5000];
    
    return config;
}

- (void)onMicStateChange:(BOOL)isMute {
    NSLog(@"onMicStateChange mute %d", isMute);
    [self.rtcManager muteMicrophone:isMute];
}

// 听不听userIDList 的声音
- (void)onSpeakerStateChange:(NSArray<NSString *> *)userIDList isMute:(BOOL)isMute {
    NSLog(@"onSpeakerStateChange mute: %d, userIDCount:%ld", isMute, userIDList.count);
    for (NSString *userID in userIDList) {
        if(![userID isEqualToString:self.viewModel.userID]) { // 过滤掉自己
            [self.rtcManager muteSpeaker:userID mute:isMute];
            NSLog(@"onMicStateChange mute: %d, userID:%@", isMute, userID);
        }
    }
}

- (void)_appWillBeTerminated {
    if (!ZGStringIsEmpty(self.currentGameID)) {
        [self onGameQuit];
    }
}

- (void)_refreshCoins {
    // 当上个游戏退出后，快速进新的游戏。有可能兑出金币的操作执行很慢导致这时才收到刷新通知。
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
        if(chargeGameInfo != nil && chargeGameInfo.gameCoin > 0) {
            long tempCoin =  chargeGameInfo.gameCoin;
            chargeGameInfo.gameCoin = 0;
            NSLog(@"6~~~~~~~~~~~~~~~~~~~~~~~~~%ld",tempCoin);
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:tempCoin completion:^(NSInteger errorCode, long gameCoins) {
                NSLog(@"7~~~~~~~~~~~~~~~~~~~~~~~~~%ld",gameCoins);
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                } else {
                    [[ZegoMiniGameEngine sharedEngine] refreshGameCoin];
                }
            }];
        }
    }
}

/**
 * 在游戏开始时把业务金币兑换成游戏金币，保证除了游戏中，其他都是自己在维护业务金币
 */
- (void)onGameStart {
    self.loadingLabel.hidden = NO;
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
        if (!ZGObjectIsEmpty(chargeGameInfo) && chargeGameInfo.gameCoin != 0) {
            long tempCoin =  chargeGameInfo.gameCoin;
            chargeGameInfo.gameCoin = 0;
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:tempCoin completion:^(NSInteger errorCode, long gameCoin) {
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                    chargeGameInfo.gameCoin = tempCoin;
                } else {
//                    chargeGameInfo.gameCoin = 0;
                    self.viewModel.refreshGameCoin = self.viewModel.refreshGameCoin + 1;
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(exchange_x_game_coin), (int)gameCoin] withFinishBlock:nil];
                }
            }];
        }
    } else {
        [MBProgressHUD showSuccess:TLLocalizedString(game_id_is_empty) withFinishBlock:nil];
    }
}

/**
 * 在游戏退出时把游戏金币兑换成业务金币，保证除了游戏中，其他都是自己在维护业务金币
 */
- (void)onGameQuit {
    NSString *gameID = self.currentGameID;
    ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
    if (!ZGObjectIsEmpty(chargeGameInfo)) {
        [ChargeUtils queryGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID completion:^(NSInteger errorCode, long gameCoin) {
            if (errorCode != 0) {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(query_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            } else {
                if (gameCoin != 0) {
                    [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:(0 - gameCoin) completion:^(NSInteger errorCode2, long obj) {
                        if (errorCode2 != 0) {
                            [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode2]] withFinishBlock:nil];
                        } else {
                            if(chargeGameInfo.gameCoin == 0) {
                                chargeGameInfo.gameCoin = gameCoin;
                                self.viewModel.refreshGameCoin = self.viewModel.refreshGameCoin + 1;
                                [[NSNotificationCenter defaultCenter] postNotificationName:@"updateGameData" object:nil];
                            }
                        }
                    }];
                }
            }
        }];
    }
}

#pragma 功能菜单测试
- (void)initMenu {
    @weakify(self)
    self.menuZoneData = @[
        @{@"name" : TLLocalizedString(get_music_switch),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameMusicSwitch:^(int errorCode, id  _Nullable retValue) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"当前游戏音乐开关：%@", [retValue intValue] == 1 ? @"YES" : @"NO"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏音乐开关失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
                  
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(set_music_switch),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine muteGameMusic:!self.isMOpen];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              } else {
                  self.isMOpen = !self.isMOpen;
              }
          }
        },
        
        @{@"name" : TLLocalizedString(get_music_volume),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameMusicVolume:^(int errorCode, id  _Nullable retValue) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"当前游戏音乐音量：%d", [retValue intValue]] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏音乐音量失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(set_music_volume),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine setGameMusicVolume:[self.valueStr intValue]];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(get_vibrate_switch),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameVibrateSwitch:^(int errorCode, id  _Nullable retValue) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"当前游戏震动开关：%@", [retValue intValue] == 1 ? @"YES" : @"NO"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏震动开关失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(set_vibrate_switch),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine enableGameVibrate:!self.isVibrate];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              } else {
                  self.isVibrate = !self.isVibrate;
              }
          }
        },
        
        @{@"name" : @"获取游戏音效开关",
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameSoundEffectSwitch:^(int errorCode, id  _Nullable retValue) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"当前游戏音效开关：%@", [retValue intValue] == 1 ? @"YES" : @"NO"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏音效开关失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(set_sound_effect_switch),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine muteGameSoundEffect:!self.isSOpen];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              } else {
                  self.isSOpen = !self.isSOpen;
              }
          }
        },
        
        @{@"name" : TLLocalizedString(get_sound_effect_volume),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine
                         getGameSoundEffectVolume:^(int errorCode, id  _Nullable retValue) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"当前游戏音效音量：%d", [retValue intValue]] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏音效音量失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(set_sound_effect_volume),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine setGameSoundEffectVolume:[self.valueStr intValue]];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(get_game_components),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameComponentList:^(int errorCode, NSDictionary *object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"修改游戏内控件：%@", [object modelToJSONString]] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:errorCode] withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        @{@"name" : TLLocalizedString(show_game_components),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine setGameComponent:self.valueStr isVisible:YES callBack:^(int errorCode, id  _Nullable object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"显示游戏控件：%@", [object intValue] == 1 ? @"执行成功" : @"执行失败"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:errorCode] withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        @{@"name" : TLLocalizedString(hide_game_components),
          @"block" : ^{
              @strongify(self)
              int ret = [ZegoMiniGameEngine.sharedEngine setGameComponent:self.valueStr isVisible:NO callBack:^(int errorCode, id  _Nullable object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"隐藏游戏控件：%@", [object intValue] == 1 ? @"执行成功" : @"执行失败"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:errorCode] withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        @{@"name" : TLLocalizedString(get_game_functions),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine getGameFunctionList:^(int errorCode, NSDictionary *object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"获取游戏可执行方法列表：%@", [object modelToJSONString]] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:errorCode] withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        @{@"name" : TLLocalizedString(show_run_game_function),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine runGameFunction:@"showGameSetup" callBack:^(int errorCode, id  _Nullable object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"显示游戏设置界面：%@", [object intValue] == 1 ? @"执行成功" : @"执行失败"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"显示游戏设置界面失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        @{@"name" : TLLocalizedString(hide_run_game_function),
          @"block" : ^{
              int ret = [ZegoMiniGameEngine.sharedEngine runGameFunction:self.valueStr callBack:^(int errorCode, id  _Nullable object) {
                  if (errorCode == 0) {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"%@", [object intValue] == 1 ? @"执行成功" : @"执行失败"] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"失败：%@", [ZegoConstants getErrorMsg:errorCode]]  withFinishBlock:nil];
                  }
              }];
              if (ret != 0) {
                  [MBProgressHUD showSuccess:[ZegoConstants getErrorMsg:ret] withFinishBlock:nil];
              }
          }
        },
        
        @{@"name" : TLLocalizedString(game_width_height),
          @"block" : ^{
              CGFloat width = self.view.frame.size.width;
              CGFloat height = [self.wdStr intValue];
              [self updateWebviewFrame:width height:height];
          }
        },
        
    ];
}

- (void)updateWebviewFrame:(CGFloat)width height:(CGFloat)height {
    
}

#pragma mark - TableViewDelegate & DataSource
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    MenuTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"MenuTableViewCell" forIndexPath:indexPath];
    if ( indexPath.row == 11 || indexPath.row == 12 || indexPath.row == 14 || indexPath.row == 15) {
        if (indexPath.row == 11 || indexPath.row == 12) {
            cell.deValue = @"gameSetup";
        } else if (indexPath.row == 14) {
            cell.deValue = @"showGameSetup";
        } else {
            cell.deValue = @"hideGameSetup";
        }
     
    } else {
        cell.deValue = @"50";
    }
    if (!cell) {
        cell = [[MenuTableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"MenuTableViewCell"];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    if (indexPath.row == 3 || indexPath.row == 9 || indexPath.row == 11 || indexPath.row == 12 || indexPath.row == 14  || indexPath.row == 15 || indexPath.row == 16) {
        [cell setMenuDictionary:self.menuZoneData[indexPath.row] isShow:YES];
    } else {
        [cell setMenuDictionary:self.menuZoneData[indexPath.row] isShow:NO];
    }
    @weakify(self)
    if(indexPath.row == 16) {
        cell.selectedBlock = ^(NSString *value) {
            @strongify(self)
            self.isSet = YES;
            self.wdStr = value;
        };
    } else {
        cell.selectedBlock = ^(NSString *value) {
            @strongify(self)
            self.isSet = YES;
            self.valueStr = value;
        };
    }
    
    return cell;
}

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.menuZoneData.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (!self.isSet) {
        if (indexPath.row == 11 || indexPath.row == 12 || indexPath.row == 14 || indexPath.row == 15) {
            if (indexPath.row == 11|| indexPath.row == 12) {
                self.valueStr = @"gameSetup";
            } else if (indexPath.row == 14) {
                self.valueStr = @"showGameSetup";
            } else {
                self.valueStr = @"hideGameSetup";
            }
         
        } else {
            self.valueStr = @"50";
        }
    }
    NSDictionary *dict = self.menuZoneData[indexPath.row];
    dispatch_block_t block = dict[@"block"];
    if (block) {
        block();
    }
}

//返回每组头部view
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    UIView *headerView = [[UIView alloc] init];
    headerView.backgroundColor = [UIColor colorWithRGB:0xfff1f1f1];
    [headerView addSubview:self.languageBtn];
    return headerView;
}

// 返回组头部view的高度
- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 50;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 50;
}

- (void)exitKeyboard {
    [self.menuZoneView endEditing:YES];
}

- (void)onClickMenuZoneAction:(UIButton *)sender {
    self.menuZoneView.hidden = !self.menuZoneView.hidden;
    self.isSet = NO;
    [self exitKeyboard];
}

- (void)onClickSelectedLanguageAction:(UIButton *)sender {
    self.isClick = !self.isClick;
    self.arrangeView.hidden = self.isClick;
}

// 右上角关闭按钮点（X）击事件
- (void)onClickCloseAction:(UIGestureRecognizer *)tap {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self.navigationController popViewControllerAnimated:YES];
    });
}

#pragma mark----UIGestureRecognizerDelegate

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch{
    // 获取点击视图的类型
    NSString * touchClass = NSStringFromClass([touch.view class]);
    // 判断点击视图的类型是不是UITableView的cell类型
    if ([touchClass isEqualToString:@"UITableViewCellContentView"]) {
        return false;
    }else{
        return true;
    }
}

#pragma mark----getter
- (UIView *)menuZoneView {
    if (!_menuZoneView) {
        _menuZoneView = [[UIView alloc] init];
//        _menuZoneView.backgroundColor = [UIColor colorWithRGB:0xffffffff alpha:0.3];
        _menuZoneView.hidden = YES;
        UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(exitKeyboard)];
        tapGR.delegate = self;
        [_menuZoneView addGestureRecognizer:tapGR];
    }
    return _menuZoneView;
}

- (UITableView *)menuZoneTable {
    if (!_menuZoneTable) {
        _menuZoneTable = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, 0, 0)];
        _menuZoneTable.delegate = self;
        _menuZoneTable.dataSource = self;
        [_menuZoneTable registerClass:[MenuTableViewCell class] forCellReuseIdentifier:@"MenuTableViewCell"];
        _menuZoneTable.backgroundColor = [UIColor whiteColor];
        if (@available(iOS 15.0, *)) {
            [UITableView appearance].sectionHeaderTopPadding = 0;
        }
    }
    return _menuZoneTable;
}

- (UIButton *)menuZoneBtn {
    if (!_menuZoneBtn) {
        _menuZoneBtn = [UIButton actionButtonWithTitle:TLLocalizedString(show_features)];
        [_menuZoneBtn addTarget:self action:@selector(onClickMenuZoneAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _menuZoneBtn;
}

- (AlignButton *)memberCountBtn {
    if (!_memberCountBtn) {
        _memberCountBtn = [[AlignButton alloc] init];
        _memberCountBtn.alignType = AlignType_TextRight;
        _memberCountBtn.padding = 3;
        _memberCountBtn.userInteractionEnabled = NO;
        _memberCountBtn.backgroundColor = [UIColor colorWithRGB:0x00000000 alpha:0.5];
        _memberCountBtn.titleLabel.font = [UIFont systemFontOfSize:10];
        _memberCountBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
        _memberCountBtn.layer.cornerRadius =  14;
        _memberCountBtn.layer.masksToBounds = YES;;
        [_memberCountBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateReserved];
        [_memberCountBtn setImage:[UIImage imageNamed:@"icon_omline"] forState:UIControlStateNormal];
    }
    return _memberCountBtn;
}

- (UIButton *)languageBtn {
    if (!_languageBtn) {
        _languageBtn = [[UIButton alloc] init];
        _languageBtn.frame = CGRectMake(0, 0, 200, 50);
        [_languageBtn setTitle:@"简体中文" forState:UIControlStateNormal];
        [_languageBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        _languageBtn.titleLabel.font =  BOLD_FONT(20);
        _languageBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
        [_languageBtn addTarget:self action:@selector(onClickSelectedLanguageAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _languageBtn;
}

- (TLArrangePopView *)arrangeView {
    if (!_arrangeView) {
        _arrangeView = [[TLArrangePopView alloc] initWithFrame:self.view.bounds
                                                 showViewFrame:CGRectMake(80,
                                                                          TOP_BAR_HEIGHT + 105,
                                                                          110,
                                                                          200)
                                                   clickHidden:YES];
        _arrangeView.userInteractionEnabled = YES;
        _arrangeView.hidden = YES;
        _isClick = YES;
    }
    return _arrangeView;
}

- (UILabel *)loadingLabel {
    if (!_loadingLabel) {
        _loadingLabel = [[UILabel alloc] init];
        _loadingLabel.textColor = [UIColor whiteColor];
        _loadingLabel.hidden = YES;
        _loadingLabel.font = BOLD_FONT(20);
        _loadingLabel.text = TLLocalizedString(loading_game);
        _loadingLabel.textAlignment = NSTextAlignmentCenter;
        [_loadingLabel sizeToFit];
    }
    return _loadingLabel;
}

- (UIImageView *)closeImageView{
    if (!_closeImageView) {
        _closeImageView = [[UIImageView alloc] init];
        _closeImageView.userInteractionEnabled = YES;
        _closeImageView.image = [UIImage imageNamed:@"icon_close"];
        _closeImageView.contentMode = UIViewContentModeCenter;
        _closeImageView.backgroundColor = [UIColor colorWithRGB:0x00000000 alpha:0.5];
        _closeImageView.layer.cornerRadius = 13 ;
        _closeImageView.layer.masksToBounds = YES;
        UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onClickCloseAction:)];
        [_closeImageView addGestureRecognizer:tapGR];
        
    }
    return _closeImageView;
}

@end
