//
//  ZGPlayGameViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/2/16.
//

#import "ZGPlayGameViewController.h"
#import "ZegoRechargeAlterView.h"
#import "ChargeUtils.h"
#import "ZGUserInfoRandom.h"

@interface ZGPlayGameViewController ()<ZegoGameEngineHandler,ZegoEventHandler>

// 游戏加载中。。。label
@property (nonatomic, strong) UILabel *loadingLabel;
// 关闭游戏按钮
@property (nonatomic, strong) UIButton *closeGameBtn;
// 是否要退出
@property (nonatomic, assign) BOOL isBack;
@end

@implementation ZGPlayGameViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setInit];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleDeviceOrientationChange) name:UIDeviceOrientationDidChangeNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    self.tabBarController.hidesBottomBarWhenPushed = YES;
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.isBack = YES;
    if (!ZGStringIsEmpty(self.currentGameID)) {
        [ZegoMiniGameEngine.sharedEngine unloadGame:NO];
        // 这里需执行onGameQuit，不能等onUnloadGame回调
        [self onGameQuit];
    }
}

- (void)setInit {
    self.isBack = NO;
    self.view.backgroundColor = [UIColor blackColor];
    [self.view addSubview:self.loadingLabel];
    [self.view addSubview:self.closeGameBtn];
    [self.loadingLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.centerY.equalTo(self.view);
    }];
    
    [self.closeGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.view).offset(-5);
        make.bottom.equalTo(self.view).offset(-kTLBottomStatusBarHeight);
        make.height.mas_equalTo(36);
        make.width.mas_equalTo(65);
    }];
    
    [ZegoMiniGameEngine.sharedEngine setGameEngineHandler:self];
    [ZegoMiniGameEngine.sharedEngine setGameContainer:self.view];
    
    // 加载游戏
    NSDictionary *gameConfig = @{
        @"roomID" : self.viewModel.roomID? :@"",
        @"customPlaySound" : @(false),
        @"minGameCoin" : @(200),
        @"useRobot" : @(true),
        @"specificConfig" : @{@"paddingTop" : @"40",
                              @"paddingBottom" : @"0",
//                              @"buttonSelect" : @(0),
//                              @"showShopButton" : @(0),
//                              @"showLuckyCoin" : @(0),
//                              @"needQuitEvent" : @(1),
        }
    };
    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                     gameMode:ZegoGameModeMallGame
                                   gameConfig:gameConfig
                                     callBack:^(NSInteger errorCode) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (ZegoMiniGameEngineSuccess == errorCode) {
                [self onGameStart];
                [self.view bringSubviewToFront:self.loadingLabel];
                [self.view bringSubviewToFront:self.closeGameBtn];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        });
    }];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_refreshCoins) name:@"updateGameData" object:nil];
}

- (void)_refreshCoins{
    // 当上个游戏退出后，快速进新的游戏。有可能兑出金币的操作执行很慢导致这时才收到刷新通知。
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
        if(chargeGameInfo != nil && chargeGameInfo.gameCoin > 0) {
            long tempCoin =  chargeGameInfo.gameCoin;
            chargeGameInfo.gameCoin = 0;
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:tempCoin completion:^(NSInteger errorCode, long gameCoins) {
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                } else {
                    [[ZegoMiniGameEngine sharedEngine] refreshGameCoin];
                }
            }];
        }
    }
}

/**
 * 在游戏开始时把业务金币兑换成游戏金币，保证除了游戏中，其他都是自己在维护业务金币
 */
- (void)onGameStart {
    self.loadingLabel.hidden = NO;
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
        if (!ZGObjectIsEmpty(chargeGameInfo) && chargeGameInfo.gameCoin != 0) {
            long  tempCoin = chargeGameInfo.gameCoin;
            chargeGameInfo.gameCoin = 0;
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:tempCoin completion:^(NSInteger errorCode, long gameCoins) {
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                    chargeGameInfo.gameCoin = tempCoin;
                } else {
//                    chargeGameInfo.gameCoin = 0;
                    self.viewModel.refreshGameCoin = self.viewModel.refreshGameCoin + 1;
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(exchange_x_game_coin), (int)gameCoins] withFinishBlock:nil];
                }
            }];
        }

    } else {
        [MBProgressHUD showSuccess:TLLocalizedString(game_id_is_empty) withFinishBlock:nil];
    }
}

/**
 * 在游戏退出时把游戏金币兑换成业务金币，保证除了游戏中，其他都是自己在维护业务金币
 */
- (void)onGameQuit {
    NSString *gameID = self.currentGameID;
    ChargeGameInfo *chargeGameInfo = [self.viewModel getChargeGameInfoWithGameID:gameID];
    if (!ZGObjectIsEmpty(chargeGameInfo)) {
        [ChargeUtils queryGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID completion:^(NSInteger errorCode, long gameCoin) {
            if (errorCode != 0) {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(query_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            } else {
                if (gameCoin != 0) {
//                    gameCoin = gameCoin - 1000;
                    [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:(0 - gameCoin) completion:^(NSInteger errorCode2, long obj) {
                        if (errorCode2 != 0) {
                            [MBProgressHUD showSuccess:[NSString stringWithFormat: TLLocalizedString(exchange_game_coin_error), [ZegoConstants getErrorMsg:(int)errorCode2]] withFinishBlock:nil];
                        } else {
                            if(chargeGameInfo.gameCoin == 0) {
                                chargeGameInfo.gameCoin = gameCoin;
                                self.viewModel.refreshGameCoin = self.viewModel.refreshGameCoin + 1;
                                [[NSNotificationCenter defaultCenter] postNotificationName:@"updateGameData" object:nil];
                            }
                            
                        }
                    }];
                }
            }
        }];
    }
}


#pragma mark - action

- (void)onClickCloseGameAction:(UIButton *)sender {
    if (!ZGStringIsEmpty(self.currentGameID)) {
        [ZegoMiniGameEngine.sharedEngine unloadGame:NO];
        [self onGameQuit];
        self.currentGameID = @"";
    }
}

#pragma mark----ZegoGameEngineHandler

- (void)onTokenWillExpire {
    NSLog(@"onTokenWillExpire .......");
//    [ZegoMiniGameEngine.sharedEngine updateToken:@"您的token"];
}

- (void)onGameLoadStateUpdate:(ZegoGameLoadState)gameLoadState {
    if (gameLoadState == ZegoGameLoaded || gameLoadState == ZegoGameLoading) {
        self.loadingLabel.hidden = YES;
    }
}

- (void)onGameStateUpdate:(ZegoGameState)gameState {
    kLog(@"onGameStateUpdate：%ld", gameState);
}

- (void)onGameStateUpdate:(ZegoGameState)gameState reasonCode:(int)reasonCode {
    kLog(@"onGameStateUpdate：%ld reasonCode：%d", gameState,reasonCode);
}

- (void)onPlayerStateUpdate:(ZegoGamePlayerState)playerState {
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(update_player_status), playerState] withFinishBlock:nil];
}

- (void)onChargeRequire:(NSString *)gameID {
    ZegoRechargeAlterView *alertView = [ZegoRechargeAlterView alertView];
    alertView.goldCoinsAmountLabel.text = [NSString stringWithFormat:@"%d", (int)self.viewModel.businessCoin];
    __weak typeof(ZegoRechargeAlterView) *weakAlertView = alertView;
    [alertView onRechargeBlock:^(NSString *gameSpin) {
        if ((int)self.viewModel.businessCoin >= (int)EXCHANGE_COIN_NUMBER && (int)self.viewModel.businessCoin > 0) {
            [ChargeUtils exchangeGameCoinWithUserID:self.viewModel.userID appID:[KeyCenter appID] gameID:gameID gameCoins:EXCHANGE_COIN_NUMBER completion:^(NSInteger errorCode, long gameCoins) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    kLog(@"onReCharge errorCode：%ld, gameCoins：%ld", errorCode, gameCoins);
                    self.viewModel.businessCoin = self.viewModel.businessCoin - EXCHANGE_COIN_NUMBER;
                    kLog(@"viewModel.businessCoin：%ld", self.viewModel.businessCoin);
                    [ZegoMiniGameEngine.sharedEngine refreshGameCoin];
                    [weakAlertView updateRechargeData:[NSString stringWithFormat:@"%d", (int)self.viewModel.businessCoin] gameCoin:[NSString stringWithFormat:@"%d", (int)gameCoins]];
                });
            }];
        } else {
            [MBProgressHUD showSuccess:TLLocalizedString(business_coin_is_not_enough) withFinishBlock:nil];
        }
    }];
    [alertView show];
}

- (void)onUnloaded:(NSString *)gameID {
    if (!self.isBack) {
        [self onGameQuit];
    }
    self.currentGameID = @"";
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)onGameSoundPlay:(NSString *)name isPlay:(BOOL)isPlay url:(NSString *)url isLoop:(BOOL)isLoop volume:(int)volume {
    kLog(@"[onGameSoundVolumeChange] name：%@, isPlay：%d, url：%@, isLoop：%d, volume：%d", name, isPlay, url, isLoop, volume);
}

- (void)onGameSoundVolumeChange:(NSString *)name volume:(int)volume {
    kLog(@"[onGameSoundVolumeChange] name：%@, volume：%d",  name, volume);
}

- (void)onGameError:(int)errorCode errorMessage:(nonnull NSString *)errorMessage {
    kLog(@"[onGameError] errorCode：%d", errorCode);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(game_running_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
}

- (void)onActionEventUpdate:(int)actionID data:(NSString *)data {
    kLog(@"[onActionEventUpdate] buttonID：%d, data：%@",  actionID, data);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onActionEventUpdate - buttonID:%d, data:%@", actionID, data] withFinishBlock:nil];
}

- (void)onGameResult:(NSString *)result {
    kLog(@"[onGameResult] result：%@", result);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onGameRoundResult - result：%@", result] withFinishBlock:nil];
}

- (ZegoGameRobotConfig *)onRobotConfigRequire:(NSString *)gameID {
    kLog(@"[onRobotConfigRequire] gameID：%@", gameID);
    ZegoGameRobotConfig *config = [[ZegoGameRobotConfig alloc] initWithRobotLevel:7 robotNickName:[ZGUserInfoRandom getRandomName] robotAvatar:[ZGUserInfoRandom getRandomAvatar] useRobotAfterTime:10 minRobotCoins:1000 maxRobotCoins:5000];

    return config;
}


- (void)onGameActionRequest:(int)actionID data:(NSString *)data callBack:(ZegoGameActionCallback *)callBack {

}

- (void)handleDeviceOrientationChange {
    if ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [UIApplication sharedApplication].statusBarOrientation == UIDeviceOrientationPortraitUpsideDown) {
            // 竖屏
            self.tabBarController.tabBar.hidden = YES;
        
        } else {
            // 横屏
            self.tabBarController.tabBar.hidden = YES;
        }
}

#pragma mark - getter

- (UILabel *)loadingLabel {
    if (!_loadingLabel) {
        _loadingLabel = [[UILabel alloc] init];
        _loadingLabel.textColor = [UIColor whiteColor];
        _loadingLabel.hidden = YES;
        _loadingLabel.font = BOLD_FONT(20);
        _loadingLabel.text = TLLocalizedString(loading_game);
        _loadingLabel.textAlignment = NSTextAlignmentCenter;
        [_loadingLabel sizeToFit];
    }
    return _loadingLabel;
}

- (UIButton *)closeGameBtn {
    if (!_closeGameBtn) {
        _closeGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(close_game)];
        _closeGameBtn.layer.cornerRadius = 18;
        _closeGameBtn.layer.masksToBounds = YES;
        _closeGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_closeGameBtn addTarget:self action:@selector(onClickCloseGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _closeGameBtn;
}

- (void)dealloc {
    NSLog(@"");
}

@end
