//
//  ZGSelectLanguageController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/3/20.
//

#import "ZGSelectLanguageController.h"
#import "LanguageTableViewCell.h"
#import "NSBundle+CLLanguage.h"
#import "TLLanguageManager.h"
#import "ViewController.h"
#import "AppDelegate.h"
#import "MQTabbarController.h"

@interface ZGSelectLanguageController ()<UITableViewDelegate, UITableViewDataSource>

@property (nonatomic, strong) UITableView *gameTableView;
@property (nonatomic, strong) NSMutableArray *dataArray;
@property (nonatomic, assign) NSInteger selectIndex;
@end

@implementation ZGSelectLanguageController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupBaseUI];
    [self initData];
}


- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    LanguageModel *model = self.dataArray[self.selectIndex];
    self.selectBlock(self.selectIndex, model.title);
}

- (void)setupBaseUI {
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"img_bg"]];
    [self.view addSubview:self.gameTableView];
    [self.gameTableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view).offset(14);
        make.right.equalTo(self.view).offset(-14);
        make.top.equalTo(self.view).offset(kTLTopStatusBarHeight + 76);
//        make.height.mas_equalTo(144);
        make.bottom.mas_equalTo(self.view.mas_bottom).offset(-15);
    }];
}

- (void)initData {
    self.selectIndex = self.defaultIndex;
    NSArray *languageArray = @[
        @{
            @"title": TLLocalizedString(language_system),
            @"language": @(ZegoGameLanguageZHHans),
            @"isSelected": @(YES),
        },
        @{
            @"title": TLLocalizedString(language_zh),
            @"language": @(ZegoGameLanguageZHHans),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_en),
            @"language": @(ZegoGameLanguageEN),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_pt),
            @"language": @(ZegoGameLanguagePT),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_es),
            @"language": @(ZegoGameLanguageES),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_ru),
            @"language": @(ZegoGameLanguageRU),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_ar),
            @"language": @(ZegoGameLanguageAR),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_de),
            @"language": @(ZegoGameLanguageDE),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_fr),
            @"language": @(ZegoGameLanguageFR),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_hi),
            @"language": @(ZegoGameLanguageHI),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_it),
            @"language": @(ZegoGameLanguageIT),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_ja),
            @"language": @(ZegoGameLanguageJA),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_ko),
            @"language": @(ZegoGameLanguageKO),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_th),
            @"language": @(ZegoGameLanguageTH),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_tr),
            @"language": @(ZegoGameLanguageTR),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_vi),
            @"language": @(ZegoGameLanguageVI),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_language_zh_TW),
            @"language": @(ZegoGameLanguageZHHant),
            @"isSelected": @(NO),
        },
        @{
            @"title": TLLocalizedString(language_ms),
            @"language": @(ZegoGameLanguageMS),
            @"isSelected": @(NO),
        }
    ];
    for (NSDictionary *dict in languageArray) {
        LanguageModel *model = [[LanguageModel alloc] init];
        model.title = dict[@"title"];
        model.language = (ZegoGameLanguage)[dict[@"language"] integerValue];
        model.isSelected = [dict[@"isSelected"] boolValue];
        [self.dataArray addObject:model];
    }
}

#pragma mark - TableViewDelegate & DataSource

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    LanguageTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"LanguageTableViewCell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.backgroundColor = [UIColor colorWithHexString:@"574979"];
    LanguageModel *model = self.dataArray[indexPath.row];
    if (self.selectIndex == indexPath.row) {
        model.isSelected = YES;
    } else {
        model.isSelected = NO;
    }
    [cell setLanguageModel:model];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 48;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    self.selectIndex = indexPath.row;
    [tableView reloadData];
    LanguageModel *model = self.dataArray[indexPath.row];
    [ZegoMiniGameEngine.sharedEngine setGameLanguage:model.language];
    if (indexPath.row == 0) {
        [TLLanguageManager setUserLanguage:nil];
    } else if (indexPath.row == 1) {
        [TLLanguageManager setUserLanguage:@"zh-Hans"];
    } else {
        [TLLanguageManager setUserLanguage:@"en"];
    }
    [ZegoMiniGameEngine.sharedEngine unInit];
    [self refreshRootViewController];
    
}

- (void)refreshRootViewController {
    //创建新的根控制器
    MQTabbarController *tabbarController = [[MQTabbarController alloc] init];
    tabbarController.hidesBottomBarWhenPushed = YES;
    tabbarController.selectedIndex = 0;
    [tabbarController.tabBar removeAllSubviews];
    [tabbarController.tabBar removeFromSuperview];
    
    UINavigationController *navigationController = tabbarController.selectedViewController;
    NSMutableArray *viewControllers = navigationController.viewControllers.mutableCopy;
    //取出我的页面，提前加载，解决返回按钮不变化
    ViewController *viewController = (ViewController *)[viewControllers firstObject];
    [viewController loadViewIfNeeded];
    //新建设置语言页面
//    ZGSelectLanguageController *languageController = [[ZGSelectLanguageController alloc] init];
//    languageController.hidesBottomBarWhenPushed = YES;
//    [viewControllers addObject:languageController];
    //解决奇怪的动画bug。
    dispatch_async(dispatch_get_main_queue(), ^{
        [UIApplication sharedApplication].keyWindow.rootViewController = tabbarController;
        navigationController.viewControllers = viewControllers;
        NSLog(@"已切换到语言 %@", [NSBundle currentLanguage]);
    });
}

- (UITableView *)gameTableView {
    if (!_gameTableView) {
        _gameTableView = [[UITableView alloc] init];
        _gameTableView.delegate = self;
        _gameTableView.dataSource = self;
        _gameTableView.layer.cornerRadius = 10.0f;
        _gameTableView.layer.masksToBounds = YES;
        [_gameTableView registerClass:[LanguageTableViewCell class] forCellReuseIdentifier:@"LanguageTableViewCell"];
        _gameTableView.backgroundColor = [UIColor colorWithHexString:@"574979"];
    }
    return _gameTableView;
}

- (NSMutableArray *)dataArray {
    if (!_dataArray) {
        _dataArray = [NSMutableArray array];
    }
    return _dataArray;
}

@end
