//
//  ZGStreamLiveViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/8.
//

#import "ZGStreamLiveViewController.h"
#import "ZegoRTCEngineManager.h"
#import <ZegoMiniGameEngine/ZegoMiniGameEngine.h>
#import "ZegoPlayGameAlterView.h"
#import "ZegoRechargeAlterView.h"
#import "ZegoLiveRenderViewRelation.h"
#import "ZegoInviteAlterView.h"
#import "ZegoAudioPlayer.h"
#import "RoomGameInfo.h"
#import "KeyCenter.h"
#import "AlignButton.h"
#import "ZegoSelectGamePicker.h"
#import "ZGUserInfoRandom.h"

@interface ZGStreamLiveViewController () <RTCEventListener, ZegoGameEngineHandler>

@property (nonatomic, strong) RoomGameInfo *roomGameInfo;
@property (nonatomic, strong) UIButton *playGameBtn;
@property (nonatomic, strong) UIButton *closeGameBtn;
@property (nonatomic, strong) UIButton *leaveGameBtn;
@property (nonatomic, strong) UIButton *switchRoomBtn;

@property (nonatomic, strong) AlignButton *joinLiveBtn;
@property (nonatomic, strong) UIButton *playShowGameBtn;
@property (nonatomic, strong) UIView *gameView;
@property (nonatomic, strong) NSMutableDictionary *gameMap;
@property (nonatomic, strong) NSMutableArray <ZegoLiveRenderViewRelation *> *streamViewList;
@property (nonatomic, strong) ZegoLiveRenderViewRelation *currentRelation;
@property (nonatomic, strong) ZegoAudioPlayer *audioPlayer;
@property (nonatomic, assign) BOOL isJoinLive;
@end

@implementation ZGStreamLiveViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupBaseUI];
    [self joinLiveRoom];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.isBack = YES;
    if (self.viewModel.isHost) {
        [self.rtcManager stopPublishStream];
        if (!ZGStringIsEmpty(self.currentGameID)) {
            [ZegoMiniGameEngine.sharedEngine unloadGame:true];
            [self onGameQuit];
            [self updateButtonVisibility];
            [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:@""];
        }
    } else {
        if (!ZGStringIsEmpty(self.currentGameID)) {
            [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
            [self onGameQuit];
            [self updateButtonVisibility];
        }
    }
    if (self.audioPlayer != nil) {
        [self.audioPlayer destroy];
        self.audioPlayer = nil;
    };
    [self.rtcManager logoutRoom];
}

- (void)setupBaseUI {
    [super setupBaseUI];
    [self.view addSubview:self.gameView];
    self.navigationItem.title = [NSString stringWithFormat:@"%@  %@", self.viewModel.userName, self.viewModel.roomID];
    self.tabBarController.hidesBottomBarWhenPushed = YES;
    [self.gameView addSubview:self.playGameBtn];
    [self.gameView addSubview:self.memberCountBtn];
    [self.gameView addSubview:self.closeImageView];
    [self.gameView addSubview:self.loadingLabel];
    [self.gameView addSubview:self.menuZoneBtn];
    [self.gameView addSubview:self.menuZoneView];
    [self.menuZoneView addSubview:self.menuZoneTable];
    [ZegoKeyWindow addSubview:self.arrangeView];
    [self.gameView addSubview:self.closeGameBtn];
    [self.gameView addSubview:self.leaveGameBtn];
    [self.gameView addSubview:self.playShowGameBtn];
    [self.gameView addSubview:self.switchRoomBtn];
    [self.gameView addSubview:self.joinLiveBtn];
    [self updateButtonVisibility];
    
    [self.gameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.view);
    }];
    
    [self.memberCountBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.gameView).offset(TOP_BAR_HEIGHT + 6);
        make.height.mas_equalTo(28);
        make.width.mas_equalTo(83);
    }];
    
    [self.closeImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.memberCountBtn.mas_right).offset(8);
        make.right.mas_equalTo(self.gameView).offset(-8);
        make.centerY.equalTo(self.memberCountBtn.mas_centerY);
        make.height.mas_equalTo(26);
        make.width.mas_equalTo(26);
    }];
    
    [self.loadingLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.gameView);
        make.centerY.equalTo(self.gameView);
    }];
    
    [self.playGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.gameView).offset(-30);
        make.bottom.equalTo(self.gameView).offset(-kTLBottomStatusBarHeight);
        make.height.mas_equalTo(36);
        make.width.mas_equalTo([NSBundle isChineseLanguage] ? 77 : 107);
    }];
    
    [self.playShowGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    
    [self.closeGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    [self.leaveGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    [self.switchRoomBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.gameView).offset(30);
        make.bottom.height.width.equalTo(self.playGameBtn);
    }];
    
    [self.joinLiveBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.playShowGameBtn.mas_left).offset(-20);
        make.bottom.equalTo(self.gameView).offset(-kTLBottomStatusBarHeight);
        make.width.mas_equalTo([NSBundle isChineseLanguage] ? 69 : 112);
        make.height.mas_equalTo(36);
    }];
    
    [self.menuZoneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(20);
        make.top.equalTo(self.gameView).offset(TOP_BAR_HEIGHT + 10);
        make.height.equalTo(self.playGameBtn);
        make.width.mas_equalTo(120);
    }];
    
    [self.menuZoneView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.menuZoneBtn).offset(5);
        make.top.mas_equalTo(self.menuZoneBtn.mas_bottom).offset(10);
        make.height.mas_equalTo(ZEGO_HEIGHT - 190 - TOP_BAR_HEIGHT);
        make.width.mas_equalTo(kScreenWidth - 10);
    }];
    
    [self.menuZoneTable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.equalTo(self.menuZoneView);
        make.height.mas_equalTo(330);
        make.width.mas_equalTo(258);
    }];
}

- (void)joinLiveRoom {
    NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
    if (self.viewModel.isHost) {
        if (streamList.count > 0) {
            // 先判断是否已经有达到上限的主播在房间里了
            [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(live_room_is_occupied)).LeeAction(TLLocalizedString(confirm), ^{
                [self.navigationController popViewControllerAnimated:YES];
            }).LeeShow();
            return;
        }
        
        ZegoLiveRenderViewRelation *relation = [[ZegoLiveRenderViewRelation alloc] init];
        relation.userID = self.viewModel.userID;
        relation.userName = self.viewModel.userName;
        
        UIView *mainStreamView = [UIView new];
        relation.renderView = mainStreamView;
        [self.view addSubview:mainStreamView];
        
        UILabel *wheatLabel = [[UILabel alloc] init];
        wheatLabel.textColor = [UIColor blackColor];
        wheatLabel.font = [UIFont systemFontOfSize:17];
        wheatLabel.text = relation.userName;
        [relation.renderView addSubview:wheatLabel];
        relation.label = wheatLabel;
        
        [self.rtcManager startPublishStream:YES];
        [self.rtcManager startPreview:mainStreamView];
        relation.streamID = self.rtcManager.publishStreamID;
        [self.streamViewList addObject:relation];
    } else {
        self.menuZoneBtn.hidden = YES;
        NSArray <ZegoRoomExtraInfo *> *roomExtraInfoList = [self.rtcManager getRoomExtraInfoList];
        if (roomExtraInfoList != nil && roomExtraInfoList.count > 0) {
            for (ZegoRoomExtraInfo * roomExtraInfo in roomExtraInfoList) {
                if ([roomExtraInfo.key isEqualToString:ROOM_GAME_INFO]) {
                    NSString *value = roomExtraInfo.value;
                    if (!ZGStringIsEmpty(value)) {
                        self.roomGameInfo = [RoomGameInfo modelWithJSON:value];
                        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                        [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                            kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode, gameInfoDetail);
                            if (errorCode != 0) {
                                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                            }
                            if (gameInfoDetail != nil) {
                                self.currentGameID = gameInfoDetail.gameID;
                                self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                // 加载游戏
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    NSDictionary *gameConfig = @{
                                        @"roomID" : self.roomGameInfo.roomID ? :@"",
                                        @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                    };
                                    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                                                     gameMode:ZegoGameModeHostsGame
                                                                   gameConfig: gameConfig
                                                                     callBack:^(NSInteger errorCode) {
                                        if (ZegoMiniGameEngineSuccess == errorCode) {
                                            self.isPlayingGame = YES;
                                            [self onGameStart];
                                            [self updateButtonVisibility];
                                        } else {
                                            [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                        }
                                    }];
                                });
                            }
                        }];
                    }
                }
            }
        }
    }
    [self handleStreamsAdded:streamList];
    [self updateButtonVisibility];
}

- (void)updateButtonVisibility{
    if (self.viewModel.isHost) {
        if (self.isPlayingGame) {
            self.playGameBtn.hidden = YES;
            self.closeGameBtn.hidden = NO;
        } else {
            self.playGameBtn.hidden = NO;
            self.closeGameBtn.hidden = YES;
        }
        self.leaveGameBtn.hidden = YES;
        self.playShowGameBtn.hidden = YES;
        self.joinLiveBtn.hidden = YES;
    } else {
        if (self.isPlayingGame) {
            self.playShowGameBtn.hidden = YES;
            self.leaveGameBtn.hidden = NO;
        } else {
            self.playShowGameBtn.hidden = YES;
            self.leaveGameBtn.hidden = YES;
        }
        self.playGameBtn.hidden = YES;
        self.closeGameBtn.hidden = YES;
        self.joinLiveBtn.hidden = YES;
    }
    [self.gameView bringSubviewToFront:self.playGameBtn];
    [self.gameView bringSubviewToFront:self.closeGameBtn];
    [self.gameView bringSubviewToFront:self.leaveGameBtn];
    [self.gameView bringSubviewToFront:self.playShowGameBtn];
    [self.gameView bringSubviewToFront:self.joinLiveBtn];
    [self.gameView bringSubviewToFront:self.menuZoneBtn];
    [self.gameView bringSubviewToFront:self.menuZoneView];
}

- (void)refreshHostView {
    NSInteger i = 0;
    NSInteger total = self.streamViewList.count;
    CGSize containerViewSize = self.view.bounds.size;
    // 如果人数是1个，则streamLayout是全屏，如果是2个以上，则占上边宽：高是16：9 （竖屏）的区域
    CGFloat playViewWidth = (containerViewSize.width - (total - 1) * 5) / total;
    CGFloat playViewHeight = total <= 1 ? containerViewSize.height : playViewWidth * 1.6;
    for (ZegoLiveRenderViewRelation *relation in self.streamViewList) {
        CGFloat x = i * playViewWidth + i * 5;
        CGFloat y = total <= 1 ? 0 : (60 + kTLTopStatusBarHeight);
        relation.renderView.frame = CGRectMake(x, y, playViewWidth, playViewHeight);
        if (i > 0) {
            relation.label.frame = CGRectMake(10, 10, playViewWidth, 20);
        }
        if (self.viewModel.isHost && i == 0) {
            [self.rtcManager startPreview:relation.renderView];
        } else {
            [self.rtcManager startPlayingStream:relation.streamID renderView:relation.renderView];
        }
        [self.view sendSubviewToBack:relation.renderView];
        i++;
    }
}

- (void)handleStreamsAdded:(NSArray <StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        if ([streamInfo.streamID hasPrefix:STREAM_PREFIX]) {
            if ([streamInfo.streamID isEqualToString:self.rtcManager.publishStreamID]) {
                continue;
            }
        }
        
        for (ZegoLiveRenderViewRelation *relation in self.streamViewList) {
            if([relation.streamID isEqualToString:streamInfo.streamID]) { //去除重复的流ID
                [self.streamViewList removeObject:relation];
            }
        }
        
        ZegoLiveRenderViewRelation *relation = [ZegoLiveRenderViewRelation new];
        relation.userID = streamInfo.userID;
        relation.userName = streamInfo.userName;
        relation.streamID = streamInfo.streamID;
        
        UIView *streamView = [UIView new];
        relation.renderView = streamView;
        [self.view addSubview:streamView];
        
        UILabel *wheatLabel = [[UILabel alloc] init];
        wheatLabel.textColor = [UIColor blackColor];
        wheatLabel.font = [UIFont systemFontOfSize:17];
        wheatLabel.text = relation.userName;
        [relation.renderView addSubview:wheatLabel];
        relation.label = wheatLabel;
        
        [self.streamViewList addObject:relation];
        [self.rtcManager startPlayingStream:streamInfo.streamID renderView:streamView];
    }
    [self refreshHostView];
}

- (void)onPlayShowGameBtnAction:(UIButton *)sender {
    self.menuZoneView.hidden = YES;
    [ZegoSelectGamePicker show:self.viewModel.showGameList isShowGround:YES type:ZegoSelectTypeHalfScreen
             selectedGameBlock:^(ZegoGameInfo *gameInfo) {
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameInfo.gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [ZegoMiniGameEngine.sharedEngine getGameInfo:gameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                }
                if (!ZGObjectIsEmpty(gameInfoDetail)) {
                    [self.gameMap setObject:gameInfoDetail forKey:gameInfo.gameID];
                }
            }];
        }
    } playGameBlock:^(ZegoJoinPlayGameModel *model) {
        kLog(@"playGameBlock:[playGameBlock] selectIndex：%ld, spin：%ld, playerNum：%d, isSoundCustomPlay：%d", model.selectIndex, model.spin, model.playerNum, model.isSoundCustomPlay);
        NSString *gameID = self.viewModel.showGameList[model.selectIndex].gameID;
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"没有获取到游戏详情"] withFinishBlock:nil];
        }
        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
        self.currentGameID = gameInfoDetail.gameID;
        NSDictionary *gameConfig = @{
            @"sceneMode" : @(3),
            @"coinIconUrl" : @"https://bobilive-com-test.jieyou.shop/h5web/GameH5/showDemo/game_bean.png",
            @"roomID" : self.viewModel.roomID ?:@"",
            @"customPlaySound" : @(model.isSoundCustomPlay),
            @"minGameCoin" : @(100)
        };
        
        [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                         gameMode:ZegoGameModeShowGame
                                       gameConfig:gameConfig
                                         callBack:^(NSInteger errorCode) {
            if (ZegoMiniGameEngineSuccess == errorCode) {
                self.isPlayingGame = YES;
                [self onGameStart];
                [self updateButtonVisibility];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        }];
    }];
                
    return;
    
    ZegoPlayGameAlterView *alertView = [[ZegoPlayGameAlterView alloc] init];
    alertView.gameList = self.viewModel.showGameList;
    alertView.isShowGround = YES;
    [alertView show];
    [alertView playGameBlock:^(JoinPlayGameModel *model) {
        kLog(@"playGameBlock:[playGameBlock] selectIndex：%ld, spin：%ld, playerNum：%d, isSoundCustomPlay：%d", model.selectIndex, model.spin, model.playerNum, model.isSoundCustomPlay);
        NSString *gameID = self.viewModel.showGameList[model.selectIndex].gameID;
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"没有获取到游戏详情"] withFinishBlock:nil];
        }
        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
        self.currentGameID = gameInfoDetail.gameID;
        
        NSDictionary *gameConfig = @{
            @"roomID" : self.viewModel.roomID ? :@"",
            @"customPlaySound" : @(model.isSoundCustomPlay),
        };
        
        [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                         gameMode:ZegoGameModeShowGame
                                       gameConfig:gameConfig
                                         callBack:^(NSInteger errorCode) {
            if (ZegoMiniGameEngineSuccess == errorCode) {
                self.isPlayingGame = YES;
                [self onGameStart];
                [self updateButtonVisibility];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        }];
    }];
    
    [alertView selectedGameBlock:^(ZegoGameInfo *gameInfo) {
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameInfo.gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [ZegoMiniGameEngine.sharedEngine getGameInfo:gameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                }
                if (!ZGObjectIsEmpty(gameInfoDetail)) {
                    [self.gameMap setObject:gameInfoDetail forKey:gameInfo.gameID];
                }
            }];
        }
    }];
}

// 房主点击选择游戏
- (void)onClickPlayGameAction:(UIButton *)sender {
    self.menuZoneView.hidden = YES;
    __block ZegoSelectGamePicker *picker = [ZegoSelectGamePicker show:self.viewModel.hostGameList isShowGround:NO type:ZegoSelectTypeFullScreen selectedGameBlock:^(ZegoGameInfo * gameInfo) {
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameInfo.gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [ZegoMiniGameEngine.sharedEngine getGameInfo:gameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                if (!ZGObjectIsEmpty(gameInfoDetail)) {
                    [self.gameMap setObject:gameInfoDetail forKey:gameInfo.gameID];
                    [picker updateGameParams:gameInfoDetail];
                }
            }];
        } else {
            [picker updateGameParams:gameInfoDetail];
        }
    } playGameBlock:^(ZegoJoinPlayGameModel *model) {
        kLog(@"playGameBlock:[playGameBlock] selectIndex：%ld, spin：%ld, playerNum：%d", model.selectIndex, model.spin, model.playerNum);
        NSString *gameID = self.viewModel.hostGameList[model.selectIndex].gameID;
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameID];
        ZegoGameTaxType taxType = model.pumpingType == 1 ? ZegoGameTaxTypeBetCoin : ZegoGameTaxTypeForWinner;
        int taxRate = model.pumpingRatio;
//        [ZegoMiniGameEngine.sharedEngine createGameRoom:gameInfoDetail.gameID roomID:self.viewModel.roomID maxPlayerNum:model.playerNum gameCoin:model.spin taxType:taxType taxRate:taxRate callBack:^(NSInteger errorCode) {
//            if (errorCode == 0) {
//                kLog(@"创建游戏房间成功");
//            }
//        }];
        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
        self.currentGameID = gameInfoDetail.gameID;
        
        RoomGameInfo *roomGameInfo = [[RoomGameInfo alloc] init];
        roomGameInfo.gameID = gameInfoDetail.gameID;
        roomGameInfo.roomID = self.viewModel.roomID;
        roomGameInfo.isSoundCustomPlay = model.isSoundCustomPlay;
        
        [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:[roomGameInfo modelToJSONObject]];

        NSDictionary *gameConfig = @{
            @"sceneMode" : @(1),
            @"coinIconUrl" : @"https://bobilive-com-test.jieyou.shop/h5web/GameH5/showDemo/game_bean.png",
            @"roomID" : roomGameInfo.roomID ? :@"",
            @"customPlaySound" : @(roomGameInfo.isSoundCustomPlay),
            @"useRobot" : @(true),
        };
        [ZegoMiniGameEngine.sharedEngine loadGame:gameInfoDetail.gameID gameMode:ZegoGameModeHostsGame
                                       gameConfig:gameConfig callBack:^(NSInteger errorCode) {
            if (ZegoMiniGameEngineSuccess == errorCode) {
                self.isPlayingGame = YES;
                [self onGameStart];
                [self updateButtonVisibility];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        }];
    }];
}

- (void)onClickCloseGameAction:(UIButton *)sender {
    [ZegoMiniGameEngine.sharedEngine unloadGame:true];
    [self updateButtonVisibility];
    [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:@""];
}

- (void)onClickLeaveGameAction:(UIButton *)sender {
    [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
}

- (void)onClickSwitchRoomAction:(UIButton *)sender {

}

- (void)onClickJoinLiveAction:(UIButton *)sender {
    self.menuZoneView.hidden = YES;
    if ([self.rtcManager getStreamInfoList].count == 0) {
        [MBProgressHUD showSuccess:TLLocalizedString(host_not_start_live) withFinishBlock:nil];
        return;
    } else if (!self.isJoinLive && [self.rtcManager getStreamInfoList].count >= 2) {
        [MBProgressHUD showSuccess:TLLocalizedString(reach_limit_of_open_mic) withFinishBlock:nil];
        return;
    }
    if (self.isJoinLive) {
        [self.joinLiveBtn setTitle:TLLocalizedString(open_mic) forState:UIControlStateNormal];
        [self.joinLiveBtn setImage:[UIImage imageNamed:@"icon_servewheat"] forState:UIControlStateNormal];
        self.joinLiveBtn.backgroundColor = [UIColor blackColor];
        [self.rtcManager stopPublishStream];
        
        for (ZegoLiveRenderViewRelation *relation in self.streamViewList) {
            NSString *streamID = (NSString *)relation.streamID;
            if ([streamID isEqualToString:self.currentRelation.streamID]) {
                [relation.renderView removeFromSuperview];
                [self.streamViewList removeObject:relation];
                break;
            }
        }
        
        [self.memberCountBtn mas_updateConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.gameView).offset(TOP_BAR_HEIGHT + 6);
            make.height.mas_equalTo(28);
            make.width.mas_equalTo(83);
        }];
        
        [self.closeImageView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.height.mas_equalTo(26);
            make.width.mas_equalTo(26);
        }];
    } else {
        [self.joinLiveBtn setTitle:TLLocalizedString(close_mic) forState:UIControlStateNormal];
        [self.joinLiveBtn setImage:[UIImage imageNamed:@"icon_dropwheat"] forState:UIControlStateNormal];
        self.joinLiveBtn.backgroundColor = [UIColor colorWithHexString:@"FF524A"];
        ZegoLiveRenderViewRelation *relation = [ZegoLiveRenderViewRelation new];
        relation.userID = self.viewModel.userID;
        relation.userName = self.viewModel.userName;
        
        UIView *streamView = [UIView new];
        relation.renderView = streamView;
        [self.view addSubview:streamView];
        
        UILabel *wheatLabel = [[UILabel alloc] init];
        wheatLabel.textColor = [UIColor blackColor];
        wheatLabel.font = [UIFont systemFontOfSize:17];
        wheatLabel.text = self.viewModel.userName;
        [relation.renderView addSubview:wheatLabel];
        relation.label = wheatLabel;
        
        [self.rtcManager startPublishStream:NO];
        [self.rtcManager startPreview:streamView];
        relation.streamID = self.rtcManager.publishStreamID;
        self.currentRelation = relation;
        [self.streamViewList addObject:relation];
        
        [self.memberCountBtn mas_updateConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.gameView).offset(kTLTopStatusBarHeight + 68);
            make.height.mas_equalTo(22);
            make.width.mas_equalTo(65);
        }];
        
        [self.closeImageView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(20);
        }];
    }
    self.isJoinLive = !self.isJoinLive;
    [self refreshHostView];
}

#pragma mark----ZegoGameEngineHandler

- (void)onGameLoadStateUpdate:(ZegoGameLoadState)gameLoadState {
    kLog(@"onGameLoadStateUpdate：%ld", gameLoadState);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"gameEvent：%ld", gameLoadState] withFinishBlock:nil];
    if (gameLoadState == ZegoGameLoaded || gameLoadState == ZegoGameLoading) {
        self.loadingLabel.hidden = YES;
    }
}

- (void)onGameStateUpdate:(ZegoGameState)gameState {
    kLog(@"onGameStateUpdate：%ld", gameState);
}

- (void)onGameStateUpdate:(ZegoGameState)gameState reasonCode:(int)reasonCode {
    kLog(@"onGameStateUpdate：%ld reasonCode：%d", gameState,reasonCode);
}

- (void)onPlayerStateUpdate:(ZegoGamePlayerState)playerState {
    kLog(@"onPlayerStateUpdate playerEvent：%ld", playerState);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(update_player_status), playerState] withFinishBlock:nil];
}

- (void)onUnloaded:(NSString *)gameID {
    if (!self.isBack) {
        [self onGameQuit];
    }
    self.currentGameID = @"";
    self.isPlayingGame = NO;
    [self updateButtonVisibility];
    if (self.audioPlayer != nil) {
        [self.audioPlayer destroy];
        self.audioPlayer = nil;
    };
    if (!self.menuZoneView.isHidden) {
        self.menuZoneView.hidden = YES;
    }
}

- (void)onGameSoundPlay:(NSString *)name isPlay:(BOOL)isPlay url:(NSString *)url isLoop:(BOOL)isLoop volume:(int)volume {
    if (self.audioPlayer != nil) {
        [self.audioPlayer playSound:name isPlay:isPlay url:url isLoop:isLoop volume:volume];
    }
//    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"播放声音 name：%@, isPlay：%d, url：%@, isLoop：%d, volume：%d", name, isPlay, url, isLoop, volume] withFinishBlock:nil];
}

- (void)onGameSoundVolumeChange:(NSString *)name volume:(int)volume {
    if (self.audioPlayer != nil) {
        [self.audioPlayer changeSoundVolume:name volume:volume];
    }
    kLog(@"[onGameSoundVolumeChange] name：%@, volume：%d",  name, volume);
}

#pragma mark----RTCEventListener

- (void)onStreamAdd:(NSArray<StreamInfo *> *)streamInfoList {
    [self handleStreamsAdded:streamInfoList];
}

- (void)onStreamRemove:(NSArray<StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        if ([streamInfo.streamID hasPrefix:STREAM_PREFIX]) {
            for (ZegoLiveRenderViewRelation *relation in self.streamViewList) {
                NSString *streamID = (NSString *)relation.streamID;
                if ([streamID isEqualToString:streamInfo.streamID]) {
                    [relation.renderView removeFromSuperview];
                    [self.streamViewList removeObject:relation];
                    [self.rtcManager stopPlayingStream:streamInfo.streamID];
                    [self refreshHostView];
                    break;
                }
            }
        }
    }
}

- (void)onRoomStateChanged:(RoomStateChangeReason)reason {
    if (reason == KickOut) {
        [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(already_login_on_other_device)).LeeAction(TLLocalizedString(confirm), ^{
            [self.navigationController popViewControllerAnimated:YES];
        }).LeeShow();
    }  else if (reason == Disconnect|| reason == LoginFailed) {
        [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(you_are_offline)).LeeAction(TLLocalizedString(confirm), ^{
            [self.navigationController popViewControllerAnimated:YES];
        }).LeeShow();
    }
}

- (void)onRoomExtraInfoUpdate:(NSString *)roomID arrayList:(NSArray<ZegoRoomExtraInfo *> *)arrayList {
    @weakify(self)
    NSArray <ZegoRoomExtraInfo *> *roomExtraInfoList = [self.rtcManager getRoomExtraInfoList];
    if (roomExtraInfoList != nil && roomExtraInfoList.count > 0) {
        for (ZegoRoomExtraInfo * roomExtraInfo in roomExtraInfoList) {
            if ([roomExtraInfo.key isEqualToString:ROOM_GAME_INFO]) {
                NSString *value = roomExtraInfo.value;
                if (!ZGStringIsEmpty(value)) {
                    self.roomGameInfo = [RoomGameInfo modelWithJSON:value];
                    if (roomExtraInfo == nil) {
                        [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString("game_room_info_error")] withFinishBlock:nil];
                        return;
                    }
                    // 先判断是否已经有达到上限的主播在房间里了
                    if (self.isPlayingGame) {
                        ZegoInviteAlterView *alterView = [ZegoInviteAlterView alertView];
                        [alterView onAgreeInviteBlock:^(ZGInviteType type) {
                            if (type == ZGAgreeInviteType) {
                                [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
                                
                                [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                                [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode2, ZegoGameInfoDetail * gameInfoDetail) {
                                    @strongify(self)
                                    kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode2, gameInfoDetail);
//                                    if (errorCode2 != 0) {
//                                        [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
//                                        return;
//                                    }
                                    if (gameInfoDetail != nil) {
                                        self.currentGameID = gameInfoDetail.gameID;
                                        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                        // 加载游戏
                                        dispatch_async(dispatch_get_main_queue(), ^{
                                            NSDictionary *gameConfig = @{
                                                @"roomID" : self.roomGameInfo.roomID? :@"",
                                                @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                            };
                                            
                                            [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID                                gameMode:ZegoGameModeHostsGame
                                                                           gameConfig:gameConfig callBack:^(NSInteger errorCode) {
                                                if (ZegoMiniGameEngineSuccess == errorCode) {
                                                    self.isPlayingGame = YES;
                                                    [self onGameStart];
                                                    [self updateButtonVisibility];
                                                } else {
                                                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                                }
                                            }];
                                        });
                                    }
                                }];
                        }
                        }];
                        [alterView show];
//                        [LEEAlert alert].config.LeeTitle(@"提示")
//                            .LeeCancelAction(@"取消", ^{}).LeeContent(@"主播开启了带玩游戏，是否停止当前的游戏，切换到主播的带玩游戏").LeeAction(@"确定", ^).LeeShow();
                    } else {
                        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                        [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode2, ZegoGameInfoDetail * gameInfoDetail) {
                            @strongify(self)
                            kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode2, gameInfoDetail);
                            if (errorCode2 != 0) {
                                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode2]] withFinishBlock:nil];
                                return;
                            }
                            if (gameInfoDetail != nil) {
                                self.currentGameID = gameInfoDetail.gameID;
                                self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                // 加载游戏
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    NSDictionary *gameConfig = @{
                                        @"roomID" : self.roomGameInfo.roomID? :@"",
                                        @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                    };
                                    
                                    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                                                     gameMode:ZegoGameModeHostsGame
                                                                   gameConfig:gameConfig
                                                                     callBack:^(NSInteger errorCode) {
                                        if (ZegoMiniGameEngineSuccess == errorCode) {
                                            self.isPlayingGame = YES;
                                            [self onGameStart];
                                            [self updateButtonVisibility];
                                        } else {
                                            [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                        }
                                    }];
                                    
                                });
                            }
                        }];
                    }
                } else {
                    // 不处理，等待游戏的unload通知
                    [self updateButtonVisibility];
                }
            }
        }
    }
}

- (void)onRoomOnlineUserCountUpdate:(NSString *)roomID count:(int)count {
    [self.memberCountBtn setTitle:[NSString stringWithFormat: TLLocalizedString(online_member), [self.rtcManager getMemberCount]] forState:UIControlStateNormal];
}

- (void)onRoomUserUpdate:(NSArray<ZegoUser *> *)userList {
    
}


#pragma mark----getter

- (UIView *)gameView {
    if (!_gameView) {
        _gameView = [[UIView alloc] init];
    }
    return _gameView;
}

- (UIButton *)playGameBtn {
    if (!_playGameBtn) {
        _playGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(chose_game)];
        _playGameBtn.layer.cornerRadius = 18;
        _playGameBtn.layer.masksToBounds = YES;
        _playGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_playGameBtn addTarget:self action:@selector(onClickPlayGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playGameBtn;
}

- (UIButton *)closeGameBtn {
    if (!_closeGameBtn) {
        _closeGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(close_game)];
        _closeGameBtn.layer.cornerRadius = 18;
        _closeGameBtn.layer.masksToBounds = YES;
        _closeGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_closeGameBtn addTarget:self action:@selector(onClickCloseGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _closeGameBtn;
}

- (UIButton *)leaveGameBtn {
    if (!_leaveGameBtn) {
        _leaveGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(quit_game)];
        _leaveGameBtn.layer.cornerRadius = 18;
        _leaveGameBtn.layer.masksToBounds = YES;
        _leaveGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _leaveGameBtn.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        [_leaveGameBtn addTarget:self action:@selector(onClickLeaveGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _leaveGameBtn;
}

- (UIButton *)switchRoomBtn {
    if (!_switchRoomBtn) {
        _switchRoomBtn = [UIButton actionButtonWithTitle:@"切换房间"];
        _switchRoomBtn.layer.cornerRadius = 18;
        _switchRoomBtn.layer.masksToBounds = YES;
        _switchRoomBtn.hidden = YES;
        _switchRoomBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _switchRoomBtn.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        [_switchRoomBtn addTarget:self action:@selector(onClickSwitchRoomAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _switchRoomBtn;
}

- (AlignButton *)joinLiveBtn {
    if (!_joinLiveBtn) {
        _joinLiveBtn = [[AlignButton alloc] init];
        _joinLiveBtn.alignType = AlignType_TextRight;
        _joinLiveBtn.padding = 8;
        _joinLiveBtn.layer.cornerRadius = 18;
        _joinLiveBtn.layer.masksToBounds = YES;
        _joinLiveBtn.hidden = YES;
        _joinLiveBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _joinLiveBtn.backgroundColor = [UIColor blackColor];
        [_joinLiveBtn setTitle:TLLocalizedString(open_mic) forState:UIControlStateNormal];
        [_joinLiveBtn setImage:[UIImage imageNamed:@"icon_servewheat"] forState:UIControlStateNormal];
        [_joinLiveBtn addTarget:self action:@selector(onClickJoinLiveAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _joinLiveBtn;
}

- (UIButton *)playShowGameBtn {
    if (!_playShowGameBtn) {
        _playShowGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(chose_game)];
        _playShowGameBtn.layer.cornerRadius = 18;
        _playShowGameBtn.layer.masksToBounds = YES;
        _playShowGameBtn.hidden = YES;
        _playShowGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _playShowGameBtn.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        [_playShowGameBtn addTarget:self action:@selector(onPlayShowGameBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playShowGameBtn;
}

- (NSMutableDictionary *)gameMap {
    if (!_gameMap) {
        _gameMap = [NSMutableDictionary dictionary];
    }
    return _gameMap;
}

- (NSMutableArray<ZegoLiveRenderViewRelation *> *)streamViewList {
    if (!_streamViewList) {
        _streamViewList = [NSMutableArray array];
    }
    return _streamViewList;
}

- (void)dealloc {
    [self.rtcManager removeRTCEventListener:self];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationWillTerminateNotification object:nil];
}
@end
