//
//  ZegoAudioroomViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/5/26.
//

#import "ZegoAudioroomViewController.h"
#import "ZegoRTCEngineManager.h"
#import <ZegoMiniGameEngine/ZegoMiniGameEngine.h>
#import "ZegoPlayGameAlterView.h"
#import "ZegoRechargeAlterView.h"
#import "ZegoLiveRenderViewRelation.h"
#import "ZegoInviteAlterView.h"
#import "ZegoAudioPlayer.h"
#import "RoomGameInfo.h"
#import "KeyCenter.h"
#import "AlignButton.h"
#import "ZegoSelectGamePicker.h"
#import "ZGUserInfoRandom.h"
#import "ZegoGameViewModel.h"
#import "ZegoAudioroomMembersView.h"
#import "ZegoSelectPlayerController.h"

#import "UIKit+AFNetworking.h"

static NSString *HOST_ROLE = @"host_role";

@interface ZegoAudioroomViewController ()<RTCEventListener, ZegoGameEngineHandler>
@property (nonatomic, weak) WKWebView *zegoWebview;

@property (nonatomic, assign) ZegoGameMode gameMode;

@end

@implementation ZegoAudioroomViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupBaseUI];
    [self joinLiveRoom];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.isBack = YES;
    if (self.viewModel.isHost) {
        [self.rtcManager stopPublishStream];
        if (!ZGStringIsEmpty(self.currentGameID)) {
            [ZegoMiniGameEngine.sharedEngine unloadGame:true];
            [self onGameQuit];
            [self updateButtonVisibility];
            [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:@""];
        }
    } else {
        if (!ZGStringIsEmpty(self.currentGameID)) {
            [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
            [self onGameQuit];
            [self updateButtonVisibility];
        }
    }
    if (self.audioPlayer != nil) {
        [self.audioPlayer destroy];
        self.audioPlayer = nil;
    };
    [self.rtcManager logoutRoom];
}

- (void)initData {
    [super initData];
    [ZegoRTCEngineManager.sharedInstance muteMicrophone:0];
}

- (void)setupBaseUI {
    [super setupBaseUI];
    
    [self.view addSubview:self.gameView];
    [self.view addSubview:self.topView];
    self.navigationItem.title = [NSString stringWithFormat:@"%@  %@", self.viewModel.userName, self.viewModel.roomID];
    self.tabBarController.hidesBottomBarWhenPushed = YES;
    
    [self.topView addSubview:self.gameIconView];
    [self.topView addSubview:self.gameNameView];
    [self.topView addSubview:self.memberCountBtn];
    [self.topView addSubview:self.closeImageView];
    [self.topView addSubview:self.startGameButton];
    [self.topView addSubview:self.seatView];
    
    [self.gameView addSubview:self.playGameBtn];
    [self.gameView addSubview:self.loadingLabel];
    
    [ZegoKeyWindow addSubview:self.menuZoneView];
    [self.topView addSubview:self.menuZoneBtn];
    [self.menuZoneView addSubview:self.menuZoneTable];
    
    [ZegoKeyWindow addSubview:self.arrangeView];
    [self.gameView addSubview:self.closeGameBtn];
    [self.gameView addSubview:self.leaveGameBtn];
    [self.gameView addSubview:self.playShowGameBtn];

    [self updateButtonVisibility];
    [self setupBaseUIConstraints];
}

- (void)updateWebviewFrame:(CGFloat)width height:(CGFloat)height {
    [super updateWebviewFrame:width height:height];

    [self.gameView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.width.mas_equalTo(width);
        make.height.mas_equalTo(height);
        make.bottom.mas_equalTo(self.view.mas_bottom);
    }];
}

- (void)joinLiveRoom {
    NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
    kLog(@"streamList is %ld", streamList.count)
    if (self.viewModel.isHost) {
        for (StreamInfo *info in streamList) {
            if([info.streamID hasSuffix:HOSTFLAG]) {
                [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(live_room_is_occupied)).LeeAction(TLLocalizedString(confirm), ^{
                    [self.navigationController popViewControllerAnimated:YES];
                }).LeeShow();
                return;
            }
        }

        [self.rtcManager startPublishStream:YES];
    } else {
        [self.rtcManager startPublishStream:NO];
        
        NSArray <ZegoRoomExtraInfo *> *roomExtraInfoList = [self.rtcManager getRoomExtraInfoList];
        kLog(@"roomExtraInfoList is %@", [roomExtraInfoList modelToJSONString])
        if (roomExtraInfoList != nil && roomExtraInfoList.count > 0) {
            for (ZegoRoomExtraInfo * roomExtraInfo in roomExtraInfoList) {
                if ([roomExtraInfo.key isEqualToString:ROOM_GAME_INFO]) {
                    NSString *value = roomExtraInfo.value;
                    if (!ZGStringIsEmpty(value)) {
                        self.roomGameInfo = [RoomGameInfo modelWithJSON:value];
                        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                        [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                            kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode, gameInfoDetail);
                            if (errorCode != 0) {
                                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                            }
                            if (gameInfoDetail != nil) {
                                self.currentGameID = gameInfoDetail.gameID;
                                self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                // 加载游戏
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    NSDictionary *gameConfig = @{
                                        @"roomID" : self.roomGameInfo.roomID ? :@"",
                                        @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                    };
                                    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                                                     gameMode:ZegoGameModeHostsGame
                                                                   gameConfig: gameConfig
                                                                     callBack:^(NSInteger errorCode) {
                                        if (ZegoMiniGameEngineSuccess == errorCode) {
                                            self.isPlayingGame = YES;
                                            [self onGameStart];
                                            [self updateButtonVisibility];
                                        } else {
                                            [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                        }
                                    }];
                                });
                            }
                        }];
                    }
                }
            }
        }
    }
    [self handleStreamsAdded:streamList];
    [self updateButtonVisibility];
}

- (void)updateButtonVisibility{
    if (self.viewModel.isHost) {
        if (self.isPlayingGame) {
            self.playGameBtn.hidden = YES;
            self.closeGameBtn.hidden = NO;
        } else {
            self.playGameBtn.hidden = NO;
            self.closeGameBtn.hidden = YES;
        }
        self.leaveGameBtn.hidden = YES;
        self.playShowGameBtn.hidden = YES;
    } else {
        if (self.isPlayingGame) {
            self.playShowGameBtn.hidden = YES;
            self.leaveGameBtn.hidden = NO;
        } else {
            self.playShowGameBtn.hidden = NO;
            self.leaveGameBtn.hidden = YES;
        }
        self.playGameBtn.hidden = YES;
        self.closeGameBtn.hidden = YES;
    }
    [self.gameView bringSubviewToFront:self.playGameBtn];
    [self.gameView bringSubviewToFront:self.closeGameBtn];
    [self.gameView bringSubviewToFront:self.leaveGameBtn];
    [self.gameView bringSubviewToFront:self.playShowGameBtn];
    
    [self.topView bringSubviewToFront:self.menuZoneBtn];
    [self.topView bringSubviewToFront:self.menuZoneView];
}

- (void)refreshSeatView {
    NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
    for (StreamInfo *info in streamList) {
        if((self.viewModel.isHost && [info.userID isEqualToString:self.viewModel.userID])
           || [info.streamID containsString:HOSTFLAG]) {
            info.isHost = YES;
            break;
        }
    }
    
    self.seatView.hidden = NO;
    [self.seatView setMembers:streamList];
    CGFloat height = self.seatView.height;
    [self.seatView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(height);
        make.left.mas_equalTo(self.topView.mas_left).offset(15.0);
        make.right.mas_equalTo(self.topView.mas_right).offset(-15.0);
        make.top.mas_equalTo(self.closeImageView.mas_bottom);
    }];
}

- (void)refreshHostView {
    [self refreshSeatView];
}

- (void)handleStreamsAdded:(NSArray <StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        if ([streamInfo.streamID hasPrefix:STREAM_PREFIX]) {
            if ([streamInfo.streamID isEqualToString:self.rtcManager.publishStreamID]) {
                continue;
            }
        }
        kLog(@"startPlayingStream streamInfo.streamID is %@", streamInfo.streamID);

        [self.rtcManager startPlayingStream:streamInfo.streamID renderView:nil];
    }
    [self refreshHostView];
}

// 其他成员点击选择游戏
- (void)onPlayShowGameBtnAction:(UIButton *)sender {
    self.menuZoneView.hidden = YES;
    @weakify(self);
    [ZegoSelectGamePicker show:self.viewModel.showGameList isShowGround:YES type:ZegoSelectTypeHalfScreen
             selectedGameBlock:^(ZegoGameInfo *gameInfo) {
        @strongify(self);
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameInfo.gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            @weakify(self);
            [ZegoMiniGameEngine.sharedEngine getGameInfo:gameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                @strongify(self);
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                }
                if (!ZGObjectIsEmpty(gameInfoDetail)) {
                    [self.gameMap setObject:gameInfoDetail forKey:gameInfo.gameID];
                }
            }];
        }
    } playGameBlock:^(ZegoJoinPlayGameModel *model) {
        kLog(@"playGameBlock:[playGameBlock] selectIndex：%ld, spin：%ld, playerNum：%d, isSoundCustomPlay：%d", model.selectIndex, model.spin, model.playerNum, model.isSoundCustomPlay);
        @strongify(self);
        NSString *gameID = self.viewModel.showGameList[model.selectIndex].gameID;
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"没有获取到游戏详情"] withFinishBlock:nil];
        }
        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
        self.currentGameID = gameInfoDetail.gameID;
        
        NSDictionary *gameConfig = @{
            @"coinIconUrl" : @"https://bobilive-com-test.jieyou.shop/h5web/GameH5/showDemo/game_bean.png",
            @"roomID" : self.viewModel.roomID ?:@"",
            @"customPlaySound" : @(model.isSoundCustomPlay),
            @"useRobot" : @(true),
            @"minGameCoin" : @(0),
            @"specificConfig" : @{@"showMatchingView" : @(0),
            }
        };
        if (ZGObjectIsEmpty(self.currentGameID)) {
            self.currentGameID = gameID;
            NSLog(@"游戏ID为空:%@",gameID);
        }
        @weakify(self);
        [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                         gameMode:ZegoGameModeShowGame
                                       gameConfig:gameConfig
                                         callBack:^(NSInteger errorCode) {
            @strongify(self);
            if (ZegoMiniGameEngineSuccess == errorCode) {
                self.isPlayingGame = YES;
                [self onGameStart];
                [self updateButtonVisibility];
                for (UIView *subview in self.gameView.subviews) {
                    if([subview isKindOfClass:[WKWebView class]]) {
                        NSLog(@"~~~~~~~~~~subview is WKWebView");
                        self.zegoWebview = (WKWebView *)subview;
                        self.zegoWebview.backgroundColor = [UIColor blueColor];
                    }
                }
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        }];
    }];
}

// 房主点击选择游戏
- (void)onClickPlayGameAction:(UIButton *)sender {
    self.menuZoneView.hidden = YES;
    
    UIAlertController *alertCon = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    UIAlertAction *hostPlayAction = [UIAlertAction actionWithTitle:TLLocalizedString(host_play) style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        self.gameMode = ZegoGameModeHostsGame;
        [self playGame:self.viewModel.hostGameList gameMode:ZegoGameModeHostsGame];
    }];
    
    UIAlertAction *matchInAction = [UIAlertAction actionWithTitle:TLLocalizedString(match_in_play) style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        self.gameMode = ZegoGameModeMatchInGame;
        [self playGame:self.viewModel.matchInGameList gameMode:ZegoGameModeMatchInGame];
    }];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:TLLocalizedString(cancel) style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        
    }];
    [alertCon addAction:hostPlayAction];
    [alertCon addAction:matchInAction];
    [alertCon addAction:cancelAction];
    
    [self presentViewController:alertCon animated:YES completion:nil];
}

- (void)playGame:(NSArray<ZegoGameInfo *> *)gamelist gameMode:(ZegoGameMode)gameMode {
    @weakify(self);
    [ZegoSelectGamePicker show:gamelist isShowGround:YES type:ZegoSelectTypeHalfScreen
             selectedGameBlock:^(ZegoGameInfo *gameInfo) {
        @strongify(self);
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameInfo.gameID];
        if (ZGObjectIsEmpty(gameInfoDetail)) {
            @weakify(self);
            [ZegoMiniGameEngine.sharedEngine getGameInfo:gameInfo.gameID callBack:^(NSInteger errorCode, ZegoGameInfoDetail *gameInfoDetail) {
                @strongify(self);
                if (errorCode != 0) {
                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                }
                if (!ZGObjectIsEmpty(gameInfoDetail)) {
                    [self.gameMap setObject:gameInfoDetail forKey:gameInfo.gameID];
                }
            }];
        }
    } playGameBlock:^(ZegoJoinPlayGameModel *model) {
        @strongify(self);
        kLog(@"playGameBlock:[playGameBlock] selectIndex：%ld, spin：%ld, playerNum：%d, isSoundCustomPlay：%d", model.selectIndex, model.spin, model.playerNum, model.isSoundCustomPlay);
        self.playGameModel = model;
        NSString *gameID = gamelist[model.selectIndex].gameID;
        ZegoGameInfoDetail *gameInfoDetail = self.gameMap[gameID];
        
        [self.gameIconView setImageWithURL:[NSURL URLWithString:gameInfoDetail.thumbnail] placeholderImage:[UIImage imageNamed:@"AppIcon"]];
        self.gameNameView.text = gameInfoDetail.gameName;
        [self.topView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(self.view.mas_top).offset(TOP_BAR_HEIGHT);
            make.left.mas_equalTo(self.view.mas_left);
            make.right.mas_equalTo(self.view.mas_right);
            make.bottom.mas_equalTo(self.startGameButton.mas_bottom);
        }];
        
        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
        self.currentGameID = gameInfoDetail.gameID;
        
        RoomGameInfo *roomGameInfo = [[RoomGameInfo alloc] init];
        roomGameInfo.gameID = gameInfoDetail.gameID;
        roomGameInfo.roomID = self.viewModel.roomID;
        roomGameInfo.isSoundCustomPlay = model.isSoundCustomPlay;
        roomGameInfo.gameMode = gameMode;
        
        self.currGameMode = gameMode;
        
        [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:[roomGameInfo modelToJSONObject]];

        NSDictionary *gameConfig = @{
            @"sceneMode" : @(1),
            @"coinIconUrl" : @"https://bobilive-com-test.jieyou.shop/h5web/GameH5/showDemo/game_bean.png",
            @"roomID" : roomGameInfo.roomID ? :@"",
            @"customPlaySound" : @(roomGameInfo.isSoundCustomPlay),
            @"useRobot" : @(true),
        };
        @weakify(self);
        [ZegoMiniGameEngine.sharedEngine loadGame:gameInfoDetail.gameID gameMode:gameMode
                                       gameConfig:gameConfig callBack:^(NSInteger errorCode) {
            @strongify(self);
            if (ZegoMiniGameEngineSuccess == errorCode) {
                self.isPlayingGame = YES;
                [self onGameStart];
                [self updateButtonVisibility];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        }];
    }];
}

- (void)onClickCloseGameAction:(UIButton *)sender {
    self.startGameButton.hidden = YES;
    [ZegoMiniGameEngine.sharedEngine unloadGame:true];
    [self updateButtonVisibility];
    [self.rtcManager setRoomExtraInfo:self.viewModel.roomID forKey:ROOM_GAME_INFO value:@""];
}

- (void)onClickLeaveGameAction:(UIButton *)sender {
    [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
}

// 选择玩家列表
- (void)onClickStartGameAction:(UIButton *)sender {
    
    NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
    ZegoGameInfoDetail *gameInfoDetail = self.gameMap[self.currentGameID];
    @weakify(self);
    ZegoSelectPlayerController *vc = [[ZegoSelectPlayerController alloc] initShow:streamList playerNumList:gameInfoDetail.playerNum spinList:gameInfoDetail.coin playGameBlock:^(ZegoSelectPlayerModel * _Nonnull model) {
        @strongify(self);
        [self onSelectedPlayesStartGame:model];
    }];
    [self presentViewController:vc animated:true completion:nil];
}

// 选择完玩家开始游戏
- (void)onSelectedPlayesStartGame:(ZegoSelectPlayerModel *)model {
    if(self.currGameMode == ZegoGameModeMatchInGame) return;
    
    NSArray *players = model.selectedPlayers;
    NSInteger gamePlayerCount = model.playerNum;
    BOOL selectedRobot = model.isUseRobot;
    
    NSMutableArray <ZegoUserSeatInfo *>*userSeatInfos = [NSMutableArray array];
    for (int i=0; i< players.count; i++) {
        StreamInfo *info = players[i];
        ZegoUserSeatInfo *user = [[ZegoUserSeatInfo alloc] initWithUserID:info.userID seatIndex:i];
        [userSeatInfos addObject:user];
    }
    
    NSMutableArray <ZegoRobotSeatInfo *>*robotSeatInfos = [NSMutableArray array];
    if(selectedRobot && players.count < gamePlayerCount) {
        NSInteger robotCount = gamePlayerCount - players.count;
        for (int i=0; i< robotCount; i++) {
            NSString *name = [ZGUserInfoRandom getRandomName];
            NSString *avtar = [ZGUserInfoRandom getRandomAvatar];
            ZegoRobotSeatInfo *robot = [[ZegoRobotSeatInfo alloc] initWithRobotName:name seatIndex:(int)userSeatInfos.count + i robotAvatar:avtar robotLevel:3 robotBalance:5000];
            [robotSeatInfos addObject:robot];
        }
    }
    ZegoGameTaxType taxType = model.pumpingType == 1 ? ZegoGameTaxTypeBetCoin : ZegoGameTaxTypeForWinner;
    int taxRate = model.pumpingRatio;
    
    ZegoStartGameConfig *gameConfig = [[ZegoStartGameConfig alloc] initWithTaxType:taxType taxRate:taxRate minGameCoin:model.spin timeOut:60 customConfig:@{@"isRobotForcedStart" : @(0)}];
    @weakify(self);
    [ZegoMiniGameEngine.sharedEngine startGame:self.currentGameID roomID:self.viewModel.roomID gameConfig:gameConfig userSeatInfoList:userSeatInfos robotSeatInfoList:robotSeatInfos callBack:^(NSInteger errorCode) {
        NSLog(@"startGameWithGameID done is %ld", (long)errorCode);
        @strongify(self);
        if(errorCode == 0) {
            self.startGameButton.hidden = YES;
        }
    }];
}

#pragma mark----ZegoGameEngineHandler

- (void)onGameActionRequest:(int)actionID data:(NSString *)data callBack:(ZegoGameActionCallback *)callBack {
    UIAlertController *alertCon = [UIAlertController alertControllerWithTitle:@"game action" message:[NSString stringWithFormat:@"actionID:%d, data:%@", actionID, data] preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction *sureAction = [UIAlertAction actionWithTitle:TLLocalizedString(sure) style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        if(callBack.onSuccess) callBack.onSuccess();
    }];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:TLLocalizedString(cancel) style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        if(callBack.onFail) callBack.onFail(1);
    }];
    [alertCon addAction:sureAction];
    [alertCon addAction:cancelAction];
    
    [self presentViewController:alertCon animated:YES completion:nil];
}

- (void)onGameLoadStateUpdate:(ZegoGameLoadState)gameLoadState {
    kLog(@"onGameLoadStateUpdate：%ld", gameLoadState);
    
    if(self.gameMode == ZegoGameModeMatchInGame ) {
        self.startGameButton.hidden = YES;
    } else {
        if(gameLoadState == ZegoGameLoaded && self.viewModel.isHost) {
            self.startGameButton.hidden = NO;
        }
    }
    
    
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onGameLoadStateUpdate：%ld", gameLoadState] withFinishBlock:nil];
    if (gameLoadState == ZegoGameLoaded || gameLoadState == ZegoGameLoading) {
        self.loadingLabel.hidden = YES;
    }
    
    
}

- (void)onGameStateUpdate:(ZegoGameState)gameState {
    kLog(@"onGameStateUpdate：%ld", gameState);
    if(self.gameMode == ZegoGameModeMatchInGame ) {
        self.startGameButton.hidden = YES;
    } else {
        if(gameState == ZegoGameOver && self.viewModel.isHost) {
            self.startGameButton.hidden = NO;
        }
    }
}

- (void)onGameStateUpdate:(ZegoGameState)gameState reasonCode:(int)reasonCode {
    kLog(@"onGameStateUpdate：%ld reasonCode：%d", gameState,reasonCode);
    if(self.gameMode == ZegoGameModeMatchInGame ) {
        self.startGameButton.hidden = YES;
    } else {
        if(gameState == ZegoGameOver && self.viewModel.isHost) {
            self.startGameButton.hidden = NO;
        }
    }
}

- (void)onGameSoundPlay:(NSString *)name isPlay:(BOOL)isPlay url:(NSString *)url isLoop:(BOOL)isLoop volume:(int)volume {
    if (self.audioPlayer != nil) {
        [self.audioPlayer playSound:name isPlay:isPlay url:url isLoop:isLoop volume:volume];
    }
}
- (void)onPlayerStateUpdate:(ZegoGamePlayerState)playerState {
    kLog(@"onPlayerStateUpdate playerEvent：%ld",playerState);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(update_player_status), playerState] withFinishBlock:nil];
}

- (void)onGameSoundVolumeChange:(NSString *)name volume:(int)volume {
    if (self.audioPlayer != nil) {
        [self.audioPlayer changeSoundVolume:name volume:volume];
    }
    kLog(@"[onGameSoundVolumeChange] name：%@, volume：%d",  name, volume);
}

- (void)onUnloaded:(NSString *)gameID {
    if (!self.isBack) {
        [self onGameQuit];
    }
    self.currentGameID = @"";
    self.isPlayingGame = NO;
    [self updateButtonVisibility];
    if (self.audioPlayer != nil) {
        [self.audioPlayer destroy];
        self.audioPlayer = nil;
    };
    
    if (!self.menuZoneView.isHidden) {
        self.menuZoneView.hidden = YES;
    }
}

#pragma mark----RTCEventListener

- (void)onStreamAdd:(NSArray<StreamInfo *> *)streamInfoList {
    [self handleStreamsAdded:streamInfoList];
}

- (void)onStreamRemove:(NSArray<StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        if ([streamInfo.streamID hasPrefix:STREAM_PREFIX]) {
            [self.rtcManager stopPlayingStream:streamInfo.streamID];
            [self refreshHostView];
        }
    }
}

- (void)onRoomStateChanged:(RoomStateChangeReason)reason {
    if (reason == KickOut) {
        [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(already_login_on_other_device)).LeeAction(TLLocalizedString(confirm), ^{
            [self.navigationController popViewControllerAnimated:YES];
        }).LeeShow();
    }  else if (reason == Disconnect|| reason == LoginFailed) {
        [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(you_are_offline)).LeeAction(TLLocalizedString(confirm), ^{
            [self.navigationController popViewControllerAnimated:YES];
        }).LeeShow();
    }
}

- (void)onRoomExtraInfoUpdate:(NSString *)roomID arrayList:(NSArray<ZegoRoomExtraInfo *> *)arrayList {
    @weakify(self)
    //获取到房主身份
    if(arrayList.count == 1) {
        NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
        for (StreamInfo *info in streamList) {
            if([info.streamID hasSuffix:HOSTFLAG]) {
                info.isHost = YES;
                break;
            }
        }
        [self refreshHostView];
    }
    
    NSArray <ZegoRoomExtraInfo *> *roomExtraInfoList = [self.rtcManager getRoomExtraInfoList];
    if (roomExtraInfoList != nil && roomExtraInfoList.count > 0) {
        for (ZegoRoomExtraInfo * roomExtraInfo in roomExtraInfoList) {
            if ([roomExtraInfo.key isEqualToString:ROOM_GAME_INFO]) {
                NSString *value = roomExtraInfo.value;
                if (!ZGStringIsEmpty(value)) {
                    self.roomGameInfo = [RoomGameInfo modelWithJSON:value];
                    if (roomExtraInfo == nil) {
                        [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString("game_room_info_error")] withFinishBlock:nil];
                        return;
                    }
                    // 先判断是否已经有达到上限的主播在房间里了
                    if (self.isPlayingGame) {
                        ZegoInviteAlterView *alterView = [ZegoInviteAlterView alertView];
                        [alterView onAgreeInviteBlock:^(ZGInviteType type) {
                            if (type == ZGAgreeInviteType) {
                                [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
                                [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                                [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode2, ZegoGameInfoDetail * gameInfoDetail) {
                                    @strongify(self)
                                    kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode2, gameInfoDetail);

                                    if (gameInfoDetail != nil) {
                                        self.currentGameID = gameInfoDetail.gameID;
                                        self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                        // 加载游戏
                                        dispatch_async(dispatch_get_main_queue(), ^{
                                            NSDictionary *gameConfig = @{
                                                @"roomID" : self.roomGameInfo.roomID? :@"",
                                                @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                            };
                                            
                                            [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID                                gameMode:self.roomGameInfo.gameMode
                                                                           gameConfig:gameConfig callBack:^(NSInteger errorCode) {
                                                if (ZegoMiniGameEngineSuccess == errorCode) {
                                                    self.isPlayingGame = YES;
                                                    [self onGameStart];
                                                    [self updateButtonVisibility];
                                                } else {
                                                    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                                }
                                            }];
                                        });
                                    }
                                }];
                        }
                        }];
                        [alterView show];
//                        [LEEAlert alert].config.LeeTitle(@"提示")
//                            .LeeCancelAction(@"取消", ^{}).LeeContent(@"主播开启了带玩游戏，是否停止当前的游戏，切换到主播的带玩游戏").LeeAction(@"确定", ^).LeeShow();
                    } else {
                        [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameView];
                        [ZegoMiniGameEngine.sharedEngine getGameInfo:self.roomGameInfo.gameID callBack:^(NSInteger errorCode2, ZegoGameInfoDetail * gameInfoDetail) {
                            @strongify(self)
                            kLog(@"getGameInfo errorCode：%ld, gameInfoDetail：%@", errorCode2, gameInfoDetail);
                            if (errorCode2 != 0) {
                                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(get_game_info_failed), [ZegoConstants getErrorMsg:(int)errorCode2]] withFinishBlock:nil];
                                return;
                            }
                            if (gameInfoDetail != nil) {
                                self.currentGameID = gameInfoDetail.gameID;
                                self.audioPlayer = [[ZegoAudioPlayer alloc] init];
                                // 加载游戏
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    NSDictionary *gameConfig = @{
                                        @"roomID" : self.roomGameInfo.roomID? :@"",
                                        @"customPlaySound" : @(self.roomGameInfo.isSoundCustomPlay),
                                    };
                                    
                                    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                                                     gameMode:self.roomGameInfo.gameMode
                                                                   gameConfig:gameConfig
                                                                     callBack:^(NSInteger errorCode) {
                                        if (ZegoMiniGameEngineSuccess == errorCode) {
                                            self.isPlayingGame = YES;
                                            [self onGameStart];
                                            [self updateButtonVisibility];
                                        } else {
                                            [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
                                        }
                                    }];
                                    
                                });
                            }
                        }];
                    }
                } else {
                    // 不处理，等待游戏的unload通知
                    [self updateButtonVisibility];
                }
            }
        }
    }
}

- (void)onRoomOnlineUserCountUpdate:(NSString *)roomID count:(int)count {
    [self.memberCountBtn setTitle:[NSString stringWithFormat: TLLocalizedString(online_member), [self.rtcManager getMemberCount]] forState:UIControlStateNormal];
}

- (void)onRoomUserUpdate:(NSArray<ZegoUser *> *)userList {
    
}

- (void)setupBaseUIConstraints {
    CGFloat height = self.seatView.height;
    
    [self.topView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_top).offset(TOP_BAR_HEIGHT);
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.bottom.mas_equalTo(self.seatView.mas_bottom);
    }];
    
    [self.gameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.topView.mas_bottom);
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.bottom.mas_equalTo(self.view.mas_bottom);
//        make.height.mas_equalTo(UIScreen.mainScreen.bounds.size.height/2);
    }];
    
    [self.gameIconView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.topView.mas_top);
        make.width.height.mas_equalTo(28);
        make.left.mas_equalTo(self.topView).offset(15.0);
    }];
    
    [self.gameNameView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(self.gameIconView.mas_centerY);
        make.left.mas_equalTo(self.gameIconView.mas_right).offset(15.0);
    }];
    
    [self.memberCountBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.topView);
        make.height.mas_equalTo(28);
        make.width.mas_equalTo(83);
    }];
    
    [self.closeImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.memberCountBtn.mas_right).offset(8);
        make.right.mas_equalTo(self.topView).offset(-8);
        make.centerY.equalTo(self.memberCountBtn.mas_centerY);
        make.height.mas_equalTo(26);
        make.width.mas_equalTo(26);
    }];
    
    
    [self.seatView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(height);
        make.left.mas_equalTo(self.topView.mas_left).offset(15.0);
        make.right.mas_equalTo(self.topView.mas_right).offset(-15.0);
        make.bottom.mas_equalTo(self.topView.mas_bottom);
        make.top.mas_equalTo(self.closeImageView.mas_bottom);
    }];
    
    [self.startGameButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(150);
        make.height.mas_equalTo(44);
        make.centerX.mas_equalTo(self.topView.mas_centerX);
        make.top.mas_equalTo(self.seatView.mas_bottom).offset(50);
        make.bottom.mas_equalTo(self.topView.mas_bottom);
    }];
    
    [self.loadingLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.gameView);
        make.centerY.equalTo(self.gameView);
    }];
    
    [self.playGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.gameView).offset(-5);
        make.bottom.equalTo(self.gameView).offset(-kTLBottomStatusBarHeight);
        make.height.mas_equalTo(36);
        make.width.mas_equalTo([NSBundle isChineseLanguage] ? 77 : 107);
    }];
    
    [self.playShowGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    [self.closeGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    [self.leaveGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.playGameBtn);
    }];
    
    [self.menuZoneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.memberCountBtn.mas_left);
        make.top.mas_equalTo(self.memberCountBtn.mas_top);
        make.height.mas_equalTo(36.0);
        make.width.mas_equalTo(120);
    }];
    
//    [self.menuZoneView mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.left.mas_equalTo(self.gameView).offset(5);
//        make.top.mas_equalTo(self.topView.mas_top).offset(10);
//        make.height.mas_equalTo(ZEGO_HEIGHT - 190 - TOP_BAR_HEIGHT);
//        make.width.mas_equalTo(kScreenWidth - 10);
//    }];
    
    [self.menuZoneView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(ZegoKeyWindow.mas_left).offset(5);
        make.top.mas_equalTo(ZegoKeyWindow.mas_top).offset(140);
        make.height.mas_equalTo(ZEGO_HEIGHT - 190 - TOP_BAR_HEIGHT);
        make.width.mas_equalTo(kScreenWidth - 10);
    }];
    
    [self.menuZoneTable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.equalTo(self.menuZoneView);
        make.height.mas_equalTo(330);
        make.width.mas_equalTo(258);
    }];
}
#pragma mark----getter
- (UIView *)gameView {
    if (!_gameView) {
        _gameView = [[UIView alloc] init];
        _gameView.backgroundColor = [UIColor orangeColor];
    }
    return _gameView;
}

- (UIView *)topView {
    if (!_topView) {
        _topView = [[UIView alloc] init];
        _topView.backgroundColor = [UIColor grayColor];
    }
    return _topView;
}

- (UIButton *)playGameBtn {
    if (!_playGameBtn) {
        _playGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(chose_game)];
        _playGameBtn.layer.cornerRadius = 18;
        _playGameBtn.layer.masksToBounds = YES;
        _playGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_playGameBtn addTarget:self action:@selector(onClickPlayGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playGameBtn;
}

- (UIButton *)closeGameBtn {
    if (!_closeGameBtn) {
        _closeGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(close_game)];
        _closeGameBtn.layer.cornerRadius = 18;
        _closeGameBtn.layer.masksToBounds = YES;
        _closeGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_closeGameBtn addTarget:self action:@selector(onClickCloseGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _closeGameBtn;
}

- (UIButton *)leaveGameBtn {
    if (!_leaveGameBtn) {
        _leaveGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(quit_game)];
        _leaveGameBtn.layer.cornerRadius = 18;
        _leaveGameBtn.layer.masksToBounds = YES;
        _leaveGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _leaveGameBtn.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        [_leaveGameBtn addTarget:self action:@selector(onClickLeaveGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _leaveGameBtn;
}

- (UIButton *)playShowGameBtn {
    if (!_playShowGameBtn) {
        _playShowGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(chose_game)];
        _playShowGameBtn.layer.cornerRadius = 18;
        _playShowGameBtn.layer.masksToBounds = YES;
        _playShowGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        _playShowGameBtn.backgroundColor = [UIColor colorWithHexString:@"5D35FF"];
        [_playShowGameBtn addTarget:self action:@selector(onPlayShowGameBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playShowGameBtn;
}

- (NSMutableDictionary *)gameMap {
    if (!_gameMap) {
        _gameMap = [NSMutableDictionary dictionary];
    }
    return _gameMap;
}

- (ZegoAudioroomMembersView *)seatView {
    if(!_seatView) {
        _seatView = [[ZegoAudioroomMembersView alloc] initWithFrame:CGRectZero];
        _seatView.hidden = YES;
//        [self.view addSubview:_seatView];
    }
    return _seatView;
}

- (UIImageView *)gameIconView {
    if(!_gameIconView) {
        _gameIconView = [[UIImageView alloc] init];
    }
    return _gameIconView;
}

- (UILabel *)gameNameView {
    if(!_gameNameView) {
        _gameNameView = [[UILabel alloc] init];
        _gameNameView.textColor = UIColor.whiteColor;
        _gameNameView.font = [UIFont systemFontOfSize:16];
    }
    return _gameNameView;
}

- (UIButton *)startGameButton {
    if(!_startGameButton) {
        _startGameButton = [[UIButton alloc] init];
        _startGameButton.layer.cornerRadius = 18;
        _startGameButton.layer.masksToBounds = YES;
        _startGameButton.titleLabel.font = BOLD_FONT(14);
        [_startGameButton setBackgroundColor:[UIColor colorWithHexString:@"6D24F6"]];
        [_startGameButton setTitle:TLLocalizedString(start_game) forState:UIControlStateNormal];
        [_startGameButton addTarget:self action:@selector(onClickStartGameAction:) forControlEvents:UIControlEventTouchUpInside];
        _startGameButton.hidden = YES;
    }
    return _startGameButton;
}

- (void)dealloc {
    [self.rtcManager removeRTCEventListener:self];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationWillTerminateNotification object:nil];
}
@end
