//
//  ZegoCloudGameViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/9/8.
//

#import "ZegoCloudGameViewController.h"
#import <ZegoMiniGameEngine/ZegoMiniGameEngine.h>
#import "MenuTableViewCell.h"
#import "ZegoRTCEngineManager.h"
#import "ChargeUtils.h"
#import "KeyCenter.h"
#import "TLArrangePopView.h"

#define CLOUD_GAME_STREAMID ([NSString stringWithFormat:@"%@cloud_game_streamID",STREAM_PREFIX])
#define CLOUD_GAME_ROOMID ([NSString stringWithFormat:@"%@%@3", LIVE_ROOM_PREFIX,ROOM_PREFIX_LIVE])

@interface ZegoCloudGameViewController ()<UITableViewDelegate, UITableViewDataSource, ZegoCloudGameEngineHandler>
// 开始游戏按钮
@property (nonatomic, strong) UIButton *startGameButton;

// 开始游戏按钮
@property (nonatomic, strong) UIButton *stopGameButton;

// 开始推流
@property (nonatomic, strong) UIButton *startPublishButton;

// 停止推流
@property (nonatomic, strong) UIButton *stopPublishButton;

// 游戏容器
@property (nonatomic, strong) UIView *gameContainView;

// 房间ID
@property (nonatomic, copy) NSString *roomID;

// 游戏开播码
@property (nonatomic, copy) NSString *gameLaunchCode;

// 选择房间view
@property (nonatomic, strong) TLArrangePopView *arrangeView;

// 是否登录
@property (nonatomic, assign) BOOL isLogin;

// 功能测试相关
@property (nonatomic, strong) UITableView *menuZoneTable;
@property (nonatomic, strong) UIButton *menuZoneBtn;
@property (nonatomic, strong) NSArray *menuZoneData;
@property (nonatomic, copy) NSString *valueStr;

@end

@implementation ZegoCloudGameViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    if (![ZegoRTCEngineManager.sharedInstance isInRoom]) {
        [ZegoRTCEngineManager.sharedInstance logoutRoom];
    }
    
    [[ZegoCloudGameEngine sharedEngine] setCloudGameContainer:self.gameContainView];
    [[ZegoCloudGameEngine sharedEngine] setCloudGameEngineHandler:self];
    [self initMenu];
    [self setupBaseUI];
    [self layout];
}

- (void)setupBaseUI {
    [self.view addSubview:self.gameContainView];
    [self.view addSubview:self.menuZoneBtn];
    [self.view addSubview:self.menuZoneTable];
    
    
    NSMutableArray *arrangeData = [NSMutableArray array];
    for (int i=0; i<6; i++) {
        NSString *roomID = [NSString stringWithFormat:@"%@%@%d", LIVE_ROOM_PREFIX,ROOM_PREFIX_LIVE,i+1];
        NSDictionary *dic = @{@"name" : roomID};
        [arrangeData addObject:dic];
    }
    self.arrangeView.arrangeData = arrangeData;
    @weakify(self)
    self.arrangeView.arrangeblock = ^(NSDictionary * _Nonnull selectedData){
        @strongify(self)
        if(self.roomID.length > 0 && ![self.roomID isEqualToString:selectedData[@"name"]]) {
            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"已选择房间%@，请退回上级界面",self.roomID] withFinishBlock:nil];
            return;
        }
        self.roomID = selectedData[@"name"];
        self.title = self.roomID;
        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"%@",self.roomID] withFinishBlock:nil];
    };
    [ZegoKeyWindow addSubview:self.arrangeView];
}

- (void)layout {
    [self.gameContainView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.view.mas_top).offset(100);
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.bottom.mas_equalTo(self.view.mas_bottom);
    }];
    
    [self.menuZoneBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.view.mas_centerX);
        make.top.mas_offset(88);
        make.width.mas_equalTo(120);
    }];
    
    [self.menuZoneTable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view).offset(5);
        make.top.mas_equalTo(self.menuZoneBtn.mas_bottom).offset(10);
        make.height.mas_equalTo(330);
        make.width.mas_equalTo(258);
    }];
}

- (void)onClickMenuZoneAction:(UIButton *)sender {
    self.menuZoneTable.hidden = !self.menuZoneTable.hidden;
}

- (void)onCloudGameError:(int)errorCode message:(NSString *)message {
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d-message:%@",errorCode,message] withFinishBlock:nil];
    NSLog(@"errorcode:%d-message:%@", errorCode, message);
    if(errorCode == 8210013) {
//        [[ZegoCloudGameEngine sharedEngine] stopCloudGame:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID callBack:^(int errorCode, id  _Nullable object) {
//            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
//        }];
//        [self startGame];
    }
}

- (void)onCloudGameFirstFrame {

    NSLog(@"onCloudGameFirstFrame");
    self.menuZoneTable.hidden = YES;
}

- (void)dealloc {
    [[ZegoCloudGameEngine sharedEngine] stopCloudGame:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID callBack:^(int errorCode, id  _Nullable object) {
        NSLog(@"dealloc stopCloudGame errorcode:%d!",errorCode);
        [ZegoRTCEngineManager.sharedInstance logoutRoom];
        
//        [ZegoMiniGameEngine.sharedEngine unInit];
    }];
    
    
}

- (void)startGame:(NSDictionary *)gameConfig {
    
    if(self.isLogin) {
        [[ZegoCloudGameEngine sharedEngine] startCloudGame:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID gameConfig:gameConfig callBack:^(int errorCode, id  _Nullable object) {
            [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
        }];
    } else {
        @weakify(self);
        [ZegoRTCEngineManager.sharedInstance loginRoom:self.roomID userID:self.viewModel.userID userName:self.viewModel.userName completion:^(int errorCode, NSDictionary * _Nonnull extendedData) {
            @strongify(self)
            if(errorCode) {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:@"login errorCode:%d", errorCode]  withFinishBlock:nil];
            } else {
                self.isLogin = YES;
                self.viewModel.roomID = self.roomID;
                /**
                 * 命令行启动，增加三个参数：width height auto 分别表示宽度，高度和自动登录。
                 * 注：如果有width和height的话，就会禁用分辨率自定义，且width必须和height成对出现，auto只有token存在才可以有效
                 * TinyLoveWar.exe token=101312 width=607 height=1000 auto=1
                 * 其中"token=101312 width=607 height=1000 auto=1"为运行参数  且width必须和height成对出现
                 * 此处每个字段间必须要有一个空格符
                 */
                if(self.gameLaunchCode.length) {
                    [[ZegoCloudGameEngine sharedEngine] startCloudGame:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID gameConfig:gameConfig callBack:^(int errorCode, id  _Nullable object) {
                        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
                    }];
                }
            }
           
            
        }];
    }
    
}

#pragma 功能菜单测试
- (void)initMenu {
    @weakify(self)
    self.menuZoneData = @[
        @{@"name" : TLLocalizedString(choose_room),
          @"block" : ^{
              @strongify(self)
              self.arrangeView.hidden = !self.arrangeView.hidden;
          }
        },
        
        @{@"name" : TLLocalizedString(get_game_launch_code),
          @"block" : ^{
              @strongify(self)
              [ChargeUtils getGameLaunchCode:KeyCenter.appID gameID:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID userID:self.viewModel.userID userName:self.viewModel.userName avatar:self.viewModel.userAvatar sex:1 completion:^(NSInteger errorCode, NSString * _Nonnull gameLaunchCode) {
                  @strongify(self);
                  if(errorCode == 0) {
                      self.gameLaunchCode = gameLaunchCode;
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"%@",gameLaunchCode] withFinishBlock:nil];
                  } else {
                      [MBProgressHUD showSuccess:[NSString stringWithFormat:@"getGameLaunchCode errorcode:%ld",errorCode] withFinishBlock:nil];
                  }
              }];
          }
        },
        
        @{@"name" : TLLocalizedString(clear_game_launch_code),
          @"block" : ^{
              @strongify(self)
              self.gameLaunchCode = nil;
          }
        },
        
        @{@"name" : TLLocalizedString(start_cloud_game),
          @"block" : ^{
              @strongify(self)
              NSString *parameters = [NSString stringWithFormat:@"token=%@ width=1080 height=1920 auto=1 -popupwindow", self.gameLaunchCode];
              if([self.gameID isEqualToString:@"EnemyAtTheGates"]) { // 兵临城下
                  parameters = [NSString stringWithFormat:@"-c %@ kcr width=1080 height=1920 auto=1", self.gameLaunchCode];
              }
              NSDictionary *gameConfig = nil;
              if(self.gameLaunchCode.length) {
                  gameConfig = @{
//                      @"resolution" : @"1080x1920",
//                      @"codecType" : @(18),
//                      @"bitRate" : @(wlBitRate),
//                      @"fps" : @(30),
                      @"runningParameters" :parameters //此处每个字段间必须要有一个空格符
                  };
              }
              [self startGame:gameConfig];
          }
        },
        
        @{@"name" : TLLocalizedString(start_cloud_game-wl),
          @"block" : ^{
              @strongify(self)
              int wlBitRate = self.valueStr.intValue;
              
              NSString *parameters = [NSString stringWithFormat:@"token=%@ width=1080 height=1920 auto=1 -popupwindow", self.gameLaunchCode];
              if([self.gameID isEqualToString:@"EnemyAtTheGates"]) { // 兵临城下
                  parameters = [NSString stringWithFormat:@"-c %@ kcr width=1080 height=1920 auto=1", self.gameLaunchCode];
              }
              NSDictionary *gameConfig = nil;
              if(self.gameLaunchCode.length) {
                  gameConfig = @{
                      @"resolution" : @"1080x1920",
                      @"codecType" : @(18),
                      @"bitRate" : @(wlBitRate),
                      @"fps" : @(30),
                      @"runningParameters" :parameters //此处每个字段间必须要有一个空格符
                  };
              }
              
              [self startGame:gameConfig];
          }
        },
        
        @{@"name" : TLLocalizedString(stop_cloud_game),
          @"block" : ^{
              @strongify(self)
              [[ZegoCloudGameEngine sharedEngine] stopCloudGame:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID callBack:^(int errorCode, id  _Nullable object) {
                  [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
              }];
              
              [ZegoRTCEngineManager.sharedInstance logoutRoom];
              self.isLogin = NO;
          }
        },
        
        @{@"name" : TLLocalizedString(start_publish_cloud_stream),
          @"block" : ^{
              @strongify(self)
              [[ZegoCloudGameEngine sharedEngine] startPublishCloudGameStream:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID streamID:CLOUD_GAME_STREAMID callBack:^(int errorCode, id  _Nullable object) {
                  NSLog(@"startPublishCloudGameStream ~~~~~~errorcode:%d, object:%@", errorCode, object);
                  [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
              }];
          }
        },
        
        @{@"name" : TLLocalizedString(stop_publish_cloud_stream),
          @"block" : ^{
              @strongify(self)
              [[ZegoCloudGameEngine sharedEngine] stopPublishCloudGameStream:self.gameID ? self.gameID : @"TinyLoveWar" roomID:self.roomID streamID:CLOUD_GAME_STREAMID callBack:^(int errorCode, id  _Nullable object) {
                  NSLog(@"stopPublishCloudGameStream ~~~~~~errorcode:%d, object:%@", errorCode, object);
                  [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
              }];
          }
        },
        
        @{@"name" : TLLocalizedString(set_stream_profile),
          @"block" : ^{
              ZegoStreamProfile *profile = [[ZegoStreamProfile alloc] initWithFps:55 minBitrate:1 maxBitrate:2];
              [[ZegoCloudGameEngine sharedEngine] setStreamProfile:profile];
          }
        },
        
        @{@"name" : TLLocalizedString(set_cloud_volume),
          @"block" : ^{
              [[ZegoCloudGameEngine sharedEngine] setPlayVolume:50];
          }
        },
        
        @{@"name" : TLLocalizedString(get_cloud_volume),
          @"block" : ^{
              [[ZegoCloudGameEngine sharedEngine] getPlayVolume:^(int errorCode, id  _Nullable object) {
                  [MBProgressHUD showSuccess:[NSString stringWithFormat:@"volume:%@",object[@"value"]] withFinishBlock:nil];
              }];
          }
        },
        
        @{@"name" : TLLocalizedString(send_keyboard_key),
          @"block" : ^{
              @strongify(self);
              [[ZegoCloudGameEngine sharedEngine] sendKeyboardEvent:self.valueStr.intValue];
          }
        },
        
    ];
}

#pragma mark - TableViewDelegate & DataSource
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    MenuTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"MenuTableViewCell" forIndexPath:indexPath];
    if (!cell) {
        cell = [[MenuTableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"MenuTableViewCell"];
    }
    
    if (indexPath.row == 10 || indexPath.row ==4) {
        [cell setMenuDictionary:self.menuZoneData[indexPath.row] isShow:YES];
    } else {
        [cell setMenuDictionary:self.menuZoneData[indexPath.row] isShow:NO];
    }
    @weakify(self)
    cell.selectedBlock = ^(NSString *value) {
        @strongify(self)
        self.valueStr = value;
    };
    return cell;
}

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.menuZoneData.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSDictionary *dict = self.menuZoneData[indexPath.row];
    dispatch_block_t block = dict[@"block"];
    if (block) {
        block();
    }
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 50;
}

#pragma mark menu
- (UIView *)gameContainView {
    if(_gameContainView == nil) {
        _gameContainView = [[UIView alloc] init];
    }
    return _gameContainView;
}

- (UIButton *)menuZoneBtn {
    if (!_menuZoneBtn) {
        _menuZoneBtn = [UIButton actionButtonWithTitle:TLLocalizedString(show_features)];
        [_menuZoneBtn addTarget:self action:@selector(onClickMenuZoneAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _menuZoneBtn;
}

- (UITableView *)menuZoneTable {
    if (!_menuZoneTable) {
        _menuZoneTable = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, 0, 0)];
        _menuZoneTable.delegate = self;
        _menuZoneTable.dataSource = self;
        _menuZoneTable.hidden = YES;
        [_menuZoneTable registerClass:[MenuTableViewCell class] forCellReuseIdentifier:@"MenuTableViewCell"];
        _menuZoneTable.backgroundColor = [UIColor whiteColor];
        if (@available(iOS 15.0, *)) {
            [UITableView appearance].sectionHeaderTopPadding = 0;
        }
    }
    return _menuZoneTable;
}

- (TLArrangePopView *)arrangeView {
    if (!_arrangeView) {
        _arrangeView = [[TLArrangePopView alloc] initWithFrame:self.view.bounds
                                                 showViewFrame:CGRectMake(80,
                                                                          TOP_BAR_HEIGHT + 105,
                                                                          180,
                                                                          200)
                                                   clickHidden:YES];
        _arrangeView.userInteractionEnabled = YES;
        _arrangeView.hidden = YES;
    }
    return _arrangeView;
}
@end
