//
//  ZegoGameRoomViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/12/29.
//

#import "ZegoGameRoomViewController.h"

@interface ZegoGameRoomViewController ()<RTCEventListener, ZegoGameEngineHandler>

// 保存玩家Specker状态
@property (nonatomic, strong) NSMutableDictionary *playersState;

@property (nonatomic, assign) BOOL isPublish;

@end

@implementation ZegoGameRoomViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}


- (void)joinLiveRoom {
    [self.rtcManager initAudioGameConfig];
    
    NSArray<StreamInfo *> * streamList = [self.rtcManager getStreamInfoList];
    kLog(@"streamList is %ld", streamList.count)
    if (self.viewModel.isHost) {
        for (StreamInfo *info in streamList) {
            if([info.streamID hasSuffix:HOSTFLAG]) {
                [LEEAlert alert].config.LeeTitle(TLLocalizedString(notify)).LeeContent(TLLocalizedString(live_room_is_occupied)).LeeAction(TLLocalizedString(confirm), ^{
                    [self.navigationController popViewControllerAnimated:YES];
                }).LeeShow();
                return;
            }
        }

        [self.rtcManager startPublishStream:YES];
        self.isPublish = true;
    }
    [self handleStreamsAdded:streamList];
    [self updateButtonVisibility];
}

- (void)handleStreamsAdded:(NSArray <StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        if ([streamInfo.streamID hasPrefix:STREAM_PREFIX]) {
            if ([streamInfo.streamID isEqualToString:self.rtcManager.publishStreamID]) {
                continue;
            }
            
            kLog(@"startPlayingStream streamInfo.streamID is %@", streamInfo.streamID);
            // playersState存在当前流新增对应的userID, 拉流根据保存的mute状态拉流
            if([self.playersState.allKeys containsObject:streamInfo.userID]) {
                [self.rtcManager muteSpeaker:streamInfo.userID mute:[self.playersState[streamInfo.userID] boolValue]];
            } else { //playersState不存在当前流新增对应的userID, 拉静音流
                [self.rtcManager muteSpeaker:streamInfo.userID mute:true];
            }
            [self.rtcManager startPlayingStream:streamInfo.streamID renderView:nil];
        }
        
    }
    [self refreshHostView];
}

#pragma mark----RTCEventListener

- (void)onStreamAdd:(NSArray<StreamInfo *> *)streamInfoList {
    [self handleStreamsAdded:streamInfoList];
}

- (void)onStreamRemove:(NSArray<StreamInfo *> *)streamInfoList {
    for (StreamInfo *streamInfo in streamInfoList) {
        [self.rtcManager stopPlayingStream:streamInfo.streamID];
        [self refreshHostView];
    }
}

- (void)onRoomPublisherStreamEvent:(ZegoStreamEvent)eventID streamID:(NSString *)streamID extraInfo:(NSString *)extraInfo {
    // 断线重连需要重新刷新玩家状态（麦克风和扬声器）
    if(eventID == ZegoStreamEventRetryPublishSuccess) {
        [ZegoMiniGameEngine.sharedEngine refreshMicAndSpeakerState];
    }
}

#pragma mark ZegoGameEngineHandler

- (void)onMicStateChange:(BOOL)isMute {
    NSLog(@"onMicStateChange mute %d", isMute);
    [self.rtcManager muteMicrophone:isMute];
    
    // 检查自己是否有推流，没有推流则推流，下次回调再来则跳过
    if(!self.isPublish) {
        [self.rtcManager startPublishStream:NO];
    }
}

// 听不听userIDList 的声音
- (void)onSpeakerStateChange:(NSArray<NSString *> *)userIDList isMute:(BOOL)isMute {
    NSLog(@"onSpeakerStateChange: %d, userIDCount:%ld", isMute, userIDList.count);
    for (NSString *userID in userIDList) {
        if(![userID isEqualToString:self.viewModel.userID]) { // 过滤掉自己
            [self.rtcManager muteSpeaker:userID mute:isMute];
            NSLog(@"onMicStateChange mute: %d, userID:%@", isMute, userID);
            [self.playersState setObject:@(isMute) forKey:userID];
        }
    }
}

- (void)onGameStateUpdate:(ZegoGameState)gameState {
    kLog(@"onGameStateUpdate：%ld", gameState);
    if(gameState == ZegoGameOver && self.viewModel.isHost) {
        self.startGameButton.hidden = NO;
    }
    // 游戏结束停止推流
    if(gameState == ZegoGameOver ){
        if(self.isPublish && !self.viewModel.isHost) {
            [self.rtcManager stopPublishStream];
            self.isPublish = false;
        }
        [self.playersState removeAllObjects];
    }
}


- (NSMutableDictionary *)playersState {
    if(_playersState == nil) {
        _playersState = [NSMutableDictionary dictionary];
    }
    return _playersState;
}

@end
