//
//  CloudGameView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/9/19.
//

#import "CloudGameView.h"
#import "GameCollectionViewCell.h"

@interface CloudGameView ()<UICollectionViewDelegate, UICollectionViewDataSource>

@end

@implementation CloudGameView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    [self addSubview:self.collectionView];
    
    [self.collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self);
        make.left.bottom.equalTo(self);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.collectionView reloadData];
}

// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.gameList.count;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    GameCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"GameCollectionViewCell" forIndexPath:indexPath];
    UIView *selectedBackgroundView = [[UIView alloc] init];
    selectedBackgroundView.layer.borderColor = [UIColor colorWithHexString:@"987EFF"].CGColor;
    selectedBackgroundView.layer.borderWidth = 1.5f;
    selectedBackgroundView.layer.cornerRadius = 10;
    selectedBackgroundView.layer.masksToBounds = YES;
    cell.selectedBackgroundView = selectedBackgroundView;
    [cell setMiniGameInfo:self.gameList[indexPath.row]];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if ([self.delegate respondsToSelector:@selector(selectCloudGameActionCellWithRow:)]) {
        [self.delegate selectCloudGameActionCellWithRow:indexPath.row];
    }
}

#pragma mark - getter

- (UICollectionView *)collectionView {
    if (!_collectionView) {
        UICollectionViewFlowLayout* layout = [[UICollectionViewFlowLayout alloc]init];
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 18;
        layout.minimumInteritemSpacing = 12;
        layout.sectionInset = UIEdgeInsetsMake(0, 16, 18, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 56) / 3,  (kScreenWidth - 56) / 3 + 26);
        _collectionView = [[UICollectionView alloc] initWithFrame:self.frame collectionViewLayout:layout];
        _collectionView.backgroundColor = [UIColor clearColor];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        [_collectionView registerClass:[GameCollectionViewCell class] forCellWithReuseIdentifier:@"GameCollectionViewCell"];
    }
    return _collectionView;
}
@end
