//
//  GameCollectionViewCell.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/1/10.
//

#import "GameCollectionViewCell.h"

@interface GameCollectionViewCell ()

@property(nonatomic, strong) UIImageView *imageView;
@property(nonatomic, strong) UILabel *nameLabel;
@end

@implementation GameCollectionViewCell

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self createUI];
    }
    return self;
}

- (void)setMiniGameInfo:(ZegoGameInfo *)gameInfo {
    [self.imageView setImageWithURL:[NSURL URLWithString:gameInfo.thumbnail] placeholder:[UIImage imageNamed:@""]];
    self.nameLabel.text = gameInfo.gameName;
}

- (void)createUI {
    CGFloat imageWidth = self.width;
    [self.contentView addSubview:self.imageView];
    [self.contentView addSubview:self.nameLabel];
    
    self.layer.cornerRadius = 8.0f;
    self.layer.masksToBounds = YES;
    
    [self.imageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.equalTo(self.contentView).offset(6);
        make.width.height.mas_equalTo(imageWidth - 12);
    }];
    
    [self.nameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.imageView.mas_bottom).offset(8);
        make.centerX.equalTo(self.imageView.mas_centerX);
        make.width.mas_equalTo(imageWidth);
        make.height.mas_equalTo(18);
    }];
}

- (UIImageView *)imageView {
    if (!_imageView) {
        _imageView = [[UIImageView alloc] init];
        _imageView.contentMode = UIViewContentModeScaleAspectFill;
        _imageView.backgroundColor = [UIColor whiteColor];
        _imageView.userInteractionEnabled = YES;
        _imageView.layer.cornerRadius = 8.0f;
        _imageView.layer.masksToBounds = YES;
    }
    return _imageView;
}

- (UILabel *)nameLabel {
    if(!_nameLabel) {
        _nameLabel = [[UILabel alloc] init];
        _nameLabel.textColor = [UIColor whiteColor];
        _nameLabel.font = REGULAR_FONT(13);
        _nameLabel.textAlignment = NSTextAlignmentCenter;
        [_nameLabel sizeToFit];
    }
    return _nameLabel;
}
@end
