//
//  GameTableViewCell.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/2/3.
//

#import "GameTableViewCell.h"
#import "ChargeGameInfo.h"

@interface GameTableViewCell ()

@property (nonatomic, strong) UIImageView *gameImageView;
@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UIButton *exchangeButton;
@property (nonatomic, strong) ChargeGameInfo *gameInfo;
@end

@implementation GameTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if(self){
        [self createUI];
    }
    return self;
}

- (void)createUI {
    [self addSubview:self.gameImageView];
    [self addSubview:self.nameLabel];
    [self addSubview:self.gameCoinsAmountLabel];
    [self.contentView addSubview:self.exchangeButton];
    
    [self.gameImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self).offset(10);
        make.bottom.equalTo(self).offset(-8);
        make.left.mas_equalTo(20);
        make.width.height.mas_equalTo(60);
    }];
    
    [self.nameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.left.equalTo(self.gameImageView.mas_right).offset(18);
        make.height.mas_equalTo(20);
    }];
    
    [self.gameCoinsAmountLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.nameLabel.mas_right).offset(5);
        make.centerX.equalTo(self);
        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.height.mas_equalTo(20);
    }];
    
    [self.exchangeButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.mas_right).offset(-20);
        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(32);
    }];
}

- (void)setMiniGameInfo:(ChargeGameInfo *)gameInfo {
    _gameInfo = gameInfo;
    [self.gameImageView setImageWithURL:[NSURL URLWithString:gameInfo.gameInfo.thumbnail] placeholder:[UIImage imageNamed:@""]];
    self.nameLabel.text = gameInfo.gameInfo.gameName;
    self.gameCoinsAmountLabel.text = [NSString stringWithFormat:@"%lu", gameInfo.gameCoin];
}

#pragma mark - action
- (void)onExchangeButtonActionButtonClick:(UIButton *)sender {
    NSLog(@"%@：兑换了10000金币", self.nameLabel.text);
    if (self.exchangeBlock) {
        self.exchangeBlock(self.gameInfo);
    }
}

#pragma mark - getter
- (UIImageView *)gameImageView {
    if (!_gameImageView) {
        _gameImageView = [[UIImageView alloc] init];
        _gameImageView.contentMode = UIViewContentModeScaleAspectFill;
        _gameImageView.backgroundColor = [UIColor whiteColor];
        _gameImageView.layer.cornerRadius = 8.0f;
        _gameImageView.layer.masksToBounds = YES;
    }
    return _gameImageView;
}

- (UILabel *)nameLabel {
    if(!_nameLabel) {
        _nameLabel = [[UILabel alloc] init];
        _nameLabel.textColor = [UIColor whiteColor];
        _nameLabel.font = [UIFont systemFontOfSize:12];
        _nameLabel.textAlignment = NSTextAlignmentCenter;
        [_nameLabel sizeToFit];
    }
    return _nameLabel;
}

- (UILabel *)gameCoinsAmountLabel {
    if (!_gameCoinsAmountLabel) {
        _gameCoinsAmountLabel = [[UILabel alloc] init];
        _gameCoinsAmountLabel.text = @"10000";
        _gameCoinsAmountLabel.textColor = [UIColor whiteColor];
        _gameCoinsAmountLabel.font = [UIFont systemFontOfSize:12];
        _gameCoinsAmountLabel.textAlignment = NSTextAlignmentCenter;
        [_gameCoinsAmountLabel sizeToFit];
    }
    return _gameCoinsAmountLabel;
}

- (UIButton *)exchangeButton {
    if (!_exchangeButton) {
        _exchangeButton = [UIButton actionButtonWithTitle:[NSString stringWithFormat: TLLocalizedString(exchange_x_game_coin), 10000]];
        _exchangeButton.titleLabel.font = [UIFont systemFontOfSize:13];
        _exchangeButton.layer.cornerRadius = 16;
        _exchangeButton.layer.masksToBounds = YES;
        [_exchangeButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_exchangeButton addTarget:self action:@selector(onExchangeButtonActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
        _exchangeButton.userInteractionEnabled = YES;
    }
    return _exchangeButton;
}
@end
