//
//  MatchPlayerView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/8/8.
//

#import "MatchPlayerView.h"
#import "UIImageView+YYWebImage.h"

#define ZGRGBColor(rgbValue) [UIColor colorWithRed:((float)((rgbValue & 0xFF0000) >> 16))/255.0 green:((float)((rgbValue & 0xFF00) >> 8))/255.0 blue:((float)(rgbValue & 0xFF))/255.0 alpha:1]

@implementation MatchPlayerUserInfo

@end


@interface MatchPlayerSeatView ()
// 座位按钮
@property (nonatomic, strong) UIButton *seatButton;

// 座位昵称
@property (nonatomic, strong) UILabel *seatLabel;

// 角色Iconview
@property (nonatomic, strong) UIImageView *roleIconView;

// 准备图标view
@property (nonatomic, strong) UIImageView *readyIconView;

// 删除图标view
@property (nonatomic, strong) UIImageView *deleteIconView;

@end

@implementation MatchPlayerSeatView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if(self) {
        [self setupSubViews];
    }
    
    return self;
}

- (void)setupSubViews {
    [self addSubview:self.seatButton];
    [self addSubview:self.seatLabel];
    [self addSubview:self.roleIconView];
    [self addSubview:self.readyIconView];
    [self addSubview:self.deleteIconView];
    
    [self.seatButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.mas_left);
        make.top.mas_equalTo(self.mas_top);
        make.right.mas_equalTo(self.mas_right);
        make.height.mas_equalTo(52.0);
    }];
    
    [self.seatLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.mas_left);
        make.bottom.mas_equalTo(self.mas_bottom);
        make.right.mas_equalTo(self.mas_right);
        make.top.mas_equalTo(self.seatButton.mas_bottom).offset(10);
    }];
    
    [self.roleIconView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.mas_top);
        make.left.mas_equalTo(self.mas_left);
        make.width.height.mas_equalTo(15.0);
    }];
    
    [self.readyIconView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.mas_centerX);
        make.bottom.mas_equalTo(self.seatButton.mas_bottom).offset(7);
        make.width.height.mas_equalTo(15.0);
    }];
    
    [self.deleteIconView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.mas_top);
        make.right.mas_equalTo(self.mas_right);
        make.width.height.mas_equalTo(15.0);
    }];
}

#pragma mark seatbutton
- (void)onClickSeatButton:(UIButton *)sender {
    
}

- (UIButton *)seatButton {
    if(!_seatButton) {
        _seatButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_seatButton setImage:[UIImage imageNamed:@"match_icon_addpeople"] forState:UIControlStateNormal];
        _seatButton.layer.cornerRadius = 52/2;
        [_seatButton addTarget:self action:@selector(onClickSeatButton:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    return _seatButton;
}

- (UILabel *)seatLabel {
    if(!_seatLabel) {
        _seatLabel = [[UILabel alloc] init];
        _seatLabel.numberOfLines = 1;
        _seatLabel.font = [UIFont systemFontOfSize:12];
        _seatLabel.textColor = ZGRGBColor(0xFFFFFF);
    }
    
    return _seatLabel;
}

- (UIImageView *)roleIconView {
    if(!_roleIconView) {
        _roleIconView = [[UIImageView alloc] init];
        _roleIconView.image = [UIImage imageNamed:@"match_icon_role"];
    }
    return _roleIconView;
}

- (UIImageView *)readyIconView {
    if(!_readyIconView) {
        _readyIconView = [[UIImageView alloc] init];
        _readyIconView.image = [UIImage imageNamed:@"match_icon_ready"];
    }
    return _readyIconView;
}

- (UIImageView *)deleteIconView {
    if(!_deleteIconView) {
        _deleteIconView = [[UIImageView alloc] init];
        _deleteIconView.image = [UIImage imageNamed:@"match_icon_delete"];
    }
    return _deleteIconView;
}

@end


@interface MatchPlayerView ()
// 1、管理员 2、可以坐下的人 3、观众
@property (nonatomic, assign) ZegoGameUserType userType;

// 最大玩家人数
@property (nonatomic, assign) int maxPlayer;

// 每个位子的占位信息
@property (nonatomic, strong) NSMutableArray *userList;

// 游戏的图标
@property (nonatomic, copy)   NSString *gameIcon;

// 左边的按钮
@property (nonatomic, strong) UIButton *leftButton;

// 右边的按钮
@property (nonatomic, strong) UIButton *rightButton;

// 中间的按钮
@property (nonatomic, strong) UIButton *centerButton;

@end

@implementation MatchPlayerView

- (instancetype)initWithFrame:(CGRect)frame userType:(ZegoGameUserType)userType maxPlayer:(int)maxPlayer userList:(NSArray <MatchPlayerUserInfo *>*)userList gameIcon:(NSString *)gameIcon {
    self = [super initWithFrame:frame];
    
    if(self) {
        self.userType = userType;
        self.maxPlayer = maxPlayer;
        self.userList = [NSMutableArray arrayWithArray:userList];
        self.gameIcon = gameIcon;
        
        [self setUPSubViews];
    }
    
    return self;
}

- (void)setUPSubViews {
    
}

#pragma mark button action

- (void)onClickLeftButton:(UIButton *)sender {
    
}

- (void)onClickRightButton:(UIButton *)sender {
    
}

- (void)onClickCenterButton:(UIButton *)sender {
    
}

- (UIImageView *)gameIconView {
    if(!_gameIconView) {
        _gameIconView = [[UIImageView alloc] init];
        [_gameIconView setImageWithURL:[NSURL URLWithString:self.gameIcon] placeholder:[UIImage imageNamed:@""]];
    }
    return _gameIconView;
}

- (UIImageView *)gameVSView {
    if(!_gameVSView) {
        _gameVSView = [[UIImageView alloc] init];
        _gameVSView.image = [UIImage imageNamed:@"match_icon_vs"];
    }
    return _gameVSView;
}

- (UIButton *)leftButton {
    if(!_leftButton) {
        _leftButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_leftButton setTitle:@"退出游戏" forState:UIControlStateNormal];
        [_leftButton setTitleColor:ZGRGBColor(0x1D3300) forState:UIControlStateNormal];
        [_leftButton.titleLabel setFont:[UIFont systemFontOfSize:15]];
        _leftButton.backgroundColor =  ZGRGBColor(0xFF76B12B);
        _leftButton.layer.cornerRadius = 22;
        _leftButton.layer.masksToBounds = true;
        [_leftButton addTarget:self action:@selector(onClickLeftButton:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    return _leftButton;
}

- (UIButton *)rightButton {
    if(!_rightButton) {
        _rightButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_rightButton setTitle:@"开始游戏" forState:UIControlStateNormal];
        [_rightButton setTitleColor:ZGRGBColor(0xFF5F2E00) forState:UIControlStateNormal];
        [_rightButton.titleLabel setFont:[UIFont systemFontOfSize:15]];
        _rightButton.backgroundColor =  ZGRGBColor(0xFFD7AD00);
        _rightButton.layer.cornerRadius = 22;
        _rightButton.layer.masksToBounds = true;
        [_rightButton addTarget:self action:@selector(onClickRightButton:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    return _rightButton;
}


- (UIButton *)centerButton {
    if(!_centerButton) {
        _centerButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_centerButton setTitle:@"加入游戏" forState:UIControlStateNormal];
        [_centerButton setTitleColor:ZGRGBColor(0xFF5F2E00) forState:UIControlStateNormal];
        [_centerButton.titleLabel setFont:[UIFont systemFontOfSize:15]];
        _centerButton.backgroundColor =  ZGRGBColor(0xFFD7AD00);
        _centerButton.layer.cornerRadius = 22;
        _centerButton.layer.masksToBounds = true;
        [_centerButton addTarget:self action:@selector(onClickCenterButton:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    return _centerButton;
}


- (NSMutableArray *)userList {
    if(!_userList) {
        _userList = [NSMutableArray array];
    }
    return _userList;
}

@end

