//
//  OpenStreamLiveView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/3/19.
//

#import "OpenStreamLiveView.h"
#import "UIButton+TLButton.h"
#import "CollectionViewCell.h"
#import "ZegoRTCEngineManager.h"
#import "ZGStreamLiveViewController.h"
#import "ZegoGameViewModel.h"
#import "ZegoCollectionReusableView.h"

@interface OpenStreamLiveView ()<UICollectionViewDelegate, UICollectionViewDataSource>

@property (nonatomic, strong) UICollectionView* collectionView;
@property (nonatomic, strong) UILabel *titleLabel;
// 蒙板
@property (nonatomic, weak) UIView *becloudView;

// 回调
@property (nonatomic, copy) JoinLiveRoomPKCompletion joinLiveRoomPKCompletion;

@property (nonatomic, strong) NSMutableArray *selectedArray;
@end

@implementation OpenStreamLiveView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    [self addSubview:self.collectionView];
    
    self.collectionView.userInteractionEnabled = YES;
    [self.collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self);
        make.height.mas_equalTo(kScreenHeight - 140 - TOP_BAR_HEIGHT - kTLBottomStatusBarHeight);
        make.top.mas_equalTo(self.titleLabel.top).offset(2);
        make.bottom.equalTo(self.mas_bottom);
    }];
    
    [self.collectionView reloadData];
}

#pragma mark - 接收传过来的block
- (void)joinLiveRoomPKClickBlock:(JoinLiveRoomPKCompletion)joinLiveRoomPKCompletion {
    self.joinLiveRoomPKCompletion = joinLiveRoomPKCompletion;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 3;
}

// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return 6;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    CollectionViewCell *cell  = [collectionView dequeueReusableCellWithReuseIdentifier:@"CollectionViewCell" forIndexPath:indexPath];
    cell.backgroundColor = [UIColor colorWithRGB:0xFF574979 alpha:0.32];
    if(indexPath.section == 0) {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(audio_room), (long)indexPath.row + 1];
    } else if(indexPath.section == 1) {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(live_room), (long)indexPath.row + 1];
    } else {
        cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(game_room), (long)indexPath.row + 1];
    }
    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    self.viewModel.isHost = YES;
    NSString *newRoomID = nil;
    if (![ZegoRTCEngineManager.sharedInstance isInRoom]) {
        [ZegoRTCEngineManager.sharedInstance logoutRoom];
    }
    if(indexPath.section == 0) {
        self.viewModel.isLive = NO;
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_CHAT, indexPath.row + 1];
    } else if(indexPath.section == 1) {
        self.viewModel.isLive = 1;
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_LIVE, indexPath.row + 1];
    } else {
        self.viewModel.isLive = 2;
        newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX,ROOM_PREFIX_GAME, indexPath.row + 1];
    }
    
    [ZegoRTCEngineManager.sharedInstance loginRoom:newRoomID userID:self.viewModel.userID userName:self.viewModel.userName completion:^(int errorCode, NSDictionary * _Nonnull extendedData) {
        self.viewModel.roomID = newRoomID;
        if (self.joinLiveRoomPKCompletion) {
            self.joinLiveRoomPKCompletion(self.viewModel);
        }
    }];
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section {
    if(section == 1 || section == 2) {
        return CGSizeMake(kScreenWidth, 40);
    } else {
        return CGSizeMake(kScreenWidth, 1);
    }
    
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath {
    if ([kind isEqualToString:UICollectionElementKindSectionHeader]) {
        ZegoCollectionReusableView *headerView = [collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:NSStringFromClass(ZegoCollectionReusableView.class) forIndexPath:indexPath];
        NSString *title = indexPath.section == 0 ? TLLocalizedString(chose_audio_room) : TLLocalizedString(chose_live_room);
        
        return headerView;
    }
    return  nil;
}


#pragma mark - getter

- (UICollectionView *)collectionView {
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc]init];
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 18;
        layout.minimumInteritemSpacing = 12;
        layout.sectionInset = UIEdgeInsetsMake(0, 16, 10, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 56) / 3,  (kScreenWidth - 56) / 3);
        _collectionView = [[UICollectionView alloc] initWithFrame:self.frame collectionViewLayout:layout];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        _collectionView.backgroundColor = [UIColor clearColor];
        [_collectionView registerClass:[CollectionViewCell class] forCellWithReuseIdentifier:@"CollectionViewCell"];
        [_collectionView registerClass:[ZegoCollectionReusableView class] forSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"ZegoCollectionReusableView"];
    }
    return _collectionView;
}

- (UILabel *)titleLabel {
   if (!_titleLabel) {
       _titleLabel = [[UILabel alloc] init];
       _titleLabel.text = TLLocalizedString(chose_live_room);
       _titleLabel.textColor = [UIColor colorWithHexString:@"FFFFFF"];
       _titleLabel.font = REGULAR_FONT(18);
       _titleLabel.textAlignment = NSTextAlignmentRight;
       [_titleLabel sizeToFit];
   }
   return _titleLabel;
}

- (NSMutableArray *)selectedArray {
    if (!_selectedArray) {
        _selectedArray = [NSMutableArray array];
    }
    return _selectedArray;
}

@end
