//
//  PlayGameView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/2/16.
//

#import "PlayGameView.h"
#import "GameCollectionViewCell.h"
@interface PlayGameView ()<UICollectionViewDelegate, UICollectionViewDataSource>

@property (nonatomic, strong) UILabel *tipLabel;
@end

@implementation PlayGameView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    [self addSubview:self.tipLabel];
    [self addSubview:self.collectionView];
    
    [self.tipLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self);
        make.left.equalTo(self.mas_left).offset(16);
        make.width.mas_equalTo(kScreenWidth - 32);
    }];
    
    [self.collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.tipLabel.mas_bottom).offset(18);
        make.left.bottom.equalTo(self);
        make.width.mas_equalTo(kScreenWidth);
    }];
    
    [self.collectionView reloadData];
}

// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.gameList.count;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    GameCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"GameCollectionViewCell" forIndexPath:indexPath];
    UIView *selectedBackgroundView = [[UIView alloc] init];
    selectedBackgroundView.layer.borderColor = [UIColor colorWithHexString:@"987EFF"].CGColor;
    selectedBackgroundView.layer.borderWidth = 1.5f;
    selectedBackgroundView.layer.cornerRadius = 10;
    selectedBackgroundView.layer.masksToBounds = YES;
    cell.selectedBackgroundView = selectedBackgroundView;
    [cell setMiniGameInfo:self.gameList[indexPath.row]];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if ([self.delegate respondsToSelector:@selector(selectGameActionCellWithRow:)]) {
        [self.delegate selectGameActionCellWithRow:indexPath.row];
    }
}

#pragma mark - getter

- (UICollectionView *)collectionView {
    if (!_collectionView) {
        UICollectionViewFlowLayout* layout = [[UICollectionViewFlowLayout alloc]init];
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 18;
        layout.minimumInteritemSpacing = 12;
        layout.sectionInset = UIEdgeInsetsMake(0, 16, 18, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 56) / 3,  (kScreenWidth - 56) / 3 + 26);
        _collectionView = [[UICollectionView alloc] initWithFrame:self.frame collectionViewLayout:layout];
        _collectionView.backgroundColor = [UIColor clearColor];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        [_collectionView registerClass:[GameCollectionViewCell class] forCellWithReuseIdentifier:@"GameCollectionViewCell"];
    }
    return _collectionView;
}

- (UILabel *)tipLabel {
    if(!_tipLabel) {
        _tipLabel = [[UILabel alloc] init];
        _tipLabel.text = TLLocalizedString(mall_note1);
        _tipLabel.numberOfLines = 0;
        _tipLabel.textColor = [UIColor colorWithRed:255/255.0 green:91/255.0 blue:91/255.0 alpha:1/1.0];
        _tipLabel.font = [UIFont systemFontOfSize:12];
        _tipLabel.textAlignment = NSTextAlignmentLeft;
        [_tipLabel sizeToFit];
    }
    return _tipLabel;
}

@end
