//
//  ZGStreamLivePKAlterView.m
//  LiveRoom_DataChannel
//
//  Created by zego on 2022/11/5.
//

#import "ZGStreamLiveAlterView.h"
#import "UIButton+TLButton.h"
#import "CollectionViewCell.h"
#import "ZegoRTCEngineManager.h"
#import "ZGStreamLiveViewController.h"
#import "ZegoGameViewModel.h"

#define ZGWindow [UIApplication sharedApplication].keyWindow

@interface ZGStreamLiveAlterView ()<UICollectionViewDelegate, UICollectionViewDataSource>

@property (nonatomic, strong) UICollectionView* collectionView;
@property (nonatomic, strong) UILabel *titleLabel;
// 蒙板
@property (nonatomic, weak) UIView *becloudView;

// 回调
@property (nonatomic, copy) JoinLiveRoomPKCompletion joinLiveRoomPKCompletion;

@property (nonatomic, strong) NSMutableArray *selectedArray;
@end

@implementation ZGStreamLiveAlterView

- (instancetype)init {
    self = [super init];
    if (self) {
        [self setupUI];
    }
    return self;
}


- (void)setupUI {
    self.backgroundColor = [UIColor colorWithHexString:@"100B18"];
    [self setCornerRadius:self];
    [self addSubview:self.titleLabel];
    [self addSubview:self.collectionView];
    [self initData];
}

- (void)initData {
    
}

#pragma mark - 设置控件圆角
- (void)setCornerRadius:(UIView *)view {
    view.layer.cornerRadius = 5.0;
    view.layer.masksToBounds = YES;
}

#pragma mark - public
- (void)show {
    
    // 蒙版
    UIView *becloudView = [[UIView alloc] initWithFrame:[UIScreen mainScreen].bounds];
    becloudView.backgroundColor = [UIColor blackColor];
    becloudView.layer.opacity = 0.3;
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closeAlertView:)];
    [becloudView addGestureRecognizer:tapGR];
    self.backgroundColor = [UIColor colorWithHexString:@"100B18"];
    [ZGWindow addSubview:becloudView];
    self.becloudView = becloudView;
    [ZGWindow addSubview:self];
    [self mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.equalTo(becloudView);
    }];
    
    [self.titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.mas_top).offset(74);
        make.left.equalTo(self).offset(16) ;
        make.height.mas_equalTo(30);
    }];
    
    self.collectionView.userInteractionEnabled = YES;
    [self.collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self);
        make.height.mas_equalTo(kScreenHeight - 140 - TOP_BAR_HEIGHT - kTLBottomStatusBarHeight);
        make.top.mas_equalTo(self.titleLabel.mas_bottom).offset(29);
        make.bottom.equalTo(self.mas_bottom);
    }];
    
    [self.collectionView reloadData];
}

- (void)setTitleString:(NSString *)titleString {
    _titleString = titleString;
    self.titleLabel.text = _titleString;
}
#pragma mark - 移除ZYInputAlertView
- (void)dismiss {
    [UIView animateWithDuration:0.3 animations:^{
        self.alpha = 0;
    } completion:^(BOOL finished) {
        if (self.dismissLiveViewCompletion) {
            self.dismissLiveViewCompletion();
        }
        [self removeFromSuperview];
        [self.becloudView removeFromSuperview];
        self.becloudView = nil;
    }];
    
}

- (void)closeAlertView:(UIButton *)sender {
    [self dismiss];
    
}

#pragma mark - 接收传过来的block
- (void)joinLiveRoomPKClickBlock:(JoinLiveRoomPKCompletion)joinLiveRoomPKCompletion {
    self.joinLiveRoomPKCompletion = joinLiveRoomPKCompletion;
}

// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return 10;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    CollectionViewCell *cell  = [collectionView dequeueReusableCellWithReuseIdentifier:@"CollectionViewCell" forIndexPath:indexPath];
    cell.backgroundColor = [UIColor colorWithRGB:0xFF574979 alpha:0.32];
    cell.titleLabel.text = [NSString stringWithFormat:@"%@\n%ld", TLLocalizedString(live_room), (long)indexPath.row + 1];
    return cell;
    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    self.viewModel.isHost = YES;
    NSString *newRoomID = [NSString stringWithFormat:@"%@%@%ld", LIVE_ROOM_PREFIX, ROOM_PREFIX_LIVE, indexPath.row + 1];
    if (![ZegoRTCEngineManager.sharedInstance isInRoom]) {
        [ZegoRTCEngineManager.sharedInstance logoutRoom];
    }
    [ZegoRTCEngineManager.sharedInstance loginRoom:newRoomID userID:self.viewModel.userID userName:self.viewModel.userName completion:^(int errorCode, NSDictionary * _Nonnull extendedData) {
        self.viewModel.roomID = newRoomID;
        if (self.joinLiveRoomPKCompletion) {
            self.joinLiveRoomPKCompletion(self.viewModel);
        }
    }];
    [self dismiss];
    
}

#pragma mark - getter

- (UICollectionView *)collectionView {
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc]init];
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 18;
        layout.minimumInteritemSpacing = 12;
        layout.sectionInset = UIEdgeInsetsMake(0, 16, 10, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 56) / 3,  (kScreenWidth - 56) / 3);
        _collectionView = [[UICollectionView alloc] initWithFrame:self.frame collectionViewLayout:layout];
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        _collectionView.backgroundColor = [UIColor colorWithHexString:@"100B18"];
        [_collectionView registerClass:[CollectionViewCell class] forCellWithReuseIdentifier:@"CollectionViewCell"];
    }
    return _collectionView;
}

- (UILabel *)titleLabel {
   if (!_titleLabel) {
       _titleLabel = [[UILabel alloc] init];
       _titleLabel.textColor = [UIColor colorWithHexString:@"FFFFFF"];
       _titleLabel.font = REGULAR_FONT(18);
       _titleLabel.textAlignment = NSTextAlignmentRight;
       [_titleLabel sizeToFit];
   }
   return _titleLabel;
}

- (NSMutableArray *)selectedArray {
    if (!_selectedArray) {
        _selectedArray = [NSMutableArray array];
    }
    return _selectedArray;
}

@end
