//
//  TLNotLoginTopView.m
//  ZegoRoomkitDemo
//
//  Created by KaelDing on 2020/7/15.
//  Copyright © 2020 zego. All rights reserved.
//

#import "ZegoGameTopView.h"

@interface ZegoGameTopView ()

@property (nonatomic, strong) UIButton *quickJoinButton;
@property (nonatomic, strong) UIButton *testLoginButton;

@end

@implementation ZegoGameTopView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    [self addSubview:self.quickJoinButton];
    [self addSubview:self.testLoginButton];
    [self.quickJoinButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self);
        make.left.equalTo(self).offset(30);
        make.height.mas_equalTo(29);
    }];
    [self.testLoginButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.quickJoinButton);
        make.left.equalTo(self.quickJoinButton.mas_right).offset(40);
        make.height.mas_equalTo(29);
    }];
}

#pragma mark - Public
- (void)selectTopViewActionButtonWithType:(ZegoTopButtonType)type {
    [self setButtonSelectedWithType:type];
}

#pragma mark - action
- (void)actionButtonClick:(UIButton *)sender {
    [self setButtonSelectedWithType:sender.tag];
    if ([self.delegate respondsToSelector:@selector(gameTopViewDidClickWithActionButton:)]) {
        [self.delegate gameTopViewDidClickWithActionButton:sender];
    }
}

#pragma mark - Private
- (UIButton *)createButton {
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setTitleColor:[UIColor colorWithHexString:@"2953ff"] forState:UIControlStateSelected];
    [button setTitleColor:[UIColor colorWithHexString:@"040404"] forState:UIControlStateNormal];
    [button addTarget:self action:@selector(actionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    button.titleLabel.font = BOLD_FONT(27);
    [button.titleLabel sizeToFit];
    return button;
}
- (void)setButtonSelectedWithType:(ZegoTopButtonType)type {
    self.quickJoinButton.selected = type == self.quickJoinButton.tag;
    self.testLoginButton.selected = type == self.testLoginButton.tag;
}

#pragma mark - getter
- (UIButton *)quickJoinButton {
    if (!_quickJoinButton) {
        _quickJoinButton = [self createButton];
        [_quickJoinButton setTitle:@"看直播" forState:UIControlStateNormal];
        _quickJoinButton.tag = ZGGameButtonTypeLive;
        _quickJoinButton.selected = YES;
    }
    return _quickJoinButton;
}
- (UIButton *)testLoginButton {
    if (!_testLoginButton) {
        _testLoginButton = [self createButton];
        [_testLoginButton setTitle:@"我的" forState:UIControlStateNormal];
        _testLoginButton.tag = ZGGameButtonTypeMine;
    }
    return _testLoginButton;
}
@end
