//
//  ZegoInviteAlterView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/3/15.
//

#import "ZegoInviteAlterView.h"

#define ZGWindow [UIApplication sharedApplication].keyWindow

@interface ZegoInviteAlterView ()

@property (nonatomic, weak) UIView *becloudView;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *agreeLabel;
@property (nonatomic, strong) UILabel *disagreeLabel;
@property (nonatomic, strong) UIButton *agreeBtn;
@property (nonatomic, strong) UIButton *disagreeBtn;
@property (nonatomic, copy) OnAgreeInviteBlock onInviteBlock;
@end

@implementation ZegoInviteAlterView

+ (instancetype)alertView {
    static ZegoInviteAlterView *helper = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        helper = [[ZegoInviteAlterView alloc] init];
        [helper setupUI];
    });
    return helper;
}

- (void)setupUI {
    self.backgroundColor = [UIColor whiteColor];
    [self setCornerRadius:self];
    [self addSubview:self.titleLabel];
    [self addSubview:self.agreeLabel];
    [self addSubview:self.disagreeLabel];
    [self addSubview:self.agreeBtn];
    [self addSubview:self.disagreeBtn];
}

#pragma mark - 设置控件圆角
- (void)setCornerRadius:(UIView *)view {
    view.layer.cornerRadius = 16.0;
    view.layer.masksToBounds = YES;
}

- (void)show {
    [self dismiss];
    // 蒙版
    UIView *becloudView = [[UIView alloc] initWithFrame:[UIScreen mainScreen].bounds];
    becloudView.backgroundColor = [UIColor blackColor];
    becloudView.layer.opacity = 0.3;
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closeAlertView:)];
    [becloudView addGestureRecognizer:tapGR];
    
    [ZGWindow addSubview:becloudView];
    self.becloudView = becloudView;
    
    // 输入框
    self.frame = CGRectMake(20 , 0, kScreenWidth - 104, [NSBundle isChineseLanguage] ?  225 : 368);
    self.center = CGPointMake(becloudView.centerX, becloudView.centerY);
    [ZGWindow addSubview:self];
    
    [self.titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(22);
        make.width.mas_equalTo(200);
        make.centerX.equalTo(self);
    }];
    
    [self.agreeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.titleLabel.mas_bottom).offset(12);
        make.left.equalTo(self).offset(22);
        make.right.equalTo(self).offset(-22);
    }];
    
    [self.disagreeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.agreeLabel.mas_bottom).offset(8);
        make.left.right.equalTo(self.agreeLabel);
    }];
    
    [self.disagreeBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.disagreeLabel.mas_bottom).offset(19);
        make.left.equalTo(self).offset(21);
        make.width.mas_equalTo((self.width - 50) / 2);
        make.height.mas_equalTo(36);
    }];
    
    [self.agreeBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.width.height.equalTo(self.disagreeBtn);
        make.left.equalTo(self.disagreeBtn.mas_right).offset(8);
        make.right.equalTo(self).offset(-21);
    }];
}

#pragma mark - 移除AlertView

- (void)dismiss {
    [self removeFromSuperview];
    [self.becloudView removeFromSuperview];
    self.becloudView = nil;
}

- (void)closeAlertView:(UIButton *)sender {
    [self dismiss];
}

#pragma mark - 接收传过来的block

- (void)onAgreeInviteBlock:(OnAgreeInviteBlock)block {
    self.onInviteBlock = block;
}


- (void)onClickAgreeBtnAction:(UIButton *)sender {
    [self dismiss];
    self.onInviteBlock(ZGAgreeInviteType);
}

- (void)onClickDisagreeBtnAction:(UIButton *)sender {
    [self dismiss];
    self.onInviteBlock(ZGDisAgreeInviteType);
}
 
#pragma mark - getter

- (UILabel *)titleLabel {
    if (!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.text = TLLocalizedString(invite_join_game);
        _titleLabel.textColor = [UIColor blackColor];
        _titleLabel.font = BOLD_FONT(16);
        _titleLabel.textAlignment = NSTextAlignmentCenter;
        [_titleLabel sizeToFit];
    }
    return _titleLabel;
}

- (UILabel *)agreeLabel {
    if (!_agreeLabel) {
        _agreeLabel = [[UILabel alloc] init];
        _agreeLabel.numberOfLines = 0;
        _agreeLabel.text = TLLocalizedString(invite_join_game_content_1);
        _agreeLabel.textColor = [UIColor blackColor];
        _agreeLabel.font = REGULAR_FONT(14);
        _agreeLabel.textAlignment = NSTextAlignmentLeft;
        [_agreeLabel sizeToFit];
    }
    return _agreeLabel;
}

- (UILabel *)disagreeLabel {
    if (!_disagreeLabel) {
        _disagreeLabel = [[UILabel alloc] init];
        _disagreeLabel.numberOfLines = 0;
        _disagreeLabel.text = TLLocalizedString(invite_join_game_content_2);
        _disagreeLabel.textColor = [UIColor blackColor];
        _disagreeLabel.font = REGULAR_FONT(14);
        _disagreeLabel.textAlignment = NSTextAlignmentLeft;
        [_disagreeLabel sizeToFit];
    }
    return _disagreeLabel;
}

- (UIButton *)agreeBtn {
    if (!_agreeBtn) {
        _agreeBtn = [UIButton actionButtonWithTitle:TLLocalizedString(agree)];
        _agreeBtn.layer.cornerRadius = 18;
        _agreeBtn.layer.masksToBounds = YES;
        _agreeBtn.titleLabel.font = REGULAR_FONT(14);
        [_agreeBtn addTarget:self action:@selector(onClickAgreeBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _agreeBtn;
}

- (UIButton *)disagreeBtn {
    if (!_disagreeBtn) {
        _disagreeBtn = [UIButton actionButtonWithTitle:TLLocalizedString(disagree)];
        _disagreeBtn.layer.cornerRadius = 18;
        _disagreeBtn.layer.masksToBounds = YES;
        _disagreeBtn.backgroundColor = [UIColor colorWithHexString:@"F8F9FB"];
        _disagreeBtn.titleLabel.font = REGULAR_FONT(14);
        [_disagreeBtn setTitleColor:[UIColor colorWithHexString:@"5E6068"] forState:UIControlStateNormal];
        [_disagreeBtn addTarget:self action:@selector(onClickDisagreeBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _disagreeBtn;
}
@end
