//
//  ZegoSelectGamePicker.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/3/16.
//

#import "ZegoSelectGamePicker.h"
#import "GameCollectionViewCell.h"
#import "ZegoPlayGameAlterView.h"

@implementation ZegoJoinPlayGameModel


@end

@interface ZegoSelectGamePicker ()<UICollectionViewDelegate, UICollectionViewDataSource, UIGestureRecognizerDelegate>
/// 容器视图
@property (nonatomic, strong) UIView *contentView;
/// 圆角mask
@property (nonatomic, strong) CAShapeLayer *maskLayer;
// 游戏父容器
@property (nonatomic, strong) UIView *gameView;
// 每局赌注金币view
@property (nonatomic, strong) UIView *goldCoinsView;
// 玩家人数view
@property (nonatomic, strong) UIView *playerNumView;
// 抽水类型view
@property (nonatomic, strong) UIView *pumpingTypeView;
// 抽水比例view
@property (nonatomic, strong) UIView *pumpingRatioView;
// 选择声音播放控制view
@property (nonatomic, strong) UIView *audioView;
// 开始游戏按钮
@property (nonatomic, strong) UIButton *playGameBtn;

// 每局赌注金币label
@property (nonatomic, strong) UIView *goldCoinsLabel;
// 玩家人数label
@property (nonatomic, strong) UIView *playerNumLabel;
// 抽水类型label
@property (nonatomic, strong) UIView *pumpingTypeLabel;
// 抽水类型label
@property (nonatomic, strong) UIView *pumpingRatioLabel;
// 选择声音播放控制label
@property (nonatomic, strong) UIView *audioLabel;
@property (nonatomic, assign) BOOL isShowGround;
// 选择类型
@property (nonatomic, assign) ZegoSelectShowType type;
// 游戏列表
@property (nonatomic, strong) NSArray <ZegoGameInfo *> *gameList;

// 每局赌注金币数据源
@property (nonatomic, strong) NSMutableArray *radioSpinArray;
// 玩家人数数据源
@property (nonatomic, strong) NSMutableArray *radioPlayerArray;
// 选择声音播放控制数据源
@property (nonatomic, strong) NSMutableArray *radioAudioTypeArray;
// 抽水类型数据源
@property (nonatomic, strong) NSMutableArray *radioPumpingTypeArray;
// 抽水比列数据源
@property (nonatomic, strong) NSMutableArray *radioPumpingRatioArray;

@property (nonatomic, strong) NSMutableArray *radioSpinButtons;
@property (nonatomic, strong) NSMutableArray *radioPlayerButtons;
@property (nonatomic, strong) NSMutableArray *radioAudioTypeButtons;
@property (nonatomic, strong) NSMutableArray *radioPumpingTypeButtons;
@property (nonatomic, strong) NSMutableArray *radioPumpingRatioButtons;

@property (nonatomic, assign) NSInteger selectIndex;
// 选择游戏金币数
@property (nonatomic, assign) NSInteger spin;
// 选择游戏玩家人数
@property (nonatomic, assign) int playerNum;
@property (nonatomic, assign) int showModeValue;
// 选择声音播放控制
@property (nonatomic, assign) int soundCustomPlay;
// 选择抽水类型
@property (nonatomic, assign) int pumpingType;
// 选择抽水比列
@property (nonatomic, assign) int pumpingRatio;

// 选择（点击）列表中游戏block
@property (nonatomic, copy) SelectedGameBlock selectedGameBlock;
// 开始游戏block
@property (nonatomic, copy) ZegoPlayGameBlock playGameBlock;

@end

@implementation ZegoSelectGamePicker

- (void)viewDidLoad {
    [super viewDidLoad];
//    [self setupUI];
    [self configureData];
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    self.maskLayer.frame = self.contentView.bounds;
    self.maskLayer.path = [UIBezierPath bezierPathWithRoundedRect:self.maskLayer.frame byRoundingCorners:UIRectCornerTopLeft | UIRectCornerTopRight cornerRadii:CGSizeMake(16, 16)].CGPath;
}


- (void)dealloc {
    
}

#pragma mark private method
/// 设置UI
- (void)setupUI {
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closePickerView:)];
    tapGR.delegate = self;
    
    if(self.type == ZegoSelectTypeFullScreen) {
        UIScrollView *scrollview = [[UIScrollView alloc] init];
        scrollview.backgroundColor = UIColor.clearColor;
        [self.view addSubview:scrollview];
        [scrollview mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self.view);
        }];
        
        [scrollview addSubview:self.contentView];
        
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(scrollview);

            make.width.equalTo(scrollview);
        }];
        
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.playGameBtn.mas_bottom).offset(kTLBottomStatusBarHeight +6);
        }];
    } else {
        [self.view addSubview:self.contentView];
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.equalTo(self.view);
            make.bottom.equalTo(self.view.mas_bottom);
        }];
        
        [self.playGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(_contentView.mas_bottom).offset(- kTLBottomStatusBarHeight - 6);
        }];
    }
    
    [self.contentView addGestureRecognizer:tapGR];
    
    [self.radioAudioTypeArray removeAllObjects];
    int i = 0;
    for (id audioType in @[@(1), @(0)]) {
        RadioModel *audioModel = [[RadioModel alloc] init];
        audioModel.num = [audioType intValue];
        audioModel.tag = [audioType integerValue];
        audioModel.isSelected = [audioType intValue] == 1 ? YES : NO;
        [self.radioAudioTypeArray addObject:audioModel];
        if (i == 0) {
            audioModel.isSelected = YES;
            self.soundCustomPlay = audioModel.tag == 1 ? YES : NO;
        } else {
            audioModel.isSelected = NO;
        }
        i++;
    }
    
    [self.radioPumpingTypeArray removeAllObjects];
    int j = 0;
    for (id pumpingType in @[@(1), @(2)]) {
        RadioModel *pumpingModel = [[RadioModel alloc] init];
        pumpingModel.num = [pumpingType intValue];
        pumpingModel.tag = [pumpingType integerValue];
        pumpingModel.isSelected = [pumpingType intValue] == 1 ? YES : NO;
        [self.radioPumpingTypeArray addObject:pumpingModel];
        if (j == 0) {
            pumpingModel.isSelected = YES;
            self.pumpingType = (int)pumpingModel.tag;
        } else {
            pumpingModel.isSelected = NO;
        }
        j++;
    }
    
    [self.radioPumpingRatioArray removeAllObjects];
    int k = 0;
    for (id pumpingRatio in @[@(0), @(1), @(2)]) {
        RadioModel *pumpingRatioModel = [[RadioModel alloc] init];
        pumpingRatioModel.num = [pumpingRatio intValue];
        pumpingRatioModel.tag = [pumpingRatio integerValue];
        pumpingRatioModel.isSelected = [pumpingRatio intValue] == 0 ? YES : NO;
        [self.radioPumpingRatioArray addObject:pumpingRatioModel];
        if (k == 0) {
            pumpingRatioModel.isSelected = YES;
            self.pumpingRatio = (int)pumpingRatioModel.tag;
        } else {
            pumpingRatioModel.isSelected = NO;
        }
        k++;
    }
    
    
    [self createAudioTypePlayerButtons];
    [self layoutRadioAudioTypeButtons];
    
    [self createPumpingTypePlayerButtons];
    [self layoutRadioPumpingTypeButtons];
    [self createPumpingRatioPlayerButtons];
    [self layoutRadioPumpingRatioButtons];
}

- (void)setType:(ZegoSelectShowType)type {
    _type = type;
    [self setupUI];
}

/// 初始化数据
- (void)configureData {
    self.selectIndex = -1;
}

#pragma mark event method
// 显示
- (void)show {
    if (self.isShowGround) {
        self.goldCoinsView.hidden = YES;
        self.playerNumView.hidden = YES;
        self.pumpingTypeView.hidden = YES;
        self.pumpingRatioView.hidden = YES;
        
        [self.audioView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.gameView.mas_bottom).offset(12);
        }];
        [self.playGameBtn mas_updateConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.audioView.mas_bottom).offset(68);
        }];
    }
    // 动画
    [UIView animateWithDuration:0.2 animations:^{
        self.view.alpha = 1;
    } completion:^(BOOL finished) {
        if(self.type != ZegoSelectTypeFullScreen) {
            [UIView animateWithDuration:0.2 animations:^{
                [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.equalTo(self.view);
                    make.bottom.equalTo(self.view);
                }];
                [self.view layoutIfNeeded];
            }];
        }
    }];
}

- (void)closePickerView:(UIGestureRecognizer *)sender {
    [self dismiss:nil];
}

// 退出
- (void)dismiss:(void (^)(void))completion {
    [self removeRadioAllviews];
    [self.view endEditing:YES];
    [UIView animateWithDuration:0.2 animations:^{
        [self.view layoutIfNeeded];
    } completion:^(BOOL finished) {
        [UIView animateWithDuration:0.2 animations:^{
            self.view.alpha = 0;
        } completion:^(BOOL finished) {
            [self dismissViewControllerAnimated:NO completion:completion];
        }];
    }];
}

#pragma mark protocol method

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch {
    if ([touch.view isEqual:self.view] || [touch.view isEqual:self.contentView]) {
        return YES;
    } else {
        return NO;
    }
}
    
// 返回每个分区的item个数
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.gameList.count;
}

// 返回每个item
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    GameCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"GameCollectionViewCell" forIndexPath:indexPath];
    UIView *selectedBackgroundView = [[UIView alloc] init];
    selectedBackgroundView.layer.borderColor = [UIColor colorWithHexString:@"987EFF"].CGColor;
    selectedBackgroundView.layer.borderWidth = 1.5f;
    selectedBackgroundView.layer.cornerRadius = 10;
    selectedBackgroundView.layer.masksToBounds = YES;
    cell.selectedBackgroundView = selectedBackgroundView;
    [cell setMiniGameInfo:self.gameList[indexPath.row]];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if (!self.isShowGround) {
        [self removeRadioAllviews];
    }
    self.selectIndex = indexPath.row;
    ZegoGameInfo *gameInfo = self.gameList[indexPath.row];
    
    if (self.selectedGameBlock) {
        self.selectedGameBlock(gameInfo);
    }
}

#pragma mark action

- (void)onClickPlayGameAction:(UIButton *)sender {
    if (self.selectIndex == -1) {
        [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(please_chose_a_game)] withFinishBlock:nil];
        return;
    }
    
    ZegoJoinPlayGameModel *model = [[ZegoJoinPlayGameModel alloc] init];
    if (self.isShowGround) {
        model.selectIndex = self.selectIndex;
        model.isSoundCustomPlay = self.soundCustomPlay;

        if (self.playGameBlock) {
            self.playGameBlock(model);
        }
        [self dismiss:nil];
        return;
    }
    if (self.spin == 0) {
//        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"请选择游戏的最小金额数"] withFinishBlock:nil];
//        return;
    }
    
    if (self.playerNum == 0) {
        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"请选择玩家人数"] withFinishBlock:nil];
        return;
    }
    
    model.selectIndex = self.selectIndex;
    model.spin = self.spin;
    model.playerNum = self.playerNum;
    model.isSoundCustomPlay = self.soundCustomPlay;
    model.pumpingType = self.pumpingType;
    model.pumpingRatio = self.pumpingRatio;
   
    if (self.playGameBlock) {
        self.playGameBlock(model);
    }
    [self dismiss:nil];
}

- (void)onRadioSpinButtonClicked:(NSInteger)tag {
    self.spin = tag;
    for (TLRadioButton *spinButton in self.radioSpinButtons) {
        [spinButton setSelected:spinButton.tag == tag];
    }
}

- (void)onRadioPlayerButtonClicked:(NSInteger)tag {
    self.playerNum = (int)tag;
    for (TLRadioButton *playerButton in self.radioPlayerButtons) {
        [playerButton setSelected:playerButton.tag == tag];
    }
}

- (void)onRadioPumpingTypeButtonClicked:(NSInteger)tag {
    self.pumpingType = (int)tag;
    for (TLRadioButton *pumpingTypeButton in self.radioPumpingTypeButtons) {
        [pumpingTypeButton setSelected:pumpingTypeButton.tag == tag];
    }
}

- (void)onRadioPumpingRatioButtonClicked:(NSInteger)tag {
    self.pumpingRatio = (int)tag;
    for (TLRadioButton *pumpingRatioButton in self.radioPumpingRatioButtons) {
        [pumpingRatioButton setSelected:pumpingRatioButton.tag == tag];
    }
}

- (void)onRadioAudioTypeButtonClicked:(NSInteger)tag {
    self.soundCustomPlay = (int)tag;
    for (TLRadioButton *audioTypeButton in self.radioAudioTypeButtons) {
        [audioTypeButton setSelected:audioTypeButton.tag == tag];
    }
}


- (void)removeRadioAllviews {
    for (TLRadioButton *radioSpinButton in self.radioSpinButtons) {
        [radioSpinButton removeFromSuperview];
    }
    for (TLRadioButton *radioPlayerButton in self.radioPlayerButtons) {
        [radioPlayerButton removeFromSuperview];
    }
    [self.radioSpinArray removeAllObjects];
    [self.radioPlayerArray removeAllObjects];
    [self.radioSpinButtons removeAllObjects];
    [self.radioPlayerButtons removeAllObjects];
}

#pragma mark publick method
/// 显示
+ (instancetype)show:(NSArray<ZegoGameInfo *> *)gameList isShowGround:(BOOL)isShowGround type:(ZegoSelectShowType)type selectedGameBlock:(nonnull selectedGameBlock)selectedGameBlock playGameBlock:(nonnull ZegoPlayGameBlock)playGameBlock {
    ZegoSelectGamePicker *picker = [ZegoSelectGamePicker makeInstance];
    picker.gameList = gameList;
    picker.isShowGround = isShowGround;
    picker.selectedGameBlock = selectedGameBlock;
    picker.playGameBlock = playGameBlock;
    picker.type = type;
    // 当前顶部控制器
    UIViewController *topVC = [UIApplication sharedApplication].delegate.window.rootViewController;
    [topVC presentViewController:picker animated:NO completion:^{
        [picker show];
    }];
    return picker;
}

- (void)updateGameParams:(ZegoGameInfoDetail *)infoDetail {
    [self.radioSpinArray removeAllObjects];
    int i = 0;
    for (id coin in infoDetail.coin) {
        RadioModel *spinModel = [[RadioModel alloc] init];
        spinModel.num = [coin intValue];
        spinModel.tag = [coin integerValue];
        
        [self.radioSpinArray addObject:spinModel];
        if (i == 0) {
            spinModel.isSelected = YES;
            self.spin = spinModel.tag;
        } else {
            spinModel.isSelected = NO;
        }
        i++;
    }
    
    [self.radioPlayerArray removeAllObjects];
    int j = 0;
    for (id player in infoDetail.playerNum) {
        RadioModel *playerModel = [[RadioModel alloc] init];
        playerModel.num = [player intValue];
        playerModel.tag = [player integerValue];
        playerModel.isSelected = NO;
        [self.radioPlayerArray addObject:playerModel];
        if (j == 0) {
            playerModel.isSelected = YES;
            self.playerNum = (int)playerModel.tag;
        } else {
            playerModel.isSelected = NO;
        }
        j++;
    }
    
    [self createRadioSpinButtons];
    [self layoutRadioSpinButtons];
    [self createRadioPlayerButtons];
    [self layoutRadioPlayerButtons];
}

- (void)createRadioSpinButtons {
    [self.radioSpinButtons removeAllObjects];
    for (RadioModel *spinModel in self.radioSpinArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", spinModel.num] isSelected:spinModel.isSelected];
        button.tag = spinModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioSpinButtonClicked:tag];
        };
        [self.radioSpinButtons addObject:button];
        [self.goldCoinsView addSubview:button];
    }
}

- (void)createRadioPlayerButtons {
    [self.radioPlayerButtons removeAllObjects];
    for (RadioModel *playerModel in self.radioPlayerArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", playerModel.num] isSelected:playerModel.isSelected];
        button.tag = playerModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioPlayerButtonClicked:tag];
        };
        [self.radioPlayerButtons addObject:button];
        [self.playerNumView addSubview:button];
    }
}

- (void)createPumpingTypePlayerButtons {
    [self.radioPumpingTypeButtons removeAllObjects];
    for (RadioModel *typeModel in self.radioPumpingTypeArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:typeModel.num == 1 ? TLLocalizedString(tax_type_1): TLLocalizedString(tax_type_2) isSelected:typeModel.isSelected];
        button.tag = typeModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioPumpingTypeButtonClicked:tag];
        };
        [self.radioPumpingTypeButtons addObject:button];
        [self.pumpingTypeView addSubview:button];
    }
}

- (void)createPumpingRatioPlayerButtons {
    [self.radioPumpingRatioButtons removeAllObjects];
    for (RadioModel *ratioModel in self.radioPumpingRatioArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", ratioModel.num] isSelected:ratioModel.isSelected];
        button.tag = ratioModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioPumpingRatioButtonClicked:tag];
        };
        [self.radioPumpingRatioButtons addObject:button];
        [self.pumpingRatioView addSubview:button];
    }
}

- (void)createAudioTypePlayerButtons {
    [self.radioAudioTypeButtons removeAllObjects];
    for (RadioModel *audioModel in self.radioAudioTypeArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:audioModel.num == 0 ? TLLocalizedString(play_sound_control_web_view): TLLocalizedString(play_sound_control_native) isSelected:audioModel.isSelected];
        button.tag = audioModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioAudioTypeButtonClicked:tag];
        };
        [self.radioAudioTypeButtons addObject:button];
        [self.audioView addSubview:button];
    }
}

- (void)layoutRadioSpinButtons {
    CGFloat spinWidth = (self.goldCoinsView.width - 40) / self.radioSpinButtons.count;
    int i = 0;
    for (TLRadioButton *radioSpinButton in self.radioSpinButtons) {
        [radioSpinButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(spinWidth);
            make.left.mas_equalTo(i * spinWidth + 20);
        }];
        i++;
        if (i == self.radioSpinButtons.count) {
            [self.goldCoinsLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                make.bottom.equalTo(self.goldCoinsView.mas_bottom).offset(-48);
            }];
        }
    }
}

- (void)layoutRadioPlayerButtons {
    CGFloat playerWidth = (self.playerNumView.width - 40) / self.radioPlayerButtons.count;
    int i = 0;
    for (TLRadioButton *radioPlayerButton in self.radioPlayerButtons) {
        [radioPlayerButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(playerWidth);
            make.left.mas_equalTo(i * playerWidth + 20);
            make.bottom.equalTo(self.playerNumView.mas_bottom).offset(-14);
        }];
        i++;
        if (i == self.radioPlayerButtons.count) {
            [self.playerNumLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                make.bottom.equalTo(self.playerNumView.mas_bottom).offset(-48);
            }];
        }
    }
}

- (void)layoutRadioPumpingTypeButtons {
    CGFloat pumpingTypeWidth = (kScreenWidth - 48) / self.radioPumpingTypeButtons.count;
    int i = 0;
    for (TLRadioButton *radioPumpingTypeButton in self.radioPumpingTypeButtons) {
        [radioPumpingTypeButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(pumpingTypeWidth);
            make.left.mas_equalTo(i * pumpingTypeWidth + 20);
            make.bottom.equalTo(self.pumpingTypeView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutRadioPumpingRatioButtons {
    CGFloat radioPumpingRatioWidth = (kScreenWidth - 48) / self.radioPumpingRatioButtons.count;
    int i = 0;
    for (TLRadioButton *radioPumpingRatioButton in self.radioPumpingRatioButtons) {
        [radioPumpingRatioButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(radioPumpingRatioWidth);
            make.left.mas_equalTo(i * radioPumpingRatioWidth + 20);
            make.bottom.equalTo(self.pumpingRatioView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutRadioAudioTypeButtons {
    CGFloat audioTypeWidth = (kScreenWidth - 48) / self.radioAudioTypeButtons.count;
    int i = 0;
    for (TLRadioButton *radioAudioTypeButton in self.radioAudioTypeButtons) {
        [radioAudioTypeButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(audioTypeWidth);
            make.left.mas_equalTo(i * audioTypeWidth + 20);
            make.bottom.equalTo(self.audioView.mas_bottom).offset(-14);
        }];
        i++;
//        if (i == self.radioAudioTypeButtons.count) {
//            [self.joinRoomBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
//                make.top.equalTo(radioAudioTypeButton.mas_bottom).offset(20);
//                make.centerX.equalTo(self);
//                make.width.mas_equalTo(100);
//                make.height.mas_equalTo(40);
//            }];
//        }
    }
}

///创建控制器实例工厂方法
+ (instancetype)makeInstance {
    ZegoSelectGamePicker *picker = [[ZegoSelectGamePicker alloc] init];
    picker.view.backgroundColor = [UIColor colorWithWhite:0 alpha:0.5];
    picker.view.alpha = 0;
    picker.modalPresentationStyle = UIModalPresentationOverFullScreen;
    return picker;
}

- (UIView *)contentView {
    if (!_contentView) {
        _contentView = [[UIView alloc] init];
        _contentView.backgroundColor = [UIColor colorWithRGB:0xFF111014 alpha:0.9];
        _contentView.userInteractionEnabled = true;
        
        // 标题
        UILabel *titleLab = [[UILabel alloc] init];
        titleLab.text = TLLocalizedString(select_game_info);
        titleLab.font = BOLD_FONT(16);
        titleLab.textColor = [UIColor whiteColor];
        titleLab.textAlignment = NSTextAlignmentLeft;
        [_contentView addSubview:titleLab];
        [titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(_contentView).offset(16);
        }];
        
        // 标题
        UIView *gameView = [[UIView alloc] init];
        gameView.layer.cornerRadius = 9;
        gameView.layer.masksToBounds = YES;
        gameView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:gameView];
        self.gameView = gameView;
        [gameView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(titleLab.mas_bottom).offset(16);
            make.left.equalTo(_contentView).offset(12);
            make.right.equalTo(_contentView).offset(-12);
        }];
        
        UILabel *gameLabel = [[UILabel alloc] init];
        gameLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        gameLabel.font = BOLD_FONT(13);
        gameLabel.textAlignment = NSTextAlignmentLeft;
        gameLabel.text = TLLocalizedString(chose_game);
        [gameView addSubview:gameLabel];
        [gameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(gameView).offset(10);
            make.left.equalTo(gameView).offset(14);
            make.height.mas_equalTo(18);
        }];
        

        UICollectionViewFlowLayout* layout = [[UICollectionViewFlowLayout alloc]init];
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 8;
        layout.minimumInteritemSpacing = 5;
        layout.sectionInset = UIEdgeInsetsMake(11, 16, 14, 16);
        layout.itemSize = CGSizeMake((kScreenWidth - 71) / 4,  (kScreenWidth - 71) / 4 + 29);
        UICollectionView *collectionView = [[UICollectionView alloc] initWithFrame:gameView.frame collectionViewLayout:layout];
        collectionView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        collectionView.delegate = self;
        collectionView.dataSource = self;
        [collectionView registerClass:[GameCollectionViewCell class] forCellWithReuseIdentifier:@"GameCollectionViewCell"];
        [gameView addSubview:collectionView];
        [collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(gameLabel.mas_bottom).offset(10);
            make.left.width.equalTo(gameView);
            make.height.mas_equalTo((kScreenWidth - 71) / 4 + 54);
            make.bottom.equalTo(gameView.mas_bottom);
        }];
        
        
        UIView *goldCoinsView = [[UIView alloc] init];
        goldCoinsView.layer.cornerRadius = 9;
        goldCoinsView.layer.masksToBounds = YES;
        goldCoinsView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:goldCoinsView];
        self.goldCoinsView = goldCoinsView;
        [goldCoinsView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(gameView.mas_bottom).offset(12);
            make.left.right.equalTo(gameView);
        }];
        
        UILabel *goldCoinsLabel = [[UILabel alloc] init];
        goldCoinsLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        goldCoinsLabel.font = BOLD_FONT(13);
        goldCoinsLabel.textAlignment = NSTextAlignmentLeft;
        goldCoinsLabel.text = TLLocalizedString(minor_game_coin);
        [goldCoinsView addSubview:goldCoinsLabel];
        self.goldCoinsLabel = goldCoinsLabel;
        [goldCoinsLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(goldCoinsView).offset(14);
            make.left.equalTo(goldCoinsView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(goldCoinsView.mas_bottom).offset(-14);
        }];
        
        UIView *playerNumView = [[UIView alloc] init];
        playerNumView.layer.cornerRadius = 9;
        playerNumView.layer.masksToBounds = YES;
        playerNumView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:playerNumView];
        self.playerNumView = playerNumView;
        
        [playerNumView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(goldCoinsView.mas_bottom).offset(12);
            make.left.right.equalTo(goldCoinsView);
        }];
        
        UILabel *playerNumLabel = [[UILabel alloc] init];
        playerNumLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        playerNumLabel.font = BOLD_FONT(13);
        playerNumLabel.textAlignment = NSTextAlignmentLeft;
        playerNumLabel.text = TLLocalizedString(player_number);
        [playerNumView addSubview:playerNumLabel];
        self.playerNumLabel = playerNumLabel;
        [playerNumLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(playerNumView).offset(14);
            make.left.equalTo(playerNumView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(playerNumView.mas_bottom).offset(-14);
        }];
        
        UIView *pumpingTypeView = [[UIView alloc] init];
        pumpingTypeView.layer.cornerRadius = 9;
        pumpingTypeView.layer.masksToBounds = YES;
        pumpingTypeView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:pumpingTypeView];
        self.pumpingTypeView = pumpingTypeView;
        [pumpingTypeView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(playerNumView.mas_bottom).offset(12);
            make.left.right.equalTo(playerNumView);
        }];
        
        UILabel *pumpingTypeLabel = [[UILabel alloc] init];
        pumpingTypeLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        pumpingTypeLabel.font = BOLD_FONT(13);
        pumpingTypeLabel.textAlignment = NSTextAlignmentLeft;
        pumpingTypeLabel.text = TLLocalizedString(tax_type);
        [pumpingTypeView addSubview:pumpingTypeLabel];
        self.pumpingTypeLabel = pumpingTypeLabel;
        [pumpingTypeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(pumpingTypeView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(pumpingTypeView.mas_bottom).offset(-48);
        }];
        
        UIView *pumpingRatioView = [[UIView alloc] init];
        pumpingRatioView.layer.cornerRadius = 9;
        pumpingRatioView.layer.masksToBounds = YES;
        pumpingRatioView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:pumpingRatioView];
        self.pumpingRatioView = pumpingRatioView;
        [pumpingRatioView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(pumpingTypeView.mas_bottom).offset(12);
            make.left.right.equalTo(pumpingTypeView);
        }];
        
        UILabel *pumpingRatioLabel = [[UILabel alloc] init];
        pumpingRatioLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        pumpingRatioLabel.font = BOLD_FONT(13);
        pumpingRatioLabel.textAlignment = NSTextAlignmentLeft;
        pumpingRatioLabel.text = TLLocalizedString(tax_rate);
        [pumpingRatioView addSubview:pumpingRatioLabel];
        self.pumpingRatioLabel = pumpingRatioLabel;
        [pumpingRatioLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(pumpingRatioView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(pumpingRatioView.mas_bottom).offset(-48);
        }];
        
        UIView *audioView = [[UIView alloc] init];
        audioView.layer.cornerRadius = 9;
        audioView.layer.masksToBounds = YES;
        audioView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:audioView];
        self.audioView = audioView;
        [audioView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(pumpingRatioView.mas_bottom).offset(12);
            make.left.right.equalTo(pumpingRatioView);
        }];
        
        UILabel *audioLabel = [[UILabel alloc] init];
        audioLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        audioLabel.font = BOLD_FONT(13);
        audioLabel.textAlignment = NSTextAlignmentLeft;
        audioLabel.text = TLLocalizedString(play_sound_control);
        [audioView addSubview:audioLabel];
        self.audioLabel = audioLabel;
        [audioLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(audioView).offset(14);
            make.left.equalTo(audioView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(audioView.mas_bottom).offset(-48);
        }];
        
        UIButton *playGameBtn = [[UIButton alloc] init];
        playGameBtn.layer.cornerRadius = 18;
        playGameBtn.layer.masksToBounds = YES;
        playGameBtn.titleLabel.font = BOLD_FONT(14);
        [playGameBtn setBackgroundColor:[UIColor colorWithHexString:@"6D24F6"]];
        [playGameBtn setTitle:TLLocalizedString(start_game) forState:UIControlStateNormal];
        [playGameBtn addTarget:self action:@selector(onClickPlayGameAction:) forControlEvents:UIControlEventTouchUpInside];
        [_contentView addSubview:playGameBtn];
        self.playGameBtn = playGameBtn;
        [playGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(audioView.mas_bottom).offset(30);
            make.centerX.equalTo(_contentView);
            make.height.mas_equalTo(36);
            make.width.mas_equalTo(124);
//            make.bottom.equalTo(_contentView.mas_bottom).offset(- kTLBottomStatusBarHeight - 6);
        }];
    }
    return _contentView;
}

- (NSMutableArray *)radioSpinButtons {
    if (!_radioSpinButtons) {
        _radioSpinButtons = [NSMutableArray array];
    }
    return _radioSpinButtons;
}

- (NSMutableArray *)radioSpinArray {
    if (!_radioSpinArray) {
        _radioSpinArray = [NSMutableArray array];
    }
    return _radioSpinArray;
}

- (NSMutableArray *)radioPlayerButtons {
    if (!_radioPlayerButtons) {
        _radioPlayerButtons = [NSMutableArray array];
    }
    return _radioPlayerButtons;
}


- (NSMutableArray *)radioPlayerArray {
    if (!_radioPlayerArray) {
        _radioPlayerArray = [NSMutableArray array];
    }
    return _radioPlayerArray;
}

- (NSMutableArray *)radioAudioTypeArray {
    if (!_radioAudioTypeArray) {
        _radioAudioTypeArray = [NSMutableArray array];
    }
    return _radioAudioTypeArray;
}

- (NSMutableArray *)radioAudioTypeButtons {
    if (!_radioAudioTypeButtons) {
        _radioAudioTypeButtons = [NSMutableArray array];
    }
    return _radioAudioTypeButtons;
}

- (NSMutableArray *)radioPumpingTypeButtons {
    if (!_radioPumpingTypeButtons) {
        _radioPumpingTypeButtons = [NSMutableArray array];
    }
    return _radioPumpingTypeButtons;
}

- (NSMutableArray *)radioPumpingTypeArray {
    if (!_radioPumpingTypeArray) {
        _radioPumpingTypeArray = [NSMutableArray array];
    }
    return _radioPumpingTypeArray;
}

- (NSMutableArray *)radioPumpingRatioButtons {
    if (!_radioPumpingRatioButtons) {
        _radioPumpingRatioButtons = [NSMutableArray array];
    }
    return _radioPumpingRatioButtons;
}

- (NSMutableArray *)radioPumpingRatioArray {
    if (!_radioPumpingRatioArray) {
        _radioPumpingRatioArray = [NSMutableArray array];
    }
    return _radioPumpingRatioArray;
}
@end

