//
//  ZegoSelectPlayerController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/6/13.
//

#import "ZegoSelectPlayerController.h"
#import "ZegoPlayGameAlterView.h"
#import "ZegoSelectMemberView.h"

@implementation ZegoSelectPlayerModel


@end

@interface ZegoSelectPlayerController ()

/// 容器视图
@property (nonatomic, strong) UIView *contentView;
// 用户列表view
@property (nonatomic, strong) ZegoSelectMemberView *playerListView;
// 每局赌注金币view
@property (nonatomic, strong) UIView *goldCoinsView;
// 玩家人数view
@property (nonatomic, strong) UIView *playerNumView;
// 抽水类型view
@property (nonatomic, strong) UIView *pumpingTypeView;
// 抽水比例view
@property (nonatomic, strong) UIView *pumpingRatioView;
// 机器人view
@property (nonatomic, strong) UIView *robotView;
// 开始游戏按钮
@property (nonatomic, strong) UIButton *playGameBtn;
// 右上角关闭图标
@property (nonatomic, strong) UIImageView *closeImageView;

// 每局赌注金币label
@property (nonatomic, strong) UILabel *goldCoinsLabel;
// 玩家人数label
@property (nonatomic, strong) UILabel *playerNumLabel;
// 抽水类型label
@property (nonatomic, strong) UILabel *pumpingTypeLabel;
// 抽水比例label
@property (nonatomic, strong) UILabel *pumpingRatioLabel;
// 若开始时人数不足，是否匹配机器人label
@property (nonatomic, strong) UILabel *robotLabel;

// 玩家列表（demo没有用户列表服务，则用RTC的流列表，demo 语聊房进房就推流）
@property (nonatomic, strong) NSArray <StreamInfo *> *playerList;
// 传过来的玩家人数列表
@property (nonatomic, copy) NSArray *playerNumList;
// 传过来的每局赌注金币列表
@property (nonatomic, copy) NSArray *spinList;

// 玩家人数数据源
@property (nonatomic, strong) NSMutableArray *playerNumArray;
// 每局赌注金币数据源
@property (nonatomic, strong) NSMutableArray *radioSpinArray;
// 机器人选择数据源
@property (nonatomic, strong) NSMutableArray *robotTypeArray;
// 抽水类型数据源
@property (nonatomic, strong) NSMutableArray *radioPumpingTypeArray;
// 抽水比列数据源
@property (nonatomic, strong) NSMutableArray *radioPumpingRatioArray;

// 玩家人数选择按钮集合
@property (nonatomic, strong) NSMutableArray *playerNumButtons;
// 每局赌注金币数按钮集合
@property (nonatomic, strong) NSMutableArray *radioSpinButtons;
// 是否匹配机器人按钮集合
@property (nonatomic, strong) NSMutableArray *robotTypeButtons;
// 抽水类型按钮集合
@property (nonatomic, strong) NSMutableArray *radioPumpingTypeButtons;
// 抽水比例按钮集合
@property (nonatomic, strong) NSMutableArray *radioPumpingRatioButtons;

// 开始游戏block
@property (nonatomic, copy) ZegoStartPlayGameBlock playGameBlock;

@property (nonatomic, assign) int playerNum;
@property (nonatomic, copy) NSArray <StreamInfo *>*selectedPlayers;
@property (nonatomic, assign) NSInteger spin;
@property (nonatomic, assign) int pumpingType;
@property (nonatomic, assign) int pumpingRatio;
@property (nonatomic, assign) BOOL isUseRobot;

@end

@implementation ZegoSelectPlayerController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor colorWithWhite:0 alpha:0.5];
    
    [self setupUI];
}

- (void)dealloc {
    NSLog(@"ZegoSelectPlayerController dealloc~~~");
}

#pragma mark private method
/// 设置UI
- (void)setupUI {
    
    UIScrollView *scrollview = [[UIScrollView alloc] init];
    scrollview.backgroundColor = UIColor.clearColor;
    [self.view addSubview:scrollview];
    [scrollview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.view);
    }];
    
    [scrollview addSubview:self.contentView];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(scrollview);

        make.width.equalTo(scrollview);
    }];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.playGameBtn.mas_bottom).offset(kTLBottomStatusBarHeight +6);
    }];
    
    //玩家人数
    [self createPlayerNumButtons];
    [self layoutPlayerNumButtons];
    
    //每局赌注金币
    [self createRadioSpinButtons];
    [self layoutRadioSpinButtons];
    
    //抽水类型
    [self createPumpingTypePlayerButtons];
    [self layoutRadioPumpingTypeButtons];
    
    //抽水比列
    [self createPumpingRatioPlayerButtons];
    [self layoutRadioPumpingRatioButtons];
    
    //是否匹配机器人
    [self createRobotTypeButtons];
    [self layoutRobotTypeButtons];
}

#pragma mark event method
// 显示
- (void)show {
    // 当前顶部控制器
    UIViewController *topVC = [UIApplication sharedApplication].delegate.window.rootViewController;
    [topVC presentViewController:self animated:NO completion:^{}];
}

// 关闭控制器
- (void)closePickerView:(UIGestureRecognizer *)sender {
    [self dismissViewControllerAnimated:NO completion:nil];
}

// 退出
- (void)dismiss:(void (^)(void))completion {
    [self removeRadioAllviews];
    [self.view endEditing:YES];
    [UIView animateWithDuration:0.2 animations:^{
        [self.view layoutIfNeeded];
    } completion:^(BOOL finished) {
        [UIView animateWithDuration:0.2 animations:^{
            self.view.alpha = 0;
        } completion:^(BOOL finished) {
            [self dismissViewControllerAnimated:NO completion:completion];
        }];
    }];
}
    
#pragma mark action
- (void)onClickPlayGameAction:(UIButton *)sender {

    NSArray *selectedPlayers = [self.playerListView getSelectedPlayer];
    ZegoSelectPlayerModel *model = [[ZegoSelectPlayerModel alloc] init];
    
    if (self.playerNum == 0) {
        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"请选择玩家人数"] withFinishBlock:nil];
        return;
    }
    
    if(self.playerNum > selectedPlayers.count && !self.isUseRobot) {
        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"玩家人数不足"] withFinishBlock:nil];
        return;
    }
    
    if(self.playerNum < selectedPlayers.count ) {
        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"玩家人数过多"] withFinishBlock:nil];
        return;
    }
    
    if (self.spin == 0) {
//        [MBProgressHUD showSuccess:[NSString stringWithFormat:@"请选择游戏的最小金额数"] withFinishBlock:nil];
//        return;
    }
    
    model.spin = self.spin;
    model.playerNum = self.playerNum;
    model.pumpingType = self.pumpingType;
    model.pumpingRatio = self.pumpingRatio;
    model.isUseRobot = self.isUseRobot;
    model.selectedPlayers = selectedPlayers;
   
    if (self.playGameBlock) {
        self.playGameBlock(model);
    }
    [self dismiss:nil];
}

//玩家人数按钮点击事件
- (void)onPlayerNumButtonClicked:(NSInteger)tag {
    self.playerNum = (int)tag;
    for (TLRadioButton *spinButton in self.playerNumButtons) {
        [spinButton setSelected:spinButton.tag == tag];
    }
}

//每局赌注金币按钮点击事件
- (void)onRadioSpinButtonClicked:(NSInteger)tag {
    self.spin = tag;
    for (TLRadioButton *spinButton in self.radioSpinButtons) {
        [spinButton setSelected:spinButton.tag == tag];
    }
}

//游戏抽水类型按钮点击事件
- (void)onRadioPumpingTypeButtonClicked:(NSInteger)tag {
    self.pumpingType = (int)tag;
    for (TLRadioButton *pumpingTypeButton in self.radioPumpingTypeButtons) {
        [pumpingTypeButton setSelected:pumpingTypeButton.tag == tag];
    }
}

//游戏抽水比例按钮点击事件
- (void)onRadioPumpingRatioButtonClicked:(NSInteger)tag {
    self.pumpingRatio = (int)tag;
    for (TLRadioButton *pumpingRatioButton in self.radioPumpingRatioButtons) {
        [pumpingRatioButton setSelected:pumpingRatioButton.tag == tag];
    }
}

//匹配机器人按钮点击事件
- (void)onRobotTypeButtonClicked:(NSInteger)tag {
    self.isUseRobot = (int)tag;
    for (TLRadioButton *audioTypeButton in self.robotTypeButtons) {
        [audioTypeButton setSelected:audioTypeButton.tag == tag];
    }
}


- (void)removeRadioAllviews {
    for (TLRadioButton *radioSpinButton in self.radioSpinButtons) {
        [radioSpinButton removeFromSuperview];
    }
    
    [self.radioSpinArray removeAllObjects];
    [self.radioSpinButtons removeAllObjects];
}

#pragma mark publick method
- (instancetype)initShow:(NSArray <StreamInfo *> *)playerList playerNumList:(NSArray *)playerNumList spinList:(NSArray *)spinList playGameBlock:(ZegoStartPlayGameBlock)playGameBlock {
    self = [super init];
    self.playerList = playerList;
    self.playerNumList = playerNumList;
    self.spinList = spinList;
    self.playGameBlock = playGameBlock;
    
    return self;
}

- (void)createPlayerNumButtons {
    [self.playerNumArray removeAllObjects];
    int p = 0;
    for (id audioType in self.playerNumList) {
        RadioModel *audioModel = [[RadioModel alloc] init];
        audioModel.num = [audioType intValue];
        audioModel.tag = [audioType integerValue];
        audioModel.isSelected = [audioType intValue] == 1 ? YES : NO;
        [self.playerNumArray addObject:audioModel];
        if (p == 0) {
            audioModel.isSelected = YES;
            self.playerNum = (int)audioModel.tag;
        } else {
            audioModel.isSelected = NO;
        }
        p++;
    }
    
    [self.playerNumButtons removeAllObjects];
    for (RadioModel *spinModel in self.playerNumArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", spinModel.num] isSelected:spinModel.isSelected];
        button.tag = spinModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onPlayerNumButtonClicked:tag];
        };
        [self.playerNumButtons addObject:button];
        [self.playerNumView addSubview:button];
    }
}

- (void)createRadioSpinButtons {
    [self.radioSpinArray removeAllObjects];
    int i = 0;
    for (id coin in self.spinList) {
        RadioModel *spinModel = [[RadioModel alloc] init];
        spinModel.num = [coin intValue];
        spinModel.tag = [coin integerValue];
        
        [self.radioSpinArray addObject:spinModel];
        if (i == 0) {
            spinModel.isSelected = YES;
            self.spin = spinModel.tag;
        } else {
            spinModel.isSelected = NO;
        }
        i++;
    }
    
    [self.radioSpinButtons removeAllObjects];
    for (RadioModel *spinModel in self.radioSpinArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", spinModel.num] isSelected:spinModel.isSelected];
        button.tag = spinModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioSpinButtonClicked:tag];
        };
        [self.radioSpinButtons addObject:button];
        [self.goldCoinsView addSubview:button];
    }
}

- (void)createPumpingTypePlayerButtons {
    [self.radioPumpingTypeArray removeAllObjects];
    int j = 0;
    for (id pumpingType in @[@(1), @(2)]) {
        RadioModel *pumpingModel = [[RadioModel alloc] init];
        pumpingModel.num = [pumpingType intValue];
        pumpingModel.tag = [pumpingType integerValue];
        pumpingModel.isSelected = [pumpingType intValue] == 1 ? YES : NO;
        [self.radioPumpingTypeArray addObject:pumpingModel];
        if (j == 0) {
            pumpingModel.isSelected = YES;
            self.pumpingType = (int)pumpingModel.tag;
        } else {
            pumpingModel.isSelected = NO;
        }
        j++;
    }
    
    [self.radioPumpingTypeButtons removeAllObjects];
    for (RadioModel *typeModel in self.radioPumpingTypeArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:typeModel.num == 1 ? TLLocalizedString(tax_type_1): TLLocalizedString(tax_type_2) isSelected:typeModel.isSelected];
        button.tag = typeModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioPumpingTypeButtonClicked:tag];
        };
        [self.radioPumpingTypeButtons addObject:button];
        [self.pumpingTypeView addSubview:button];
    }
}

- (void)createPumpingRatioPlayerButtons {
    [self.radioPumpingRatioArray removeAllObjects];
    int k = 0;
    for (id pumpingRatio in @[@(0), @(1), @(2)]) {
        RadioModel *pumpingRatioModel = [[RadioModel alloc] init];
        pumpingRatioModel.num = [pumpingRatio intValue];
        pumpingRatioModel.tag = [pumpingRatio integerValue];
        pumpingRatioModel.isSelected = [pumpingRatio intValue] == 0 ? YES : NO;
        [self.radioPumpingRatioArray addObject:pumpingRatioModel];
        if (k == 0) {
            pumpingRatioModel.isSelected = YES;
            self.pumpingRatio = (int)pumpingRatioModel.tag;
        } else {
            pumpingRatioModel.isSelected = NO;
        }
        k++;
    }
    
    [self.radioPumpingRatioButtons removeAllObjects];
    for (RadioModel *ratioModel in self.radioPumpingRatioArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:[NSString stringWithFormat:@"%d", ratioModel.num] isSelected:ratioModel.isSelected];
        button.tag = ratioModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRadioPumpingRatioButtonClicked:tag];
        };
        [self.radioPumpingRatioButtons addObject:button];
        [self.pumpingRatioView addSubview:button];
    }
}

- (void)createRobotTypeButtons {
    [self.robotTypeArray removeAllObjects];
    int i = 0;
    for (id audioType in @[@(0), @(1)]) {
        RadioModel *audioModel = [[RadioModel alloc] init];
        audioModel.num = [audioType intValue];
        audioModel.tag = [audioType integerValue];
        audioModel.isSelected = [audioType intValue] == 1 ? YES : NO;
        [self.robotTypeArray addObject:audioModel];
        if (i == 0) {
            audioModel.isSelected = YES;
            self.isUseRobot = audioModel.tag == 1 ? YES : NO;
        } else {
            audioModel.isSelected = NO;
        }
        i++;
    }
    
    for (RadioModel *audioModel in self.robotTypeArray) {
        TLRadioButton *button = [UIButton radionButtonWithTitle:audioModel.num == 0 ? TLLocalizedString(play_use_robot_no): TLLocalizedString(play_use_robot_yes) isSelected:audioModel.isSelected];
        button.tag = audioModel.tag;
        button.actionBlock = ^(NSInteger tag){
            [self onRobotTypeButtonClicked:tag];
        };
        [self.robotTypeButtons addObject:button];
        [self.robotView addSubview:button];
    }
}

- (void)layoutRadioSpinButtons {
    
    CGFloat spinWidth = (kScreenWidth - 48) / self.radioSpinButtons.count;
    int i = 0;
    for (TLRadioButton *radioSpinButton in self.radioSpinButtons) {
        [radioSpinButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(spinWidth);
            make.left.mas_equalTo(i * spinWidth + 20);
            make.bottom.equalTo(self.goldCoinsView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutRadioPumpingTypeButtons {
    CGFloat pumpingTypeWidth = (kScreenWidth - 48) / self.radioPumpingTypeButtons.count;
    int i = 0;
    for (TLRadioButton *radioPumpingTypeButton in self.radioPumpingTypeButtons) {
        [radioPumpingTypeButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(pumpingTypeWidth);
            make.left.mas_equalTo(i * pumpingTypeWidth + 20);
            make.bottom.equalTo(self.pumpingTypeView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutRadioPumpingRatioButtons {
    CGFloat radioPumpingRatioWidth = (kScreenWidth - 48) / self.radioPumpingRatioButtons.count;
    int i = 0;
    for (TLRadioButton *radioPumpingRatioButton in self.radioPumpingRatioButtons) {
        [radioPumpingRatioButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(radioPumpingRatioWidth);
            make.left.mas_equalTo(i * radioPumpingRatioWidth + 20);
            make.bottom.equalTo(self.pumpingRatioView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutPlayerNumButtons {
    CGFloat playerNumWidth = (kScreenWidth - 48) / self.playerNumButtons.count;
    int i = 0;
    for (TLRadioButton *button in self.playerNumButtons) {
        [button mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(20);
            make.width.mas_equalTo(playerNumWidth);
            make.left.mas_equalTo(i * playerNumWidth + 20);
            make.bottom.equalTo(self.playerNumView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (void)layoutRobotTypeButtons {
    CGFloat audioTypeWidth = (kScreenWidth - 48) / self.robotTypeButtons.count;
    int i = 0;
    for (TLRadioButton *radioAudioTypeButton in self.robotTypeButtons) {
        [radioAudioTypeButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(46);
            make.height.mas_equalTo(40);
            make.width.mas_equalTo(audioTypeWidth);
            make.left.mas_equalTo(i * audioTypeWidth + 20);
            make.bottom.equalTo(self.robotView.mas_bottom).offset(-14);
        }];
        i++;
    }
}

- (UIView *)contentView {
    if (!_contentView) {
        _contentView = [[UIView alloc] init];
        _contentView.backgroundColor = [UIColor colorWithRGB:0xFF111014 alpha:0.9];
        _contentView.userInteractionEnabled = true;
        
        _closeImageView = [[UIImageView alloc] init];
        _closeImageView.userInteractionEnabled = YES;
        _closeImageView.image = [UIImage imageNamed:@"icon_close"];
        _closeImageView.contentMode = UIViewContentModeCenter;
        _closeImageView.backgroundColor = [UIColor colorWithRGB:0x00000000 alpha:0.5];
        _closeImageView.layer.cornerRadius = 13;
        _closeImageView.layer.masksToBounds = YES;
        UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closePickerView:)];
        [_closeImageView addGestureRecognizer:tapGR];
        [_contentView addSubview:_closeImageView];
        [_closeImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(_contentView).offset(16);
            make.right.mas_equalTo(_contentView.mas_right).offset(-16);
            make.width.height.mas_equalTo(32);
        }];
        
        // 标题
        UILabel *titleLab = [[UILabel alloc] init];
        titleLab.text = TLLocalizedString(chose_game_player);
        titleLab.font = BOLD_FONT(16);
        titleLab.textColor = [UIColor whiteColor];
        titleLab.textAlignment = NSTextAlignmentLeft;
        [_contentView addSubview:titleLab];
        [titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(_contentView).offset(16);
        }];
        
        UIView *playerNumView = [[UIView alloc] init];
        playerNumView.layer.cornerRadius = 9;
        playerNumView.layer.masksToBounds = YES;
        playerNumView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:playerNumView];
        self.playerNumView = playerNumView;
        [playerNumView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(titleLab.mas_bottom).offset(12);
            make.left.equalTo(_contentView).offset(12);
            make.right.equalTo(_contentView).offset(-12);
        }];
        
        UILabel *playerNumLabel = [[UILabel alloc] init];
        playerNumLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        playerNumLabel.font = BOLD_FONT(13);
        playerNumLabel.textAlignment = NSTextAlignmentLeft;
        playerNumLabel.text = TLLocalizedString(player_number);
        [playerNumView addSubview:playerNumLabel];
        self.playerNumLabel = playerNumLabel;
        [playerNumLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(playerNumView).offset(14);
            make.left.equalTo(playerNumView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(playerNumView.mas_bottom).offset(-48);
        }];
        
        // 成员列表
        CGFloat h = self.playerList.count * 40 + 20;
        ZegoSelectMemberView *playListView = [[ZegoSelectMemberView alloc] initWithFrame:CGRectZero];
        playListView.layer.cornerRadius = 9;
        playListView.layer.masksToBounds = YES;
        playListView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:playListView];
        [playListView updateMembers:self.playerList];
        self.playerListView = playListView;
        [playListView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.mas_equalTo(playerNumView.mas_bottom).offset(12);
            make.left.right.equalTo(playerNumView);
            make.height.mas_equalTo(h);
        }];
        
        UIView *goldCoinsView = [[UIView alloc] init];
        goldCoinsView.layer.cornerRadius = 9;
        goldCoinsView.layer.masksToBounds = YES;
        goldCoinsView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:goldCoinsView];
        self.goldCoinsView = goldCoinsView;
        [goldCoinsView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(playListView.mas_bottom).offset(12);
            make.left.right.equalTo(playListView);
        }];
        
        UILabel *goldCoinsLabel = [[UILabel alloc] init];
        goldCoinsLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        goldCoinsLabel.font = BOLD_FONT(13);
        goldCoinsLabel.textAlignment = NSTextAlignmentLeft;
        goldCoinsLabel.text = TLLocalizedString(minor_game_coin);
        [goldCoinsView addSubview:goldCoinsLabel];
        self.goldCoinsLabel = goldCoinsLabel;
        [goldCoinsLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(goldCoinsView).offset(14);
            make.left.equalTo(goldCoinsView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(goldCoinsView.mas_bottom).offset(-48);
        }];
        
        //抽水类型
        UIView *pumpingTypeView = [[UIView alloc] init];
        pumpingTypeView.layer.cornerRadius = 9;
        pumpingTypeView.layer.masksToBounds = YES;
        pumpingTypeView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:pumpingTypeView];
        self.pumpingTypeView = pumpingTypeView;
        [pumpingTypeView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(goldCoinsView.mas_bottom).offset(12);
            make.left.right.equalTo(goldCoinsView);
        }];
        
        UILabel *pumpingTypeLabel = [[UILabel alloc] init];
        pumpingTypeLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        pumpingTypeLabel.font = BOLD_FONT(13);
        pumpingTypeLabel.textAlignment = NSTextAlignmentLeft;
        pumpingTypeLabel.text = TLLocalizedString(tax_type);
        [pumpingTypeView addSubview:pumpingTypeLabel];
        self.pumpingTypeLabel = pumpingTypeLabel;
        [pumpingTypeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(pumpingTypeView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(pumpingTypeView.mas_bottom).offset(-48);
        }];
        
        //抽水比例
        UIView *pumpingRatioView = [[UIView alloc] init];
        pumpingRatioView.layer.cornerRadius = 9;
        pumpingRatioView.layer.masksToBounds = YES;
        pumpingRatioView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:pumpingRatioView];
        self.pumpingRatioView = pumpingRatioView;
        [pumpingRatioView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(pumpingTypeView.mas_bottom).offset(12);
            make.left.right.equalTo(pumpingTypeView);
        }];
        
        UILabel *pumpingRatioLabel = [[UILabel alloc] init];
        pumpingRatioLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        pumpingRatioLabel.font = BOLD_FONT(13);
        pumpingRatioLabel.textAlignment = NSTextAlignmentLeft;
        pumpingRatioLabel.text = TLLocalizedString(tax_rate);
        [pumpingRatioView addSubview:pumpingRatioLabel];
        self.pumpingRatioLabel = pumpingRatioLabel;
        [pumpingRatioLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.equalTo(pumpingRatioView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(pumpingRatioView.mas_bottom).offset(-48);
        }];
        
        //是否机器人匹配
        UIView *robotView = [[UIView alloc] init];
        robotView.layer.cornerRadius = 9;
        robotView.layer.masksToBounds = YES;
        robotView.backgroundColor = [UIColor colorWithHexString:@"574979"];
        [_contentView addSubview:robotView];
        self.robotView = robotView;
        [robotView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(pumpingRatioView.mas_bottom).offset(12);
            make.left.right.equalTo(pumpingRatioView);
        }];
        
        UILabel *robotLabel = [[UILabel alloc] init];
        robotLabel.textColor = [UIColor colorWithRGB:0xffffffff alpha:0.6];
        robotLabel.font = BOLD_FONT(13);
        robotLabel.textAlignment = NSTextAlignmentLeft;
        robotLabel.text = TLLocalizedString(play_use_robot);
        [robotView addSubview:robotLabel];
        self.robotLabel = robotLabel;
        [robotLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(robotView).offset(14);
            make.left.equalTo(robotView).offset(14);
            make.height.mas_equalTo(18);
            make.bottom.equalTo(robotView.mas_bottom).offset(-48);
        }];
        
        UIButton *playGameBtn = [[UIButton alloc] init];
        playGameBtn.layer.cornerRadius = 18;
        playGameBtn.layer.masksToBounds = YES;
        playGameBtn.titleLabel.font = BOLD_FONT(14);
        [playGameBtn setBackgroundColor:[UIColor colorWithHexString:@"6D24F6"]];
        [playGameBtn setTitle:TLLocalizedString(start_game) forState:UIControlStateNormal];
        [playGameBtn addTarget:self action:@selector(onClickPlayGameAction:) forControlEvents:UIControlEventTouchUpInside];
        [_contentView addSubview:playGameBtn];
        self.playGameBtn = playGameBtn;
        [playGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(robotView.mas_bottom).offset(30);
            make.centerX.equalTo(_contentView);
            make.height.mas_equalTo(36);
            make.width.mas_equalTo(124);
//            make.bottom.equalTo(_contentView.mas_bottom).offset(- kTLBottomStatusBarHeight - 6);
        }];
    }
    return _contentView;
}


- (NSMutableArray *)playerNumArray {
    if(!_playerNumArray) {
        _playerNumArray = [NSMutableArray array];
    }
    return _playerNumArray;
}

- (NSMutableArray *)playerNumButtons {
    if(!_playerNumButtons) {
        _playerNumButtons = [NSMutableArray array];
    }
    return _playerNumButtons;
}

- (NSMutableArray *)radioSpinButtons {
    if (!_radioSpinButtons) {
        _radioSpinButtons = [NSMutableArray array];
    }
    return _radioSpinButtons;
}

- (NSMutableArray *)radioSpinArray {
    if (!_radioSpinArray) {
        _radioSpinArray = [NSMutableArray array];
    }
    return _radioSpinArray;
}

- (NSMutableArray *)robotTypeArray {
    if (!_robotTypeArray) {
        _robotTypeArray = [NSMutableArray array];
    }
    return _robotTypeArray;
}

- (NSMutableArray *)robotTypeButtons {
    if (!_robotTypeButtons) {
        _robotTypeButtons = [NSMutableArray array];
    }
    return _robotTypeButtons;
}

- (NSMutableArray *)radioPumpingTypeButtons {
    if (!_radioPumpingTypeButtons) {
        _radioPumpingTypeButtons = [NSMutableArray array];
    }
    return _radioPumpingTypeButtons;
}

- (NSMutableArray *)radioPumpingTypeArray {
    if (!_radioPumpingTypeArray) {
        _radioPumpingTypeArray = [NSMutableArray array];
    }
    return _radioPumpingTypeArray;
}

- (NSMutableArray *)radioPumpingRatioButtons {
    if (!_radioPumpingRatioButtons) {
        _radioPumpingRatioButtons = [NSMutableArray array];
    }
    return _radioPumpingRatioButtons;
}

- (NSMutableArray *)radioPumpingRatioArray {
    if (!_radioPumpingRatioArray) {
        _radioPumpingRatioArray = [NSMutableArray array];
    }
    return _radioPumpingRatioArray;
}

@end
